/* Generated by Cython 0.29.2 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_2"
#define CYTHON_HEX_VERSION 0x001D02F0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_USE_DICT_VERSIONS
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
        static PY_UINT64_T __pyx_dict_version = 0;\
        static PyObject *__pyx_dict_cached_value = NULL;\
        if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
            (VAR) = __pyx_dict_cached_value;\
        } else {\
            (VAR) = __pyx_dict_cached_value = (LOOKUP);\
            __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
        }\
    }
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0; // PyThread_create_key reports success always
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif // TSS (Thread Specific Storage) API
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
  #define PyObject_Unicode             PyObject_Str
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : (Py_INCREF(func), func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__bzenet
#define __PYX_HAVE_API__bzenet
/* Early includes */
#include <string.h>
#include <stdio.h>
#include "pythread.h"
#include <stdint.h>
#include "enet/types.h"
#include "enet/enet.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "enet.pyx",
  "stringsource",
  "type.pxd",
  "bool.pxd",
  "complex.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_6bzenet_Socket;
struct __pyx_obj_6bzenet_Address;
struct __pyx_obj_6bzenet_Packet;
struct __pyx_obj_6bzenet_Peer;
struct __pyx_obj_6bzenet_Event;
struct __pyx_obj_6bzenet_Host;

/* "enet.pyx":181
 *     int enet_socket_send(ENetSocket socket, ENetAddress *address, ENetBuffer *buffer, size_t size)
 * 
 * cdef enum:             # <<<<<<<<<<<<<<
 *     MAXHOSTNAME = 257
 * 
 */
enum  {
  __pyx_e_6bzenet_MAXHOSTNAME = 0x101
};

/* "enet.pyx":207
 * cdef class Address
 * 
 * cdef class Socket:             # <<<<<<<<<<<<<<
 *     """
 *     Socket (int socket)
 */
struct __pyx_obj_6bzenet_Socket {
  PyObject_HEAD
  ENetSocket _enet_socket;
};


/* "enet.pyx":205
 * PEER_STATE_ZOMBIE = ENET_PEER_STATE_ZOMBIE
 * 
 * cdef class Address             # <<<<<<<<<<<<<<
 * 
 * cdef class Socket:
 */
struct __pyx_obj_6bzenet_Address {
  PyObject_HEAD
  ENetAddress _enet_address;
};


/* "enet.pyx":311
 *             self._enet_address.port = value
 * 
 * cdef class Packet:             # <<<<<<<<<<<<<<
 *     """
 *     Packet (str dataContents, int flags)
 */
struct __pyx_obj_6bzenet_Packet {
  PyObject_HEAD
  ENetPacket *_enet_packet;
  PyBoolObject *sent;
};


/* "enet.pyx":390
 *             self.sent = value
 * 
 * cdef class Peer:             # <<<<<<<<<<<<<<
 *     """
 *     Peer ()
 */
struct __pyx_obj_6bzenet_Peer {
  PyObject_HEAD
  ENetPeer *_enet_peer;
};


/* "enet.pyx":795
 *                 return self._enet_peer.eventData
 * 
 * cdef class Event:             # <<<<<<<<<<<<<<
 *     """
 *     Event ()
 */
struct __pyx_obj_6bzenet_Event {
  PyObject_HEAD
  ENetEvent _enet_event;
  struct __pyx_obj_6bzenet_Packet *_packet;
};


/* "enet.pyx":847
 * cdef host_static_instances = WeakValueDictionary()
 * 
 * cdef class Host:             # <<<<<<<<<<<<<<
 *     """
 *     Host (Address address, int peerCount, int channelLimit,
 */
struct __pyx_obj_6bzenet_Host {
  PyObject_HEAD
  ENetHost *_enet_host;
  PyBoolObject *dealloc;
  PyObject *_interceptCallback;
  PyObject *__weakref__;
};


/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_ENetSocket(ENetSocket value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_ENetPacketFlag(ENetPacketFlag value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_ENetEventType(ENetEventType value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_ENetPeerState(ENetPeerState value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enet_uint16(enet_uint16 value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enet_uint32(enet_uint32 value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enet_uint8(enet_uint8 value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE ENetSocket __Pyx_PyInt_As_ENetSocket(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned char __Pyx_PyInt_As_unsigned_char(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE enet_uint16 __Pyx_PyInt_As_enet_uint16(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE enet_uint32 __Pyx_PyInt_As_enet_uint32(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE enet_uint8 __Pyx_PyInt_As_enet_uint8(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'cpython.version' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.exc' */

/* Module declarations from 'cpython.module' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'cpython.tuple' */

/* Module declarations from 'cpython.list' */

/* Module declarations from 'cpython.sequence' */

/* Module declarations from 'cpython.mapping' */

/* Module declarations from 'cpython.iterator' */

/* Module declarations from 'cpython.number' */

/* Module declarations from 'cpython.int' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.bool' */
static PyTypeObject *__pyx_ptype_7cpython_4bool_bool = 0;

/* Module declarations from 'cpython.long' */

/* Module declarations from 'cpython.float' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.complex' */
static PyTypeObject *__pyx_ptype_7cpython_7complex_complex = 0;

/* Module declarations from 'cpython.string' */

/* Module declarations from 'cpython.unicode' */

/* Module declarations from 'cpython.dict' */

/* Module declarations from 'cpython.instance' */

/* Module declarations from 'cpython.function' */

/* Module declarations from 'cpython.method' */

/* Module declarations from 'cpython.weakref' */

/* Module declarations from 'cpython.getargs' */

/* Module declarations from 'cpython.pythread' */

/* Module declarations from 'cpython.pystate' */

/* Module declarations from 'cpython.cobject' */

/* Module declarations from 'cpython.oldbuffer' */

/* Module declarations from 'cpython.set' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'cpython.bytes' */

/* Module declarations from 'cpython.pycapsule' */

/* Module declarations from 'cpython' */

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'bzenet' */
static PyTypeObject *__pyx_ptype_6bzenet_Address = 0;
static PyTypeObject *__pyx_ptype_6bzenet_Socket = 0;
static PyTypeObject *__pyx_ptype_6bzenet_Packet = 0;
static PyTypeObject *__pyx_ptype_6bzenet_Peer = 0;
static PyTypeObject *__pyx_ptype_6bzenet_Event = 0;
static PyTypeObject *__pyx_ptype_6bzenet_Host = 0;
static PyObject *__pyx_v_6bzenet_host_static_instances = 0;
static int __cdecl __pyx_f_6bzenet_intercept_callback(ENetHost *, ENetEvent *); /*proto*/
static PyObject *__pyx_f_6bzenet___pyx_unpickle_Socket__set_state(struct __pyx_obj_6bzenet_Socket *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "bzenet"
extern int __pyx_module_is_main_bzenet;
int __pyx_module_is_main_bzenet = 0;

/* Implementation of 'bzenet' */
static PyObject *__pyx_builtin_NotImplementedError;
static PyObject *__pyx_builtin_IOError;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_MemoryError;
static const char __pyx_k_[] = "*";
static const char __pyx_k_0_1[] = "{0}:{1}";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_Host[] = "Host";
static const char __pyx_k_Peer[] = "Peer";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_host[] = "host";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_port[] = "port";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_time[] = "time";
static const char __pyx_k_Event[] = "Event";
static const char __pyx_k_ascii[] = "ascii";
static const char __pyx_k_flags[] = "flags";
static const char __pyx_k_Packet[] = "Packet";
static const char __pyx_k_Socket[] = "Socket";
static const char __pyx_k_atexit[] = "atexit";
static const char __pyx_k_bzenet[] = "bzenet";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_packet[] = "packet";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_Address[] = "Address";
static const char __pyx_k_IOError[] = "IOError";
static const char __pyx_k_address[] = "address";
static const char __pyx_k_timeout[] = "timeout";
static const char __pyx_k_weakref[] = "weakref";
static const char __pyx_k_enet_pyx[] = "enet.pyx";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_is_valid[] = "is_valid";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_register[] = "register";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_time_max[] = "time_max";
static const char __pyx_k_time_min[] = "time_min";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_channelID[] = "channelID";
static const char __pyx_k_fast_drop[] = "fast_drop";
static const char __pyx_k_intercept[] = "intercept";
static const char __pyx_k_peerCount[] = "peerCount";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_check_valid[] = "check_valid";
static const char __pyx_k_enet_atexit[] = "_enet_atexit";
static const char __pyx_k_channelCount[] = "channelCount";
static const char __pyx_k_channelLimit[] = "channelLimit";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_EVENT_TYPE_NONE[] = "EVENT_TYPE_NONE";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_PEER_STATE_ZOMBIE[] = "PEER_STATE_ZOMBIE";
static const char __pyx_k_incomingBandwidth[] = "incomingBandwidth";
static const char __pyx_k_outgoingBandwidth[] = "outgoingBandwidth";
static const char __pyx_k_Connection_failure[] = "Connection failure!";
static const char __pyx_k_EVENT_TYPE_CONNECT[] = "EVENT_TYPE_CONNECT";
static const char __pyx_k_EVENT_TYPE_RECEIVE[] = "EVENT_TYPE_RECEIVE";
static const char __pyx_k_Resolution_failure[] = "Resolution failure!";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_NotImplementedError[] = "NotImplementedError";
static const char __pyx_k_WeakValueDictionary[] = "WeakValueDictionary";
static const char __pyx_k_pyx_unpickle_Socket[] = "__pyx_unpickle_Socket";
static const char __pyx_k_PACKET_FLAG_RELIABLE[] = "PACKET_FLAG_RELIABLE";
static const char __pyx_k_PEER_STATE_CONNECTED[] = "PEER_STATE_CONNECTED";
static const char __pyx_k_EVENT_TYPE_DISCONNECT[] = "EVENT_TYPE_DISCONNECT";
static const char __pyx_k_PEER_STATE_CONNECTING[] = "PEER_STATE_CONNECTING";
static const char __pyx_k_packetThrottleInterval[] = "packetThrottleInterval";
static const char __pyx_k_PACKET_FLAG_NO_ALLOCATE[] = "PACKET_FLAG_NO_ALLOCATE";
static const char __pyx_k_PACKET_FLAG_UNSEQUENCED[] = "PACKET_FLAG_UNSEQUENCED";
static const char __pyx_k_PEER_STATE_DISCONNECTED[] = "PEER_STATE_DISCONNECTED";
static const char __pyx_k_PEER_STATE_DISCONNECTING[] = "PEER_STATE_DISCONNECTING";
static const char __pyx_k_Empty_Peer_object_accessed[] = "Empty Peer object accessed!";
static const char __pyx_k_packetThrottleAcceleration[] = "packetThrottleAcceleration";
static const char __pyx_k_packetThrottleDeceleration[] = "packetThrottleDeceleration";
static const char __pyx_k_PEER_STATE_DISCONNECT_LATER[] = "PEER_STATE_DISCONNECT_LATER";
static const char __pyx_k_PEER_STATE_CONNECTION_PENDING[] = "PEER_STATE_CONNECTION_PENDING";
static const char __pyx_k_PACKET_FLAG_UNRELIABLE_FRAGMENT[] = "PACKET_FLAG_UNRELIABLE_FRAGMENT";
static const char __pyx_k_PEER_STATE_CONNECTION_SUCCEEDED[] = "PEER_STATE_CONNECTION_SUCCEEDED";
static const char __pyx_k_Packet_has_not_been_initiliazed[] = "Packet has not been initiliazed properly!";
static const char __pyx_k_Pickling_of_struct_members_such[] = "Pickling of struct members such as self._enet_address must be explicitly requested with @auto_pickle(True)";
static const char __pyx_k_Unable_to_create_host_structure[] = "Unable to create host structure!";
static const char __pyx_k_Incompatible_checksums_s_vs_0xd7[] = "Incompatible checksums (%s vs 0xd719157 = (_enet_socket))";
static const char __pyx_k_PEER_STATE_ACKNOWLEDGING_CONNECT[] = "PEER_STATE_ACKNOWLEDGING_CONNECT";
static const char __pyx_k_PEER_STATE_ACKNOWLEDGING_DISCONN[] = "PEER_STATE_ACKNOWLEDGING_DISCONNECT";
static const char __pyx_k_Servicing_error_probably_disconn[] = "Servicing error - probably disconnected.";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static const char __pyx_k_self__enet_event_cannot_be_conve[] = "self._enet_event cannot be converted to a Python object for pickling";
static const char __pyx_k_self__enet_packet_cannot_be_conv[] = "self._enet_packet cannot be converted to a Python object for pickling";
static const char __pyx_k_self__enet_peer_cannot_be_conver[] = "self._enet_peer cannot be converted to a Python object for pickling";
static PyObject *__pyx_kp_s_;
static PyObject *__pyx_kp_s_0_1;
static PyObject *__pyx_n_s_Address;
static PyObject *__pyx_kp_s_Connection_failure;
static PyObject *__pyx_n_s_EVENT_TYPE_CONNECT;
static PyObject *__pyx_n_s_EVENT_TYPE_DISCONNECT;
static PyObject *__pyx_n_s_EVENT_TYPE_NONE;
static PyObject *__pyx_n_s_EVENT_TYPE_RECEIVE;
static PyObject *__pyx_kp_s_Empty_Peer_object_accessed;
static PyObject *__pyx_n_s_Event;
static PyObject *__pyx_n_s_Host;
static PyObject *__pyx_n_s_IOError;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xd7;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_n_s_NotImplementedError;
static PyObject *__pyx_n_s_PACKET_FLAG_NO_ALLOCATE;
static PyObject *__pyx_n_s_PACKET_FLAG_RELIABLE;
static PyObject *__pyx_n_s_PACKET_FLAG_UNRELIABLE_FRAGMENT;
static PyObject *__pyx_n_s_PACKET_FLAG_UNSEQUENCED;
static PyObject *__pyx_n_s_PEER_STATE_ACKNOWLEDGING_CONNECT;
static PyObject *__pyx_n_s_PEER_STATE_ACKNOWLEDGING_DISCONN;
static PyObject *__pyx_n_s_PEER_STATE_CONNECTED;
static PyObject *__pyx_n_s_PEER_STATE_CONNECTING;
static PyObject *__pyx_n_s_PEER_STATE_CONNECTION_PENDING;
static PyObject *__pyx_n_s_PEER_STATE_CONNECTION_SUCCEEDED;
static PyObject *__pyx_n_s_PEER_STATE_DISCONNECTED;
static PyObject *__pyx_n_s_PEER_STATE_DISCONNECTING;
static PyObject *__pyx_n_s_PEER_STATE_DISCONNECT_LATER;
static PyObject *__pyx_n_s_PEER_STATE_ZOMBIE;
static PyObject *__pyx_n_s_Packet;
static PyObject *__pyx_kp_s_Packet_has_not_been_initiliazed;
static PyObject *__pyx_n_s_Peer;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_kp_s_Pickling_of_struct_members_such;
static PyObject *__pyx_kp_s_Resolution_failure;
static PyObject *__pyx_kp_s_Servicing_error_probably_disconn;
static PyObject *__pyx_n_s_Socket;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_s_Unable_to_create_host_structure;
static PyObject *__pyx_n_s_WeakValueDictionary;
static PyObject *__pyx_n_s_address;
static PyObject *__pyx_n_s_ascii;
static PyObject *__pyx_n_s_atexit;
static PyObject *__pyx_n_s_bzenet;
static PyObject *__pyx_n_s_channelCount;
static PyObject *__pyx_n_s_channelID;
static PyObject *__pyx_n_s_channelLimit;
static PyObject *__pyx_n_s_check_valid;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_enet_atexit;
static PyObject *__pyx_kp_s_enet_pyx;
static PyObject *__pyx_n_s_fast_drop;
static PyObject *__pyx_n_s_flags;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_host;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_incomingBandwidth;
static PyObject *__pyx_n_s_intercept;
static PyObject *__pyx_n_s_is_valid;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
static PyObject *__pyx_n_s_outgoingBandwidth;
static PyObject *__pyx_n_s_packet;
static PyObject *__pyx_n_s_packetThrottleAcceleration;
static PyObject *__pyx_n_s_packetThrottleDeceleration;
static PyObject *__pyx_n_s_packetThrottleInterval;
static PyObject *__pyx_n_s_peerCount;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_port;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_Socket;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_register;
static PyObject *__pyx_kp_s_self__enet_event_cannot_be_conve;
static PyObject *__pyx_kp_s_self__enet_packet_cannot_be_conv;
static PyObject *__pyx_kp_s_self__enet_peer_cannot_be_conver;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_time;
static PyObject *__pyx_n_s_time_max;
static PyObject *__pyx_n_s_time_min;
static PyObject *__pyx_n_s_timeout;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_weakref;
static PyObject *__pyx_pf_6bzenet_6Socket_send(struct __pyx_obj_6bzenet_Socket *__pyx_v_self, struct __pyx_obj_6bzenet_Address *__pyx_v_address, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6bzenet_6Socket_2fileno(struct __pyx_obj_6bzenet_Socket *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_6Socket_4__reduce_cython__(struct __pyx_obj_6bzenet_Socket *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_6Socket_6__setstate_cython__(struct __pyx_obj_6bzenet_Socket *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6bzenet_7Address___init__(struct __pyx_obj_6bzenet_Address *__pyx_v_self, PyObject *__pyx_v_host, PyObject *__pyx_v_port); /* proto */
static PyObject *__pyx_pf_6bzenet_7Address_2__str__(struct __pyx_obj_6bzenet_Address *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_7Address_4__richcmp__(struct __pyx_obj_6bzenet_Address *__pyx_v_self, PyObject *__pyx_v_obj, PyObject *__pyx_v_op); /* proto */
static PyObject *__pyx_pf_6bzenet_7Address_4host___get__(struct __pyx_obj_6bzenet_Address *__pyx_v_self); /* proto */
static int __pyx_pf_6bzenet_7Address_4host_2__set__(struct __pyx_obj_6bzenet_Address *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6bzenet_7Address_8hostname___get__(struct __pyx_obj_6bzenet_Address *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_7Address_4port___get__(struct __pyx_obj_6bzenet_Address *__pyx_v_self); /* proto */
static int __pyx_pf_6bzenet_7Address_4port_2__set__(struct __pyx_obj_6bzenet_Address *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6bzenet_7Address_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Address *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_7Address_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Address *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6bzenet_6Packet___init__(struct __pyx_obj_6bzenet_Packet *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_flags); /* proto */
static void __pyx_pf_6bzenet_6Packet_2__dealloc__(struct __pyx_obj_6bzenet_Packet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_6Packet_4is_valid(struct __pyx_obj_6bzenet_Packet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_6Packet_4data___get__(struct __pyx_obj_6bzenet_Packet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_6Packet_10dataLength___get__(struct __pyx_obj_6bzenet_Packet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_6Packet_5flags___get__(struct __pyx_obj_6bzenet_Packet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_6Packet_4sent___get__(struct __pyx_obj_6bzenet_Packet *__pyx_v_self); /* proto */
static int __pyx_pf_6bzenet_6Packet_4sent_2__set__(struct __pyx_obj_6bzenet_Packet *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6bzenet_6Packet_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Packet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_6Packet_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Packet *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer___richcmp__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_obj, PyObject *__pyx_v_op); /* proto */
static Py_hash_t __pyx_pf_6bzenet_4Peer_2__hash__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_4send(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_channelID, struct __pyx_obj_6bzenet_Packet *__pyx_v_packet); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_6receive(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, unsigned char __pyx_v_channelID); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_8reset(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_10timeout(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_time, PyObject *__pyx_v_time_min, PyObject *__pyx_v_time_max); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_12ping(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_14disconnect(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_16disconnect_later(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_18disconnect_now(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_20check_valid(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_4host___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_14outgoingPeerID___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_14incomingPeerID___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_9connectID___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_17outgoingSessionID___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_17incomingSessionID___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_7address___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_4data___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static int __pyx_pf_6bzenet_4Peer_4data_2__set__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_5state___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_12channelCount___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_17incomingBandwidth___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_17outgoingBandwidth___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_30incomingBandwidthThrottleEpoch___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_30outgoingBandwidthThrottleEpoch___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_17incomingDataTotal___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_17outgoingDataTotal___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_12lastSendTime___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_15lastReceiveTime___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_11nextTimeout___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_15earliestTimeout___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_15packetLossEpoch___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_11packetsSent___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_11packetsLost___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_10packetLoss___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_18packetLossVariance___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_14packetThrottle___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_19packetThrottleLimit___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_21packetThrottleCounter___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_19packetThrottleEpoch___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_26packetThrottleAcceleration___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static int __pyx_pf_6bzenet_4Peer_26packetThrottleAcceleration_2__set__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_26packetThrottleDeceleration___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static int __pyx_pf_6bzenet_4Peer_26packetThrottleDeceleration_2__set__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_22packetThrottleInterval___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static int __pyx_pf_6bzenet_4Peer_22packetThrottleInterval_2__set__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_17lastRoundTripTime___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_19lowestRoundTripTime___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_25lastRoundTripTimeVariance___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_28highestRoundTripTimeVariance___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_13roundTripTime___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_21roundTripTimeVariance___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_3mtu___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_10windowSize___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_21reliableDataInTransit___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_30outgoingReliableSequenceNumber___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_13needsDispatch___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_24incomingUnsequencedGroup___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_24outgoingUnsequencedGroup___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_9eventData___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_22__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Peer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Peer_24__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Peer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6bzenet_5Event___init__(struct __pyx_obj_6bzenet_Event *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_5Event_4type___get__(struct __pyx_obj_6bzenet_Event *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_5Event_4peer___get__(struct __pyx_obj_6bzenet_Event *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_5Event_9channelID___get__(struct __pyx_obj_6bzenet_Event *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_5Event_4data___get__(struct __pyx_obj_6bzenet_Event *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_5Event_6packet___get__(struct __pyx_obj_6bzenet_Event *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_5Event_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Event *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_5Event_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Event *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6bzenet_4Host___init__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, struct __pyx_obj_6bzenet_Address *__pyx_v_address, PyObject *__pyx_v_peerCount, PyObject *__pyx_v_channelLimit, PyObject *__pyx_v_incomingBandwidth, PyObject *__pyx_v_outgoingBandwidth); /* proto */
static Py_hash_t __pyx_pf_6bzenet_4Host_2__hash__(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static int __pyx_pf_6bzenet_4Host_4__cinit__(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static void __pyx_pf_6bzenet_4Host_6__dealloc__(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_8connect(struct __pyx_obj_6bzenet_Host *__pyx_v_self, struct __pyx_obj_6bzenet_Address *__pyx_v_address, PyObject *__pyx_v_channelCount, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_10check_events(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_12service(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_timeout, PyObject *__pyx_v_fast_drop); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_14flush(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_16broadcast(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_channelID, struct __pyx_obj_6bzenet_Packet *__pyx_v_packet); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_18compress_with_range_coder(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_6socket___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_7address___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_17incomingBandwidth___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static int __pyx_pf_6bzenet_4Host_17incomingBandwidth_2__set__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_17outgoingBandwidth___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static int __pyx_pf_6bzenet_4Host_17outgoingBandwidth_2__set__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_5peers___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_9peerCount___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_12channelLimit___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static int __pyx_pf_6bzenet_4Host_12channelLimit_2__set__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_13totalSentData___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static int __pyx_pf_6bzenet_4Host_13totalSentData_2__set__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_16totalSentPackets___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static int __pyx_pf_6bzenet_4Host_16totalSentPackets_2__set__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_17totalReceivedData___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static int __pyx_pf_6bzenet_4Host_17totalReceivedData_2__set__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_20totalReceivedPackets___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static int __pyx_pf_6bzenet_4Host_20totalReceivedPackets_2__set__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_9intercept___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static int __pyx_pf_6bzenet_4Host_9intercept_2__set__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_20__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Host *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzenet_4Host_22__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Host *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6bzenet__enet_atexit(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_6bzenet_2__pyx_unpickle_Socket(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_6bzenet_Address(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6bzenet_Socket(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6bzenet_Packet(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6bzenet_Peer(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6bzenet_Event(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6bzenet_Host(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_3;
static PyObject *__pyx_int_225546583;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_codeobj__19;
static PyObject *__pyx_codeobj__21;
/* Late includes */

/* "enet.pyx":220
 *     cdef ENetSocket _enet_socket
 * 
 *     def send(self, Address address, data):             # <<<<<<<<<<<<<<
 *         cdef ENetBuffer buffer
 *         data = data if isinstance(data, bytes) else data.encode()
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_6Socket_1send(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6bzenet_6Socket_1send(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_6bzenet_Address *__pyx_v_address = 0;
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("send (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_address,&__pyx_n_s_data,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_address)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("send", 1, 2, 2, 1); __PYX_ERR(0, 220, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "send") < 0)) __PYX_ERR(0, 220, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_address = ((struct __pyx_obj_6bzenet_Address *)values[0]);
    __pyx_v_data = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("send", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 220, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzenet.Socket.send", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_address), __pyx_ptype_6bzenet_Address, 1, "address", 0))) __PYX_ERR(0, 220, __pyx_L1_error)
  __pyx_r = __pyx_pf_6bzenet_6Socket_send(((struct __pyx_obj_6bzenet_Socket *)__pyx_v_self), __pyx_v_address, __pyx_v_data);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_6Socket_send(struct __pyx_obj_6bzenet_Socket *__pyx_v_self, struct __pyx_obj_6bzenet_Address *__pyx_v_address, PyObject *__pyx_v_data) {
  ENetBuffer __pyx_v_buffer;
  int __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  char *__pyx_t_6;
  Py_ssize_t __pyx_t_7;
  __Pyx_RefNannySetupContext("send", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "enet.pyx":222
 *     def send(self, Address address, data):
 *         cdef ENetBuffer buffer
 *         data = data if isinstance(data, bytes) else data.encode()             # <<<<<<<<<<<<<<
 * 
 *         buffer.data = <char*>data
 */
  __pyx_t_2 = PyBytes_Check(__pyx_v_data); 
  if ((__pyx_t_2 != 0)) {
    __Pyx_INCREF(__pyx_v_data);
    __pyx_t_1 = __pyx_v_data;
  } else {
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_encode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 222, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 222, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "enet.pyx":224
 *         data = data if isinstance(data, bytes) else data.encode()
 * 
 *         buffer.data = <char*>data             # <<<<<<<<<<<<<<
 *         buffer.dataLength = len(data)
 * 
 */
  __pyx_t_6 = __Pyx_PyObject_AsWritableString(__pyx_v_data); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 224, __pyx_L1_error)
  __pyx_v_buffer.data = ((char *)__pyx_t_6);

  /* "enet.pyx":225
 * 
 *         buffer.data = <char*>data
 *         buffer.dataLength = len(data)             # <<<<<<<<<<<<<<
 * 
 *         cdef int result = enet_socket_send(self._enet_socket,
 */
  __pyx_t_7 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 225, __pyx_L1_error)
  __pyx_v_buffer.dataLength = __pyx_t_7;

  /* "enet.pyx":227
 *         buffer.dataLength = len(data)
 * 
 *         cdef int result = enet_socket_send(self._enet_socket,             # <<<<<<<<<<<<<<
 *             &address._enet_address, &buffer, 1)
 *         return result
 */
  __pyx_v_result = enet_socket_send(__pyx_v_self->_enet_socket, (&__pyx_v_address->_enet_address), (&__pyx_v_buffer), 1);

  /* "enet.pyx":229
 *         cdef int result = enet_socket_send(self._enet_socket,
 *             &address._enet_address, &buffer, 1)
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     def fileno(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "enet.pyx":220
 *     cdef ENetSocket _enet_socket
 * 
 *     def send(self, Address address, data):             # <<<<<<<<<<<<<<
 *         cdef ENetBuffer buffer
 *         data = data if isinstance(data, bytes) else data.encode()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("bzenet.Socket.send", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":231
 *         return result
 * 
 *     def fileno(self):             # <<<<<<<<<<<<<<
 *         return self._enet_socket
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_6Socket_3fileno(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6bzenet_6Socket_3fileno(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fileno (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_6Socket_2fileno(((struct __pyx_obj_6bzenet_Socket *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_6Socket_2fileno(struct __pyx_obj_6bzenet_Socket *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("fileno", 0);

  /* "enet.pyx":232
 * 
 *     def fileno(self):
 *         return self._enet_socket             # <<<<<<<<<<<<<<
 * 
 * cdef class Address:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_ENetSocket(__pyx_v_self->_enet_socket); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "enet.pyx":231
 *         return result
 * 
 *     def fileno(self):             # <<<<<<<<<<<<<<
 *         return self._enet_socket
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Socket.fileno", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_6Socket_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6bzenet_6Socket_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_6Socket_4__reduce_cython__(((struct __pyx_obj_6bzenet_Socket *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_6Socket_4__reduce_cython__(struct __pyx_obj_6bzenet_Socket *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._enet_socket,)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetSocket(__pyx_v_self->_enet_socket); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._enet_socket,)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_2 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__dict = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":7
 *     state = (self._enet_socket,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v__dict);
    __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = False
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._enet_socket,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = False             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Socket, (type(self), 0xd719157, None), state
 */
  /*else*/ {
    __pyx_v_use_setstate = 0;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Socket, (type(self), 0xd719157, None), state
 *     else:
 */
  __pyx_t_4 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_4) {

    /* "(tree fragment)":13
 *         use_setstate = False
 *     if use_setstate:
 *         return __pyx_unpickle_Socket, (type(self), 0xd719157, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Socket, (type(self), 0xd719157, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_Socket); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_225546583);
    __Pyx_GIVEREF(__pyx_int_225546583);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_int_225546583);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_2, 2, Py_None);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Socket, (type(self), 0xd719157, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Socket, (type(self), 0xd719157, None), state
 *     else:
 *         return __pyx_unpickle_Socket, (type(self), 0xd719157, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Socket__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_Socket); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_225546583);
    __Pyx_GIVEREF(__pyx_int_225546583);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_int_225546583);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
    __pyx_t_5 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("bzenet.Socket.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Socket, (type(self), 0xd719157, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Socket__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_6Socket_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6bzenet_6Socket_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_6Socket_6__setstate_cython__(((struct __pyx_obj_6bzenet_Socket *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_6Socket_6__setstate_cython__(struct __pyx_obj_6bzenet_Socket *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_Socket, (type(self), 0xd719157, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Socket__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6bzenet___pyx_unpickle_Socket__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Socket, (type(self), 0xd719157, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Socket__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Socket.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":253
 *     cdef ENetAddress _enet_address
 * 
 *     def __init__(self, host, port):             # <<<<<<<<<<<<<<
 *         if host is not None:
 *             # Convert the hostname to a byte string if needed
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_7Address_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6bzenet_7Address_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_host = 0;
  PyObject *__pyx_v_port = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_host,&__pyx_n_s_port,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_host)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_port)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 253, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 253, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_host = values[0];
    __pyx_v_port = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 253, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzenet.Address.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6bzenet_7Address___init__(((struct __pyx_obj_6bzenet_Address *)__pyx_v_self), __pyx_v_host, __pyx_v_port);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_7Address___init__(struct __pyx_obj_6bzenet_Address *__pyx_v_self, PyObject *__pyx_v_host, PyObject *__pyx_v_port) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "enet.pyx":254
 * 
 *     def __init__(self, host, port):
 *         if host is not None:             # <<<<<<<<<<<<<<
 *             # Convert the hostname to a byte string if needed
 *             self.host = host if isinstance(host, bytes) else bytes(host, "ascii")
 */
  __pyx_t_1 = (__pyx_v_host != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "enet.pyx":256
 *         if host is not None:
 *             # Convert the hostname to a byte string if needed
 *             self.host = host if isinstance(host, bytes) else bytes(host, "ascii")             # <<<<<<<<<<<<<<
 *         else:
 *             self.host = None
 */
    __pyx_t_2 = PyBytes_Check(__pyx_v_host); 
    if ((__pyx_t_2 != 0)) {
      __Pyx_INCREF(__pyx_v_host);
      __pyx_t_3 = __pyx_v_host;
    } else {
      __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 256, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_v_host);
      __Pyx_GIVEREF(__pyx_v_host);
      PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_host);
      __Pyx_INCREF(__pyx_n_s_ascii);
      __Pyx_GIVEREF(__pyx_n_s_ascii);
      PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_n_s_ascii);
      __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)(&PyBytes_Type)), __pyx_t_4, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 256, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_3 = __pyx_t_5;
      __pyx_t_5 = 0;
    }
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_host, __pyx_t_3) < 0) __PYX_ERR(0, 256, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "enet.pyx":254
 * 
 *     def __init__(self, host, port):
 *         if host is not None:             # <<<<<<<<<<<<<<
 *             # Convert the hostname to a byte string if needed
 *             self.host = host if isinstance(host, bytes) else bytes(host, "ascii")
 */
    goto __pyx_L3;
  }

  /* "enet.pyx":258
 *             self.host = host if isinstance(host, bytes) else bytes(host, "ascii")
 *         else:
 *             self.host = None             # <<<<<<<<<<<<<<
 *         self.port = port
 * 
 */
  /*else*/ {
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_host, Py_None) < 0) __PYX_ERR(0, 258, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "enet.pyx":259
 *         else:
 *             self.host = None
 *         self.port = port             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_port, __pyx_v_port) < 0) __PYX_ERR(0, 259, __pyx_L1_error)

  /* "enet.pyx":253
 *     cdef ENetAddress _enet_address
 * 
 *     def __init__(self, host, port):             # <<<<<<<<<<<<<<
 *         if host is not None:
 *             # Convert the hostname to a byte string if needed
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("bzenet.Address.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":261
 *         self.port = port
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "{0}:{1}".format(self.host, self.port)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_7Address_3__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_7Address_3__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_7Address_2__str__(((struct __pyx_obj_6bzenet_Address *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_7Address_2__str__(struct __pyx_obj_6bzenet_Address *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "enet.pyx":262
 * 
 *     def __str__(self):
 *         return "{0}:{1}".format(self.host, self.port)             # <<<<<<<<<<<<<<
 * 
 *     def __richcmp__(self, obj, op):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_host); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_port); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 262, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 262, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 262, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 262, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "enet.pyx":261
 *         self.port = port
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "{0}:{1}".format(self.host, self.port)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("bzenet.Address.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":264
 *         return "{0}:{1}".format(self.host, self.port)
 * 
 *     def __richcmp__(self, obj, op):             # <<<<<<<<<<<<<<
 *         if isinstance(obj, Address):
 *             if op == 2:
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_7Address_5__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_arg_op); /*proto*/
static PyObject *__pyx_pw_6bzenet_7Address_5__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_arg_op) {
  PyObject *__pyx_v_op = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__richcmp__ (wrapper)", 0);
  __pyx_v_op = __Pyx_PyInt_From_int(__pyx_arg_op); if (unlikely(!__pyx_v_op)) __PYX_ERR(0, 264, __pyx_L3_error)
  __Pyx_GOTREF(__pyx_v_op);
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzenet.Address.__richcmp__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6bzenet_7Address_4__richcmp__(((struct __pyx_obj_6bzenet_Address *)__pyx_v_self), ((PyObject *)__pyx_v_obj), ((PyObject *)__pyx_v_op));

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_op);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_7Address_4__richcmp__(struct __pyx_obj_6bzenet_Address *__pyx_v_self, PyObject *__pyx_v_obj, PyObject *__pyx_v_op) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("__richcmp__", 0);

  /* "enet.pyx":265
 * 
 *     def __richcmp__(self, obj, op):
 *         if isinstance(obj, Address):             # <<<<<<<<<<<<<<
 *             if op == 2:
 *                 # This is a '==' operation
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_obj, __pyx_ptype_6bzenet_Address); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "enet.pyx":266
 *     def __richcmp__(self, obj, op):
 *         if isinstance(obj, Address):
 *             if op == 2:             # <<<<<<<<<<<<<<
 *                 # This is a '==' operation
 *                 return (obj.host == self.host) and (obj.port == self.port)
 */
    __pyx_t_3 = __Pyx_PyInt_EqObjC(__pyx_v_op, __pyx_int_2, 2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 266, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 266, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_2) {

      /* "enet.pyx":268
 *             if op == 2:
 *                 # This is a '==' operation
 *                 return (obj.host == self.host) and (obj.port == self.port)             # <<<<<<<<<<<<<<
 *             elif op == 3:
 *                 # This is a '!=' operation
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_obj, __pyx_n_s_host); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 268, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_host); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 268, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = PyObject_RichCompare(__pyx_t_4, __pyx_t_5, Py_EQ); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 268, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 268, __pyx_L1_error)
      if (__pyx_t_2) {
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else {
        __Pyx_INCREF(__pyx_t_6);
        __pyx_t_3 = __pyx_t_6;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        goto __pyx_L5_bool_binop_done;
      }
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_obj, __pyx_n_s_port); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 268, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_port); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 268, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_6, __pyx_t_5, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 268, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_3 = __pyx_t_4;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_L5_bool_binop_done:;
      __pyx_r = __pyx_t_3;
      __pyx_t_3 = 0;
      goto __pyx_L0;

      /* "enet.pyx":266
 *     def __richcmp__(self, obj, op):
 *         if isinstance(obj, Address):
 *             if op == 2:             # <<<<<<<<<<<<<<
 *                 # This is a '==' operation
 *                 return (obj.host == self.host) and (obj.port == self.port)
 */
    }

    /* "enet.pyx":269
 *                 # This is a '==' operation
 *                 return (obj.host == self.host) and (obj.port == self.port)
 *             elif op == 3:             # <<<<<<<<<<<<<<
 *                 # This is a '!=' operation
 *                 return (obj.host != self.host) or (obj.port != self.port)
 */
    __pyx_t_3 = __Pyx_PyInt_EqObjC(__pyx_v_op, __pyx_int_3, 3, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 269, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 269, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_2) {

      /* "enet.pyx":271
 *             elif op == 3:
 *                 # This is a '!=' operation
 *                 return (obj.host != self.host) or (obj.port != self.port)             # <<<<<<<<<<<<<<
 * 
 *         raise NotImplementedError
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_obj, __pyx_n_s_host); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 271, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_host); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 271, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = PyObject_RichCompare(__pyx_t_4, __pyx_t_5, Py_NE); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 271, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 271, __pyx_L1_error)
      if (!__pyx_t_2) {
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else {
        __Pyx_INCREF(__pyx_t_6);
        __pyx_t_3 = __pyx_t_6;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        goto __pyx_L7_bool_binop_done;
      }
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_obj, __pyx_n_s_port); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 271, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_port); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 271, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_6, __pyx_t_5, Py_NE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 271, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_3 = __pyx_t_4;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_L7_bool_binop_done:;
      __pyx_r = __pyx_t_3;
      __pyx_t_3 = 0;
      goto __pyx_L0;

      /* "enet.pyx":269
 *                 # This is a '==' operation
 *                 return (obj.host == self.host) and (obj.port == self.port)
 *             elif op == 3:             # <<<<<<<<<<<<<<
 *                 # This is a '!=' operation
 *                 return (obj.host != self.host) or (obj.port != self.port)
 */
    }

    /* "enet.pyx":265
 * 
 *     def __richcmp__(self, obj, op):
 *         if isinstance(obj, Address):             # <<<<<<<<<<<<<<
 *             if op == 2:
 *                 # This is a '==' operation
 */
  }

  /* "enet.pyx":273
 *                 return (obj.host != self.host) or (obj.port != self.port)
 * 
 *         raise NotImplementedError             # <<<<<<<<<<<<<<
 * 
 *     property host:
 */
  __Pyx_Raise(__pyx_builtin_NotImplementedError, 0, 0, 0);
  __PYX_ERR(0, 273, __pyx_L1_error)

  /* "enet.pyx":264
 *         return "{0}:{1}".format(self.host, self.port)
 * 
 *     def __richcmp__(self, obj, op):             # <<<<<<<<<<<<<<
 *         if isinstance(obj, Address):
 *             if op == 2:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("bzenet.Address.__richcmp__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":276
 * 
 *     property host:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             cdef char host[MAXHOSTNAME]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_7Address_4host_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_7Address_4host_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_7Address_4host___get__(((struct __pyx_obj_6bzenet_Address *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_7Address_4host___get__(struct __pyx_obj_6bzenet_Address *__pyx_v_self) {
  char __pyx_v_host[__pyx_e_6bzenet_MAXHOSTNAME];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":279
 *             cdef char host[MAXHOSTNAME]
 * 
 *             if self._enet_address.host == ENET_HOST_ANY:             # <<<<<<<<<<<<<<
 *                 return "*"
 *             elif self._enet_address.host:
 */
  __pyx_t_1 = ((__pyx_v_self->_enet_address.host == ENET_HOST_ANY) != 0);
  if (__pyx_t_1) {

    /* "enet.pyx":280
 * 
 *             if self._enet_address.host == ENET_HOST_ANY:
 *                 return "*"             # <<<<<<<<<<<<<<
 *             elif self._enet_address.host:
 *                 if enet_address_get_host_ip(&self._enet_address, host, MAXHOSTNAME):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_s_);
    __pyx_r = __pyx_kp_s_;
    goto __pyx_L0;

    /* "enet.pyx":279
 *             cdef char host[MAXHOSTNAME]
 * 
 *             if self._enet_address.host == ENET_HOST_ANY:             # <<<<<<<<<<<<<<
 *                 return "*"
 *             elif self._enet_address.host:
 */
  }

  /* "enet.pyx":281
 *             if self._enet_address.host == ENET_HOST_ANY:
 *                 return "*"
 *             elif self._enet_address.host:             # <<<<<<<<<<<<<<
 *                 if enet_address_get_host_ip(&self._enet_address, host, MAXHOSTNAME):
 *                     raise IOError("Resolution failure!")
 */
  __pyx_t_1 = (__pyx_v_self->_enet_address.host != 0);
  if (__pyx_t_1) {

    /* "enet.pyx":282
 *                 return "*"
 *             elif self._enet_address.host:
 *                 if enet_address_get_host_ip(&self._enet_address, host, MAXHOSTNAME):             # <<<<<<<<<<<<<<
 *                     raise IOError("Resolution failure!")
 *                 return unicode(host, "ascii")
 */
    __pyx_t_1 = (enet_address_get_host_ip((&__pyx_v_self->_enet_address), __pyx_v_host, __pyx_e_6bzenet_MAXHOSTNAME) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "enet.pyx":283
 *             elif self._enet_address.host:
 *                 if enet_address_get_host_ip(&self._enet_address, host, MAXHOSTNAME):
 *                     raise IOError("Resolution failure!")             # <<<<<<<<<<<<<<
 *                 return unicode(host, "ascii")
 * 
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_IOError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 283, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 283, __pyx_L1_error)

      /* "enet.pyx":282
 *                 return "*"
 *             elif self._enet_address.host:
 *                 if enet_address_get_host_ip(&self._enet_address, host, MAXHOSTNAME):             # <<<<<<<<<<<<<<
 *                     raise IOError("Resolution failure!")
 *                 return unicode(host, "ascii")
 */
    }

    /* "enet.pyx":284
 *                 if enet_address_get_host_ip(&self._enet_address, host, MAXHOSTNAME):
 *                     raise IOError("Resolution failure!")
 *                 return unicode(host, "ascii")             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, value):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_FromString(__pyx_v_host); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 284, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 284, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_ascii);
    __Pyx_GIVEREF(__pyx_n_s_ascii);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_n_s_ascii);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)(&PyUnicode_Type)), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 284, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "enet.pyx":281
 *             if self._enet_address.host == ENET_HOST_ANY:
 *                 return "*"
 *             elif self._enet_address.host:             # <<<<<<<<<<<<<<
 *                 if enet_address_get_host_ip(&self._enet_address, host, MAXHOSTNAME):
 *                     raise IOError("Resolution failure!")
 */
  }

  /* "enet.pyx":276
 * 
 *     property host:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             cdef char host[MAXHOSTNAME]
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Address.host.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":286
 *                 return unicode(host, "ascii")
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             if not value or value == "*":
 *                 self._enet_address.host = ENET_HOST_ANY
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_7Address_4host_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6bzenet_7Address_4host_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_7Address_4host_2__set__(((struct __pyx_obj_6bzenet_Address *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_7Address_4host_2__set__(struct __pyx_obj_6bzenet_Address *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  char *__pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "enet.pyx":287
 * 
 *         def __set__(self, value):
 *             if not value or value == "*":             # <<<<<<<<<<<<<<
 *                 self._enet_address.host = ENET_HOST_ANY
 *             else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 287, __pyx_L1_error)
  __pyx_t_3 = ((!__pyx_t_2) != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (__Pyx_PyString_Equals(__pyx_v_value, __pyx_kp_s_, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 287, __pyx_L1_error)
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "enet.pyx":288
 *         def __set__(self, value):
 *             if not value or value == "*":
 *                 self._enet_address.host = ENET_HOST_ANY             # <<<<<<<<<<<<<<
 *             else:
 *                 if enet_address_set_host(&self._enet_address, value):
 */
    __pyx_v_self->_enet_address.host = ENET_HOST_ANY;

    /* "enet.pyx":287
 * 
 *         def __set__(self, value):
 *             if not value or value == "*":             # <<<<<<<<<<<<<<
 *                 self._enet_address.host = ENET_HOST_ANY
 *             else:
 */
    goto __pyx_L3;
  }

  /* "enet.pyx":290
 *                 self._enet_address.host = ENET_HOST_ANY
 *             else:
 *                 if enet_address_set_host(&self._enet_address, value):             # <<<<<<<<<<<<<<
 *                     raise IOError("Resolution failure!")
 * 
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_PyObject_AsWritableString(__pyx_v_value); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 290, __pyx_L1_error)
    __pyx_t_1 = (enet_address_set_host((&__pyx_v_self->_enet_address), __pyx_t_4) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "enet.pyx":291
 *             else:
 *                 if enet_address_set_host(&self._enet_address, value):
 *                     raise IOError("Resolution failure!")             # <<<<<<<<<<<<<<
 * 
 *     property hostname:
 */
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_IOError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 291, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __PYX_ERR(0, 291, __pyx_L1_error)

      /* "enet.pyx":290
 *                 self._enet_address.host = ENET_HOST_ANY
 *             else:
 *                 if enet_address_set_host(&self._enet_address, value):             # <<<<<<<<<<<<<<
 *                     raise IOError("Resolution failure!")
 * 
 */
    }
  }
  __pyx_L3:;

  /* "enet.pyx":286
 *                 return unicode(host, "ascii")
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             if not value or value == "*":
 *                 self._enet_address.host = ENET_HOST_ANY
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("bzenet.Address.host.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":294
 * 
 *     property hostname:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             cdef char host[MAXHOSTNAME]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_7Address_8hostname_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_7Address_8hostname_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_7Address_8hostname___get__(((struct __pyx_obj_6bzenet_Address *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_7Address_8hostname___get__(struct __pyx_obj_6bzenet_Address *__pyx_v_self) {
  char __pyx_v_host[__pyx_e_6bzenet_MAXHOSTNAME];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":297
 *             cdef char host[MAXHOSTNAME]
 * 
 *             if self._enet_address.host == ENET_HOST_ANY:             # <<<<<<<<<<<<<<
 *                 return "*"
 *             elif self._enet_address.host:
 */
  __pyx_t_1 = ((__pyx_v_self->_enet_address.host == ENET_HOST_ANY) != 0);
  if (__pyx_t_1) {

    /* "enet.pyx":298
 * 
 *             if self._enet_address.host == ENET_HOST_ANY:
 *                 return "*"             # <<<<<<<<<<<<<<
 *             elif self._enet_address.host:
 *                 if enet_address_get_host(&self._enet_address, host, MAXHOSTNAME):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_s_);
    __pyx_r = __pyx_kp_s_;
    goto __pyx_L0;

    /* "enet.pyx":297
 *             cdef char host[MAXHOSTNAME]
 * 
 *             if self._enet_address.host == ENET_HOST_ANY:             # <<<<<<<<<<<<<<
 *                 return "*"
 *             elif self._enet_address.host:
 */
  }

  /* "enet.pyx":299
 *             if self._enet_address.host == ENET_HOST_ANY:
 *                 return "*"
 *             elif self._enet_address.host:             # <<<<<<<<<<<<<<
 *                 if enet_address_get_host(&self._enet_address, host, MAXHOSTNAME):
 *                     raise IOError("Resolution failure!")
 */
  __pyx_t_1 = (__pyx_v_self->_enet_address.host != 0);
  if (__pyx_t_1) {

    /* "enet.pyx":300
 *                 return "*"
 *             elif self._enet_address.host:
 *                 if enet_address_get_host(&self._enet_address, host, MAXHOSTNAME):             # <<<<<<<<<<<<<<
 *                     raise IOError("Resolution failure!")
 *                 return unicode(host, "ascii")
 */
    __pyx_t_1 = (enet_address_get_host((&__pyx_v_self->_enet_address), __pyx_v_host, __pyx_e_6bzenet_MAXHOSTNAME) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "enet.pyx":301
 *             elif self._enet_address.host:
 *                 if enet_address_get_host(&self._enet_address, host, MAXHOSTNAME):
 *                     raise IOError("Resolution failure!")             # <<<<<<<<<<<<<<
 *                 return unicode(host, "ascii")
 * 
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_IOError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 301, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 301, __pyx_L1_error)

      /* "enet.pyx":300
 *                 return "*"
 *             elif self._enet_address.host:
 *                 if enet_address_get_host(&self._enet_address, host, MAXHOSTNAME):             # <<<<<<<<<<<<<<
 *                     raise IOError("Resolution failure!")
 *                 return unicode(host, "ascii")
 */
    }

    /* "enet.pyx":302
 *                 if enet_address_get_host(&self._enet_address, host, MAXHOSTNAME):
 *                     raise IOError("Resolution failure!")
 *                 return unicode(host, "ascii")             # <<<<<<<<<<<<<<
 * 
 *     property port:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_FromString(__pyx_v_host); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 302, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 302, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_ascii);
    __Pyx_GIVEREF(__pyx_n_s_ascii);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_n_s_ascii);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)(&PyUnicode_Type)), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 302, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "enet.pyx":299
 *             if self._enet_address.host == ENET_HOST_ANY:
 *                 return "*"
 *             elif self._enet_address.host:             # <<<<<<<<<<<<<<
 *                 if enet_address_get_host(&self._enet_address, host, MAXHOSTNAME):
 *                     raise IOError("Resolution failure!")
 */
  }

  /* "enet.pyx":294
 * 
 *     property hostname:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             cdef char host[MAXHOSTNAME]
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Address.hostname.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":305
 * 
 *     property port:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_address.port
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_7Address_4port_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_7Address_4port_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_7Address_4port___get__(((struct __pyx_obj_6bzenet_Address *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_7Address_4port___get__(struct __pyx_obj_6bzenet_Address *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":306
 *     property port:
 *         def __get__(self):
 *             return self._enet_address.port             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_enet_uint16(__pyx_v_self->_enet_address.port); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 306, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "enet.pyx":305
 * 
 *     property port:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_address.port
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Address.port.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":308
 *             return self._enet_address.port
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self._enet_address.port = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_7Address_4port_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6bzenet_7Address_4port_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_7Address_4port_2__set__(((struct __pyx_obj_6bzenet_Address *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_7Address_4port_2__set__(struct __pyx_obj_6bzenet_Address *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  enet_uint16 __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "enet.pyx":309
 * 
 *         def __set__(self, value):
 *             self._enet_address.port = value             # <<<<<<<<<<<<<<
 * 
 * cdef class Packet:
 */
  __pyx_t_1 = __Pyx_PyInt_As_enet_uint16(__pyx_v_value); if (unlikely((__pyx_t_1 == ((enet_uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 309, __pyx_L1_error)
  __pyx_v_self->_enet_address.port = __pyx_t_1;

  /* "enet.pyx":308
 *             return self._enet_address.port
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self._enet_address.port = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("bzenet.Address.port.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self._enet_address must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_7Address_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6bzenet_7Address_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_7Address_6__reduce_cython__(((struct __pyx_obj_6bzenet_Address *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_7Address_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Address *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self._enet_address must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self._enet_address must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self._enet_address must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Address.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self._enet_address must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self._enet_address must be explicitly requested with @auto_pickle(True)")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_7Address_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6bzenet_7Address_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_7Address_8__setstate_cython__(((struct __pyx_obj_6bzenet_Address *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_7Address_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Address *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self._enet_address must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self._enet_address must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self._enet_address must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self._enet_address must be explicitly requested with @auto_pickle(True)")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Address.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":344
 *     cdef bool sent
 * 
 *     def __init__(self, data=None, flags=0):             # <<<<<<<<<<<<<<
 *         if data is not None:
 *             self._enet_packet = enet_packet_create(data, len(data), flags)
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_6Packet_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6bzenet_6Packet_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_flags = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_flags,0};
    PyObject* values[2] = {0,0};
    values[0] = ((PyObject *)Py_None);
    values[1] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flags);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 344, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_data = values[0];
    __pyx_v_flags = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 344, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzenet.Packet.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6bzenet_6Packet___init__(((struct __pyx_obj_6bzenet_Packet *)__pyx_v_self), __pyx_v_data, __pyx_v_flags);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_6Packet___init__(struct __pyx_obj_6bzenet_Packet *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  char *__pyx_t_3;
  Py_ssize_t __pyx_t_4;
  enet_uint32 __pyx_t_5;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "enet.pyx":345
 * 
 *     def __init__(self, data=None, flags=0):
 *         if data is not None:             # <<<<<<<<<<<<<<
 *             self._enet_packet = enet_packet_create(data, len(data), flags)
 * 
 */
  __pyx_t_1 = (__pyx_v_data != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "enet.pyx":346
 *     def __init__(self, data=None, flags=0):
 *         if data is not None:
 *             self._enet_packet = enet_packet_create(data, len(data), flags)             # <<<<<<<<<<<<<<
 * 
 *         # This will get set to True when a peer.send() is called with the Packet
 */
    __pyx_t_3 = __Pyx_PyObject_AsWritableString(__pyx_v_data); if (unlikely((!__pyx_t_3) && PyErr_Occurred())) __PYX_ERR(0, 346, __pyx_L1_error)
    __pyx_t_4 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 346, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyInt_As_enet_uint32(__pyx_v_flags); if (unlikely((__pyx_t_5 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 346, __pyx_L1_error)
    __pyx_v_self->_enet_packet = enet_packet_create(__pyx_t_3, __pyx_t_4, __pyx_t_5);

    /* "enet.pyx":345
 * 
 *     def __init__(self, data=None, flags=0):
 *         if data is not None:             # <<<<<<<<<<<<<<
 *             self._enet_packet = enet_packet_create(data, len(data), flags)
 * 
 */
  }

  /* "enet.pyx":351
 *         # to ensure we don't try to destroy this packet as ENET will handle that
 *         # for us.
 *         self.sent = False             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __Pyx_INCREF(Py_False);
  __Pyx_GIVEREF(Py_False);
  __Pyx_GOTREF(__pyx_v_self->sent);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->sent));
  __pyx_v_self->sent = ((PyBoolObject *)Py_False);

  /* "enet.pyx":344
 *     cdef bool sent
 * 
 *     def __init__(self, data=None, flags=0):             # <<<<<<<<<<<<<<
 *         if data is not None:
 *             self._enet_packet = enet_packet_create(data, len(data), flags)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("bzenet.Packet.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":353
 *         self.sent = False
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.is_valid() and not self.sent:
 *             enet_packet_destroy(self._enet_packet)
 */

/* Python wrapper */
static void __pyx_pw_6bzenet_6Packet_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_6bzenet_6Packet_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_6bzenet_6Packet_2__dealloc__(((struct __pyx_obj_6bzenet_Packet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_6bzenet_6Packet_2__dealloc__(struct __pyx_obj_6bzenet_Packet *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "enet.pyx":354
 * 
 *     def __dealloc__(self):
 *         if self.is_valid() and not self.sent:             # <<<<<<<<<<<<<<
 *             enet_packet_destroy(self._enet_packet)
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_valid); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 354, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 354, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 354, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_5) {
  } else {
    __pyx_t_1 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->sent)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 354, __pyx_L1_error)
  __pyx_t_6 = ((!__pyx_t_5) != 0);
  __pyx_t_1 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "enet.pyx":355
 *     def __dealloc__(self):
 *         if self.is_valid() and not self.sent:
 *             enet_packet_destroy(self._enet_packet)             # <<<<<<<<<<<<<<
 * 
 *     def is_valid(self):
 */
    enet_packet_destroy(__pyx_v_self->_enet_packet);

    /* "enet.pyx":354
 * 
 *     def __dealloc__(self):
 *         if self.is_valid() and not self.sent:             # <<<<<<<<<<<<<<
 *             enet_packet_destroy(self._enet_packet)
 * 
 */
  }

  /* "enet.pyx":353
 *         self.sent = False
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.is_valid() and not self.sent:
 *             enet_packet_destroy(self._enet_packet)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("bzenet.Packet.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "enet.pyx":357
 *             enet_packet_destroy(self._enet_packet)
 * 
 *     def is_valid(self):             # <<<<<<<<<<<<<<
 *         if self._enet_packet:
 *             return True
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_6Packet_5is_valid(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6bzenet_6Packet_5is_valid(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_valid (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_6Packet_4is_valid(((struct __pyx_obj_6bzenet_Packet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_6Packet_4is_valid(struct __pyx_obj_6bzenet_Packet *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("is_valid", 0);

  /* "enet.pyx":358
 * 
 *     def is_valid(self):
 *         if self._enet_packet:             # <<<<<<<<<<<<<<
 *             return True
 *         else:
 */
  __pyx_t_1 = (__pyx_v_self->_enet_packet != 0);
  if (__pyx_t_1) {

    /* "enet.pyx":359
 *     def is_valid(self):
 *         if self._enet_packet:
 *             return True             # <<<<<<<<<<<<<<
 *         else:
 *             return False
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_True);
    __pyx_r = Py_True;
    goto __pyx_L0;

    /* "enet.pyx":358
 * 
 *     def is_valid(self):
 *         if self._enet_packet:             # <<<<<<<<<<<<<<
 *             return True
 *         else:
 */
  }

  /* "enet.pyx":361
 *             return True
 *         else:
 *             return False             # <<<<<<<<<<<<<<
 * 
 *     property data:
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;
  }

  /* "enet.pyx":357
 *             enet_packet_destroy(self._enet_packet)
 * 
 *     def is_valid(self):             # <<<<<<<<<<<<<<
 *         if self._enet_packet:
 *             return True
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":364
 * 
 *     property data:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.is_valid():
 *                 return (<char *>self._enet_packet.data)[:self._enet_packet.dataLength]
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_6Packet_4data_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_6Packet_4data_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_6Packet_4data___get__(((struct __pyx_obj_6bzenet_Packet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_6Packet_4data___get__(struct __pyx_obj_6bzenet_Packet *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":365
 *     property data:
 *         def __get__(self):
 *             if self.is_valid():             # <<<<<<<<<<<<<<
 *                 return (<char *>self._enet_packet.data)[:self._enet_packet.dataLength]
 *             else:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(__pyx_t_4)) {

    /* "enet.pyx":366
 *         def __get__(self):
 *             if self.is_valid():
 *                 return (<char *>self._enet_packet.data)[:self._enet_packet.dataLength]             # <<<<<<<<<<<<<<
 *             else:
 *                 raise MemoryError("Packet has not been initiliazed properly!")
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyBytes_FromStringAndSize(((char *)__pyx_v_self->_enet_packet->data) + 0, __pyx_v_self->_enet_packet->dataLength - 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 366, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":365
 *     property data:
 *         def __get__(self):
 *             if self.is_valid():             # <<<<<<<<<<<<<<
 *                 return (<char *>self._enet_packet.data)[:self._enet_packet.dataLength]
 *             else:
 */
  }

  /* "enet.pyx":368
 *                 return (<char *>self._enet_packet.data)[:self._enet_packet.dataLength]
 *             else:
 *                 raise MemoryError("Packet has not been initiliazed properly!")             # <<<<<<<<<<<<<<
 * 
 *     property dataLength:
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 368, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 368, __pyx_L1_error)
  }

  /* "enet.pyx":364
 * 
 *     property data:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.is_valid():
 *                 return (<char *>self._enet_packet.data)[:self._enet_packet.dataLength]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Packet.data.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":371
 * 
 *     property dataLength:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.is_valid():
 *                 return self._enet_packet.dataLength
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_6Packet_10dataLength_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_6Packet_10dataLength_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_6Packet_10dataLength___get__(((struct __pyx_obj_6bzenet_Packet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_6Packet_10dataLength___get__(struct __pyx_obj_6bzenet_Packet *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":372
 *     property dataLength:
 *         def __get__(self):
 *             if self.is_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_packet.dataLength
 *             else:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 372, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 372, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 372, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(__pyx_t_4)) {

    /* "enet.pyx":373
 *         def __get__(self):
 *             if self.is_valid():
 *                 return self._enet_packet.dataLength             # <<<<<<<<<<<<<<
 *             else:
 *                 raise MemoryError("Packet has not been initiliazed properly!")
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->_enet_packet->dataLength); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 373, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":372
 *     property dataLength:
 *         def __get__(self):
 *             if self.is_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_packet.dataLength
 *             else:
 */
  }

  /* "enet.pyx":375
 *                 return self._enet_packet.dataLength
 *             else:
 *                 raise MemoryError("Packet has not been initiliazed properly!")             # <<<<<<<<<<<<<<
 * 
 *     property flags:
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 375, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 375, __pyx_L1_error)
  }

  /* "enet.pyx":371
 * 
 *     property dataLength:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.is_valid():
 *                 return self._enet_packet.dataLength
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Packet.dataLength.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":378
 * 
 *     property flags:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.is_valid():
 *                 return self._enet_packet.flags
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_6Packet_5flags_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_6Packet_5flags_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_6Packet_5flags___get__(((struct __pyx_obj_6bzenet_Packet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_6Packet_5flags___get__(struct __pyx_obj_6bzenet_Packet *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":379
 *     property flags:
 *         def __get__(self):
 *             if self.is_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_packet.flags
 *             else:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(__pyx_t_4)) {

    /* "enet.pyx":380
 *         def __get__(self):
 *             if self.is_valid():
 *                 return self._enet_packet.flags             # <<<<<<<<<<<<<<
 *             else:
 *                 raise MemoryError("Packet has not been initiliazed properly!")
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_packet->flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":379
 *     property flags:
 *         def __get__(self):
 *             if self.is_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_packet.flags
 *             else:
 */
  }

  /* "enet.pyx":382
 *                 return self._enet_packet.flags
 *             else:
 *                 raise MemoryError("Packet has not been initiliazed properly!")             # <<<<<<<<<<<<<<
 * 
 *     property sent:
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 382, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 382, __pyx_L1_error)
  }

  /* "enet.pyx":378
 * 
 *     property flags:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.is_valid():
 *                 return self._enet_packet.flags
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Packet.flags.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":385
 * 
 *     property sent:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.sent
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_6Packet_4sent_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_6Packet_4sent_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_6Packet_4sent___get__(((struct __pyx_obj_6bzenet_Packet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_6Packet_4sent___get__(struct __pyx_obj_6bzenet_Packet *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":386
 *     property sent:
 *         def __get__(self):
 *             return self.sent             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.sent = value
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->sent));
  __pyx_r = ((PyObject *)__pyx_v_self->sent);
  goto __pyx_L0;

  /* "enet.pyx":385
 * 
 *     property sent:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.sent
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":387
 *         def __get__(self):
 *             return self.sent
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.sent = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_6Packet_4sent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6bzenet_6Packet_4sent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_6Packet_4sent_2__set__(((struct __pyx_obj_6bzenet_Packet *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_6Packet_4sent_2__set__(struct __pyx_obj_6bzenet_Packet *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "enet.pyx":388
 *             return self.sent
 *         def __set__(self, value):
 *             self.sent = value             # <<<<<<<<<<<<<<
 * 
 * cdef class Peer:
 */
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_7cpython_4bool_bool))))) __PYX_ERR(0, 388, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->sent);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->sent));
  __pyx_v_self->sent = ((PyBoolObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "enet.pyx":387
 *         def __get__(self):
 *             return self.sent
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.sent = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Packet.sent.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._enet_packet cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_6Packet_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6bzenet_6Packet_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_6Packet_6__reduce_cython__(((struct __pyx_obj_6bzenet_Packet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_6Packet_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Packet *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._enet_packet cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._enet_packet cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._enet_packet cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Packet.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._enet_packet cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._enet_packet cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_6Packet_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6bzenet_6Packet_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_6Packet_8__setstate_cython__(((struct __pyx_obj_6bzenet_Packet *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_6Packet_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Packet *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self._enet_packet cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._enet_packet cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._enet_packet cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._enet_packet cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Packet.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":422
 *     cdef ENetPeer *_enet_peer
 * 
 *     def __richcmp__(self, obj, op):             # <<<<<<<<<<<<<<
 *         if isinstance(obj, Peer):
 *             if op == 2:
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_1__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_arg_op); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_1__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_arg_op) {
  PyObject *__pyx_v_op = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__richcmp__ (wrapper)", 0);
  __pyx_v_op = __Pyx_PyInt_From_int(__pyx_arg_op); if (unlikely(!__pyx_v_op)) __PYX_ERR(0, 422, __pyx_L3_error)
  __Pyx_GOTREF(__pyx_v_op);
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzenet.Peer.__richcmp__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6bzenet_4Peer___richcmp__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self), ((PyObject *)__pyx_v_obj), ((PyObject *)__pyx_v_op));

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_op);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer___richcmp__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_obj, PyObject *__pyx_v_op) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__richcmp__", 0);

  /* "enet.pyx":423
 * 
 *     def __richcmp__(self, obj, op):
 *         if isinstance(obj, Peer):             # <<<<<<<<<<<<<<
 *             if op == 2:
 *                 return self.address == obj.address
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_obj, __pyx_ptype_6bzenet_Peer); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "enet.pyx":424
 *     def __richcmp__(self, obj, op):
 *         if isinstance(obj, Peer):
 *             if op == 2:             # <<<<<<<<<<<<<<
 *                 return self.address == obj.address
 *             elif op == 3:
 */
    __pyx_t_3 = __Pyx_PyInt_EqObjC(__pyx_v_op, __pyx_int_2, 2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 424, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 424, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_2) {

      /* "enet.pyx":425
 *         if isinstance(obj, Peer):
 *             if op == 2:
 *                 return self.address == obj.address             # <<<<<<<<<<<<<<
 *             elif op == 3:
 *                 return self.address != obj.address
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_address); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 425, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_obj, __pyx_n_s_address); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 425, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = PyObject_RichCompare(__pyx_t_3, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 425, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_r = __pyx_t_5;
      __pyx_t_5 = 0;
      goto __pyx_L0;

      /* "enet.pyx":424
 *     def __richcmp__(self, obj, op):
 *         if isinstance(obj, Peer):
 *             if op == 2:             # <<<<<<<<<<<<<<
 *                 return self.address == obj.address
 *             elif op == 3:
 */
    }

    /* "enet.pyx":426
 *             if op == 2:
 *                 return self.address == obj.address
 *             elif op == 3:             # <<<<<<<<<<<<<<
 *                 return self.address != obj.address
 *         raise NotImplementedError
 */
    __pyx_t_5 = __Pyx_PyInt_EqObjC(__pyx_v_op, __pyx_int_3, 3, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 426, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 426, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__pyx_t_2) {

      /* "enet.pyx":427
 *                 return self.address == obj.address
 *             elif op == 3:
 *                 return self.address != obj.address             # <<<<<<<<<<<<<<
 *         raise NotImplementedError
 * 
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_address); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_obj, __pyx_n_s_address); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_t_5, __pyx_t_4, Py_NE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_r = __pyx_t_3;
      __pyx_t_3 = 0;
      goto __pyx_L0;

      /* "enet.pyx":426
 *             if op == 2:
 *                 return self.address == obj.address
 *             elif op == 3:             # <<<<<<<<<<<<<<
 *                 return self.address != obj.address
 *         raise NotImplementedError
 */
    }

    /* "enet.pyx":423
 * 
 *     def __richcmp__(self, obj, op):
 *         if isinstance(obj, Peer):             # <<<<<<<<<<<<<<
 *             if op == 2:
 *                 return self.address == obj.address
 */
  }

  /* "enet.pyx":428
 *             elif op == 3:
 *                 return self.address != obj.address
 *         raise NotImplementedError             # <<<<<<<<<<<<<<
 * 
 *     def __hash__(self):
 */
  __Pyx_Raise(__pyx_builtin_NotImplementedError, 0, 0, 0);
  __PYX_ERR(0, 428, __pyx_L1_error)

  /* "enet.pyx":422
 *     cdef ENetPeer *_enet_peer
 * 
 *     def __richcmp__(self, obj, op):             # <<<<<<<<<<<<<<
 *         if isinstance(obj, Peer):
 *             if op == 2:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("bzenet.Peer.__richcmp__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":430
 *         raise NotImplementedError
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         return <uintptr_t>self._enet_peer
 * 
 */

/* Python wrapper */
static Py_hash_t __pyx_pw_6bzenet_4Peer_3__hash__(PyObject *__pyx_v_self); /*proto*/
static Py_hash_t __pyx_pw_6bzenet_4Peer_3__hash__(PyObject *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__hash__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_2__hash__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_hash_t __pyx_pf_6bzenet_4Peer_2__hash__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__hash__", 0);

  /* "enet.pyx":431
 * 
 *     def __hash__(self):
 *         return <uintptr_t>self._enet_peer             # <<<<<<<<<<<<<<
 * 
 *     def send(self, channelID, Packet packet):
 */
  __pyx_r = ((uintptr_t)__pyx_v_self->_enet_peer);
  goto __pyx_L0;

  /* "enet.pyx":430
 *         raise NotImplementedError
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         return <uintptr_t>self._enet_peer
 * 
 */

  /* function exit code */
  __pyx_L0:;
  if (unlikely(__pyx_r == -1) && !PyErr_Occurred()) __pyx_r = -2;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":433
 *         return <uintptr_t>self._enet_peer
 * 
 *     def send(self, channelID, Packet packet):             # <<<<<<<<<<<<<<
 *         """
 *         send (int channelID, Packet packet)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_5send(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6bzenet_4Peer_4send[] = "\n        send (int channelID, Packet packet)\n\n        Queues a packet to be sent.\n\n        returns 0 on success, < 0 on failure\n        ";
static PyObject *__pyx_pw_6bzenet_4Peer_5send(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_channelID = 0;
  struct __pyx_obj_6bzenet_Packet *__pyx_v_packet = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("send (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_channelID,&__pyx_n_s_packet,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_channelID)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_packet)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("send", 1, 2, 2, 1); __PYX_ERR(0, 433, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "send") < 0)) __PYX_ERR(0, 433, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_channelID = values[0];
    __pyx_v_packet = ((struct __pyx_obj_6bzenet_Packet *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("send", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 433, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzenet.Peer.send", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_packet), __pyx_ptype_6bzenet_Packet, 1, "packet", 0))) __PYX_ERR(0, 433, __pyx_L1_error)
  __pyx_r = __pyx_pf_6bzenet_4Peer_4send(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self), __pyx_v_channelID, __pyx_v_packet);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_4send(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_channelID, struct __pyx_obj_6bzenet_Packet *__pyx_v_packet) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  enet_uint8 __pyx_t_6;
  __Pyx_RefNannySetupContext("send", 0);

  /* "enet.pyx":442
 *         """
 * 
 *         if self.check_valid() and packet.is_valid():             # <<<<<<<<<<<<<<
 *             packet.sent = True
 *             return enet_peer_send(self._enet_peer, channelID, packet._enet_packet)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_5) {
  } else {
    __pyx_t_1 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_packet), __pyx_n_s_is_valid); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_1 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "enet.pyx":443
 * 
 *         if self.check_valid() and packet.is_valid():
 *             packet.sent = True             # <<<<<<<<<<<<<<
 *             return enet_peer_send(self._enet_peer, channelID, packet._enet_packet)
 * 
 */
    __Pyx_INCREF(Py_True);
    __Pyx_GIVEREF(Py_True);
    __Pyx_GOTREF(__pyx_v_packet->sent);
    __Pyx_DECREF(((PyObject *)__pyx_v_packet->sent));
    __pyx_v_packet->sent = ((PyBoolObject *)Py_True);

    /* "enet.pyx":444
 *         if self.check_valid() and packet.is_valid():
 *             packet.sent = True
 *             return enet_peer_send(self._enet_peer, channelID, packet._enet_packet)             # <<<<<<<<<<<<<<
 * 
 *     def receive(self, unsigned char channelID):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_6 = __Pyx_PyInt_As_enet_uint8(__pyx_v_channelID); if (unlikely((__pyx_t_6 == ((enet_uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 444, __pyx_L1_error)
    __pyx_t_2 = __Pyx_PyInt_From_int(enet_peer_send(__pyx_v_self->_enet_peer, __pyx_t_6, __pyx_v_packet->_enet_packet)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 444, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "enet.pyx":442
 *         """
 * 
 *         if self.check_valid() and packet.is_valid():             # <<<<<<<<<<<<<<
 *             packet.sent = True
 *             return enet_peer_send(self._enet_peer, channelID, packet._enet_packet)
 */
  }

  /* "enet.pyx":433
 *         return <uintptr_t>self._enet_peer
 * 
 *     def send(self, channelID, Packet packet):             # <<<<<<<<<<<<<<
 *         """
 *         send (int channelID, Packet packet)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("bzenet.Peer.send", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":446
 *             return enet_peer_send(self._enet_peer, channelID, packet._enet_packet)
 * 
 *     def receive(self, unsigned char channelID):             # <<<<<<<<<<<<<<
 *         """
 *         receive (int channelID)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_7receive(PyObject *__pyx_v_self, PyObject *__pyx_arg_channelID); /*proto*/
static char __pyx_doc_6bzenet_4Peer_6receive[] = "\n        receive (int channelID)\n\n        Attempts to dequeue any incoming queued packet.\n        ";
static PyObject *__pyx_pw_6bzenet_4Peer_7receive(PyObject *__pyx_v_self, PyObject *__pyx_arg_channelID) {
  unsigned char __pyx_v_channelID;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("receive (wrapper)", 0);
  assert(__pyx_arg_channelID); {
    __pyx_v_channelID = __Pyx_PyInt_As_unsigned_char(__pyx_arg_channelID); if (unlikely((__pyx_v_channelID == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzenet.Peer.receive", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6bzenet_4Peer_6receive(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self), ((unsigned char)__pyx_v_channelID));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_6receive(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, unsigned char __pyx_v_channelID) {
  struct __pyx_obj_6bzenet_Packet *__pyx_v_packet = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("receive", 0);

  /* "enet.pyx":453
 *         """
 * 
 *         if self.check_valid():             # <<<<<<<<<<<<<<
 *             packet = Packet()
 *             (<Packet> packet)._enet_packet = enet_peer_receive(self._enet_peer, &channelID)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":454
 * 
 *         if self.check_valid():
 *             packet = Packet()             # <<<<<<<<<<<<<<
 *             (<Packet> packet)._enet_packet = enet_peer_receive(self._enet_peer, &channelID)
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_6bzenet_Packet)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 454, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_packet = ((struct __pyx_obj_6bzenet_Packet *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "enet.pyx":455
 *         if self.check_valid():
 *             packet = Packet()
 *             (<Packet> packet)._enet_packet = enet_peer_receive(self._enet_peer, &channelID)             # <<<<<<<<<<<<<<
 * 
 *             if packet._enet_packet:
 */
    __pyx_v_packet->_enet_packet = enet_peer_receive(__pyx_v_self->_enet_peer, (&__pyx_v_channelID));

    /* "enet.pyx":457
 *             (<Packet> packet)._enet_packet = enet_peer_receive(self._enet_peer, &channelID)
 * 
 *             if packet._enet_packet:             # <<<<<<<<<<<<<<
 *                 return packet
 *             else:
 */
    __pyx_t_4 = (__pyx_v_packet->_enet_packet != 0);
    if (__pyx_t_4) {

      /* "enet.pyx":458
 * 
 *             if packet._enet_packet:
 *                 return packet             # <<<<<<<<<<<<<<
 *             else:
 *                 return None
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(((PyObject *)__pyx_v_packet));
      __pyx_r = ((PyObject *)__pyx_v_packet);
      goto __pyx_L0;

      /* "enet.pyx":457
 *             (<Packet> packet)._enet_packet = enet_peer_receive(self._enet_peer, &channelID)
 * 
 *             if packet._enet_packet:             # <<<<<<<<<<<<<<
 *                 return packet
 *             else:
 */
    }

    /* "enet.pyx":460
 *                 return packet
 *             else:
 *                 return None             # <<<<<<<<<<<<<<
 * 
 *     def reset(self):
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      goto __pyx_L0;
    }

    /* "enet.pyx":453
 *         """
 * 
 *         if self.check_valid():             # <<<<<<<<<<<<<<
 *             packet = Packet()
 *             (<Packet> packet)._enet_packet = enet_peer_receive(self._enet_peer, &channelID)
 */
  }

  /* "enet.pyx":446
 *             return enet_peer_send(self._enet_peer, channelID, packet._enet_packet)
 * 
 *     def receive(self, unsigned char channelID):             # <<<<<<<<<<<<<<
 *         """
 *         receive (int channelID)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.receive", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_packet);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":462
 *                 return None
 * 
 *     def reset(self):             # <<<<<<<<<<<<<<
 *         """
 *         reset ()
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_9reset(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6bzenet_4Peer_8reset[] = "\n        reset ()\n\n        Forcefully disconnects a peer.\n        ";
static PyObject *__pyx_pw_6bzenet_4Peer_9reset(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("reset (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_8reset(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_8reset(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("reset", 0);

  /* "enet.pyx":469
 *         """
 * 
 *         if self.check_valid():             # <<<<<<<<<<<<<<
 *             enet_peer_reset(self._enet_peer)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":470
 * 
 *         if self.check_valid():
 *             enet_peer_reset(self._enet_peer)             # <<<<<<<<<<<<<<
 * 
 *     def timeout(self, time, time_min, time_max):
 */
    enet_peer_reset(__pyx_v_self->_enet_peer);

    /* "enet.pyx":469
 *         """
 * 
 *         if self.check_valid():             # <<<<<<<<<<<<<<
 *             enet_peer_reset(self._enet_peer)
 * 
 */
  }

  /* "enet.pyx":462
 *                 return None
 * 
 *     def reset(self):             # <<<<<<<<<<<<<<
 *         """
 *         reset ()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.reset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":472
 *             enet_peer_reset(self._enet_peer)
 * 
 *     def timeout(self, time, time_min, time_max):             # <<<<<<<<<<<<<<
 *         """
 *         timeout (int time, int time_min, int time_max)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_11timeout(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6bzenet_4Peer_10timeout[] = "\n        timeout (int time, int time_min, int time_max)\n\n        Sets the timeout parameters for a peer.\n        ";
static PyObject *__pyx_pw_6bzenet_4Peer_11timeout(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_time = 0;
  PyObject *__pyx_v_time_min = 0;
  PyObject *__pyx_v_time_max = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("timeout (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_time,&__pyx_n_s_time_min,&__pyx_n_s_time_max,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_time)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_time_min)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("timeout", 1, 3, 3, 1); __PYX_ERR(0, 472, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_time_max)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("timeout", 1, 3, 3, 2); __PYX_ERR(0, 472, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "timeout") < 0)) __PYX_ERR(0, 472, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_time = values[0];
    __pyx_v_time_min = values[1];
    __pyx_v_time_max = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("timeout", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 472, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzenet.Peer.timeout", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6bzenet_4Peer_10timeout(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self), __pyx_v_time, __pyx_v_time_min, __pyx_v_time_max);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_10timeout(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_time, PyObject *__pyx_v_time_min, PyObject *__pyx_v_time_max) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  enet_uint32 __pyx_t_5;
  enet_uint32 __pyx_t_6;
  enet_uint32 __pyx_t_7;
  __Pyx_RefNannySetupContext("timeout", 0);

  /* "enet.pyx":479
 *         """
 * 
 *         if self.check_valid():             # <<<<<<<<<<<<<<
 *             enet_peer_timeout(self._enet_peer, time, time_min, time_max)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 479, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 479, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 479, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":480
 * 
 *         if self.check_valid():
 *             enet_peer_timeout(self._enet_peer, time, time_min, time_max)             # <<<<<<<<<<<<<<
 * 
 *     def ping(self):
 */
    __pyx_t_5 = __Pyx_PyInt_As_enet_uint32(__pyx_v_time); if (unlikely((__pyx_t_5 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 480, __pyx_L1_error)
    __pyx_t_6 = __Pyx_PyInt_As_enet_uint32(__pyx_v_time_min); if (unlikely((__pyx_t_6 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 480, __pyx_L1_error)
    __pyx_t_7 = __Pyx_PyInt_As_enet_uint32(__pyx_v_time_max); if (unlikely((__pyx_t_7 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 480, __pyx_L1_error)
    enet_peer_timeout(__pyx_v_self->_enet_peer, __pyx_t_5, __pyx_t_6, __pyx_t_7);

    /* "enet.pyx":479
 *         """
 * 
 *         if self.check_valid():             # <<<<<<<<<<<<<<
 *             enet_peer_timeout(self._enet_peer, time, time_min, time_max)
 * 
 */
  }

  /* "enet.pyx":472
 *             enet_peer_reset(self._enet_peer)
 * 
 *     def timeout(self, time, time_min, time_max):             # <<<<<<<<<<<<<<
 *         """
 *         timeout (int time, int time_min, int time_max)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.timeout", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":482
 *             enet_peer_timeout(self._enet_peer, time, time_min, time_max)
 * 
 *     def ping(self):             # <<<<<<<<<<<<<<
 *         """
 *         ping ()
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_13ping(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6bzenet_4Peer_12ping[] = "\n        ping ()\n\n        Sends a ping request to a peer.\n        ";
static PyObject *__pyx_pw_6bzenet_4Peer_13ping(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ping (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_12ping(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_12ping(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("ping", 0);

  /* "enet.pyx":489
 *         """
 * 
 *         if self.check_valid():             # <<<<<<<<<<<<<<
 *             enet_peer_ping(self._enet_peer)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 489, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 489, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 489, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":490
 * 
 *         if self.check_valid():
 *             enet_peer_ping(self._enet_peer)             # <<<<<<<<<<<<<<
 * 
 *     def disconnect(self, data=0):
 */
    enet_peer_ping(__pyx_v_self->_enet_peer);

    /* "enet.pyx":489
 *         """
 * 
 *         if self.check_valid():             # <<<<<<<<<<<<<<
 *             enet_peer_ping(self._enet_peer)
 * 
 */
  }

  /* "enet.pyx":482
 *             enet_peer_timeout(self._enet_peer, time, time_min, time_max)
 * 
 *     def ping(self):             # <<<<<<<<<<<<<<
 *         """
 *         ping ()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.ping", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":492
 *             enet_peer_ping(self._enet_peer)
 * 
 *     def disconnect(self, data=0):             # <<<<<<<<<<<<<<
 *         """
 *         disconnect ()
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_15disconnect(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6bzenet_4Peer_14disconnect[] = "\n        disconnect ()\n\n        Request a disconnection from a peer.\n        ";
static PyObject *__pyx_pw_6bzenet_4Peer_15disconnect(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("disconnect (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "disconnect") < 0)) __PYX_ERR(0, 492, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_data = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("disconnect", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 492, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzenet.Peer.disconnect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6bzenet_4Peer_14disconnect(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self), __pyx_v_data);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_14disconnect(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  enet_uint32 __pyx_t_5;
  __Pyx_RefNannySetupContext("disconnect", 0);

  /* "enet.pyx":499
 *         """
 * 
 *         if self.check_valid():             # <<<<<<<<<<<<<<
 *             enet_peer_disconnect(self._enet_peer, data)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":500
 * 
 *         if self.check_valid():
 *             enet_peer_disconnect(self._enet_peer, data)             # <<<<<<<<<<<<<<
 * 
 *     def disconnect_later(self, data=0):
 */
    __pyx_t_5 = __Pyx_PyInt_As_enet_uint32(__pyx_v_data); if (unlikely((__pyx_t_5 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 500, __pyx_L1_error)
    enet_peer_disconnect(__pyx_v_self->_enet_peer, __pyx_t_5);

    /* "enet.pyx":499
 *         """
 * 
 *         if self.check_valid():             # <<<<<<<<<<<<<<
 *             enet_peer_disconnect(self._enet_peer, data)
 * 
 */
  }

  /* "enet.pyx":492
 *             enet_peer_ping(self._enet_peer)
 * 
 *     def disconnect(self, data=0):             # <<<<<<<<<<<<<<
 *         """
 *         disconnect ()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.disconnect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":502
 *             enet_peer_disconnect(self._enet_peer, data)
 * 
 *     def disconnect_later(self, data=0):             # <<<<<<<<<<<<<<
 *         """
 *         disconnect_later ()
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_17disconnect_later(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6bzenet_4Peer_16disconnect_later[] = "\n        disconnect_later ()\n\n        Request a disconnection from a peer, but only after all queued outgoing\n        packets are sent.\n        ";
static PyObject *__pyx_pw_6bzenet_4Peer_17disconnect_later(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("disconnect_later (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "disconnect_later") < 0)) __PYX_ERR(0, 502, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_data = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("disconnect_later", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 502, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzenet.Peer.disconnect_later", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6bzenet_4Peer_16disconnect_later(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self), __pyx_v_data);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_16disconnect_later(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  enet_uint32 __pyx_t_5;
  __Pyx_RefNannySetupContext("disconnect_later", 0);

  /* "enet.pyx":510
 *         """
 * 
 *         if self.check_valid():             # <<<<<<<<<<<<<<
 *             enet_peer_disconnect_later(self._enet_peer, data)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 510, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":511
 * 
 *         if self.check_valid():
 *             enet_peer_disconnect_later(self._enet_peer, data)             # <<<<<<<<<<<<<<
 * 
 *     def disconnect_now(self, data=0):
 */
    __pyx_t_5 = __Pyx_PyInt_As_enet_uint32(__pyx_v_data); if (unlikely((__pyx_t_5 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 511, __pyx_L1_error)
    enet_peer_disconnect_later(__pyx_v_self->_enet_peer, __pyx_t_5);

    /* "enet.pyx":510
 *         """
 * 
 *         if self.check_valid():             # <<<<<<<<<<<<<<
 *             enet_peer_disconnect_later(self._enet_peer, data)
 * 
 */
  }

  /* "enet.pyx":502
 *             enet_peer_disconnect(self._enet_peer, data)
 * 
 *     def disconnect_later(self, data=0):             # <<<<<<<<<<<<<<
 *         """
 *         disconnect_later ()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.disconnect_later", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":513
 *             enet_peer_disconnect_later(self._enet_peer, data)
 * 
 *     def disconnect_now(self, data=0):             # <<<<<<<<<<<<<<
 *         """
 *         disconnect_now ()
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_19disconnect_now(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6bzenet_4Peer_18disconnect_now[] = "\n        disconnect_now ()\n\n        Force an immediate disconnection from a peer.\n        ";
static PyObject *__pyx_pw_6bzenet_4Peer_19disconnect_now(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("disconnect_now (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "disconnect_now") < 0)) __PYX_ERR(0, 513, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_data = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("disconnect_now", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 513, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzenet.Peer.disconnect_now", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6bzenet_4Peer_18disconnect_now(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self), __pyx_v_data);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_18disconnect_now(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  enet_uint32 __pyx_t_5;
  __Pyx_RefNannySetupContext("disconnect_now", 0);

  /* "enet.pyx":520
 *         """
 * 
 *         if self.check_valid():             # <<<<<<<<<<<<<<
 *             enet_peer_disconnect_now(self._enet_peer, data)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 520, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 520, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 520, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":521
 * 
 *         if self.check_valid():
 *             enet_peer_disconnect_now(self._enet_peer, data)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_5 = __Pyx_PyInt_As_enet_uint32(__pyx_v_data); if (unlikely((__pyx_t_5 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 521, __pyx_L1_error)
    enet_peer_disconnect_now(__pyx_v_self->_enet_peer, __pyx_t_5);

    /* "enet.pyx":520
 *         """
 * 
 *         if self.check_valid():             # <<<<<<<<<<<<<<
 *             enet_peer_disconnect_now(self._enet_peer, data)
 * 
 */
  }

  /* "enet.pyx":513
 *             enet_peer_disconnect_later(self._enet_peer, data)
 * 
 *     def disconnect_now(self, data=0):             # <<<<<<<<<<<<<<
 *         """
 *         disconnect_now ()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.disconnect_now", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":524
 * 
 * 
 *     def check_valid(self):             # <<<<<<<<<<<<<<
 *         """
 *         check_valid ()
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_21check_valid(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6bzenet_4Peer_20check_valid[] = "\n        check_valid ()\n\n        Returns True if there is a valid enet_peer set\n        Raises a Memory error if not\n\n        ";
static PyObject *__pyx_pw_6bzenet_4Peer_21check_valid(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("check_valid (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_20check_valid(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_20check_valid(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("check_valid", 0);

  /* "enet.pyx":532
 * 
 *         """
 *         if self._enet_peer:             # <<<<<<<<<<<<<<
 *             return True
 *         else:
 */
  __pyx_t_1 = (__pyx_v_self->_enet_peer != 0);
  if (likely(__pyx_t_1)) {

    /* "enet.pyx":533
 *         """
 *         if self._enet_peer:
 *             return True             # <<<<<<<<<<<<<<
 *         else:
 *             raise MemoryError("Empty Peer object accessed!")
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_True);
    __pyx_r = Py_True;
    goto __pyx_L0;

    /* "enet.pyx":532
 * 
 *         """
 *         if self._enet_peer:             # <<<<<<<<<<<<<<
 *             return True
 *         else:
 */
  }

  /* "enet.pyx":535
 *             return True
 *         else:
 *             raise MemoryError("Empty Peer object accessed!")             # <<<<<<<<<<<<<<
 * 
 *     property host:
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 535, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 535, __pyx_L1_error)
  }

  /* "enet.pyx":524
 * 
 * 
 *     def check_valid(self):             # <<<<<<<<<<<<<<
 *         """
 *         check_valid ()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("bzenet.Peer.check_valid", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":538
 * 
 *     property host:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 # be a bit like pickle here. otherwise Host
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_4host_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_4host_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_4host___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_4host___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_v_h = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  ENetHost *__pyx_t_5;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":539
 *     property host:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 # be a bit like pickle here. otherwise Host
 *                 # would create a new enet host inside __init__
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":542
 *                 # be a bit like pickle here. otherwise Host
 *                 # would create a new enet host inside __init__
 *                 h = Host.__new__(Host)             # <<<<<<<<<<<<<<
 *                 (<Host> h)._enet_host = self._enet_peer.host
 *                 return h
 */
    __pyx_t_1 = ((PyObject *)__pyx_tp_new_6bzenet_Host(((PyTypeObject *)__pyx_ptype_6bzenet_Host), __pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 542, __pyx_L1_error)
    __Pyx_GOTREF(((PyObject *)__pyx_t_1));
    __pyx_v_h = ((PyObject *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "enet.pyx":543
 *                 # would create a new enet host inside __init__
 *                 h = Host.__new__(Host)
 *                 (<Host> h)._enet_host = self._enet_peer.host             # <<<<<<<<<<<<<<
 *                 return h
 * 
 */
    __pyx_t_5 = __pyx_v_self->_enet_peer->host;
    ((struct __pyx_obj_6bzenet_Host *)__pyx_v_h)->_enet_host = __pyx_t_5;

    /* "enet.pyx":544
 *                 h = Host.__new__(Host)
 *                 (<Host> h)._enet_host = self._enet_peer.host
 *                 return h             # <<<<<<<<<<<<<<
 * 
 *     property outgoingPeerID:
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_h);
    __pyx_r = __pyx_v_h;
    goto __pyx_L0;

    /* "enet.pyx":539
 *     property host:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 # be a bit like pickle here. otherwise Host
 *                 # would create a new enet host inside __init__
 */
  }

  /* "enet.pyx":538
 * 
 *     property host:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 # be a bit like pickle here. otherwise Host
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.host.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_h);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":547
 * 
 *     property outgoingPeerID:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingPeerID
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_14outgoingPeerID_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_14outgoingPeerID_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_14outgoingPeerID___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_14outgoingPeerID___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":548
 *     property outgoingPeerID:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.outgoingPeerID
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 548, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 548, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 548, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":549
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingPeerID             # <<<<<<<<<<<<<<
 * 
 *     property incomingPeerID:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint16(__pyx_v_self->_enet_peer->outgoingPeerID); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 549, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":548
 *     property outgoingPeerID:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.outgoingPeerID
 * 
 */
  }

  /* "enet.pyx":547
 * 
 *     property outgoingPeerID:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingPeerID
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.outgoingPeerID.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":552
 * 
 *     property incomingPeerID:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.incomingPeerID
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_14incomingPeerID_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_14incomingPeerID_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_14incomingPeerID___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_14incomingPeerID___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":553
 *     property incomingPeerID:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.incomingPeerID
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":554
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.incomingPeerID             # <<<<<<<<<<<<<<
 * 
 *     property connectID:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint16(__pyx_v_self->_enet_peer->incomingPeerID); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 554, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":553
 *     property incomingPeerID:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.incomingPeerID
 * 
 */
  }

  /* "enet.pyx":552
 * 
 *     property incomingPeerID:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.incomingPeerID
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.incomingPeerID.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":557
 * 
 *     property connectID:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.connectID
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_9connectID_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_9connectID_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_9connectID___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_9connectID___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":558
 *     property connectID:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.connectID
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 558, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 558, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 558, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":559
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.connectID             # <<<<<<<<<<<<<<
 * 
 *     property outgoingSessionID:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->connectID); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 559, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":558
 *     property connectID:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.connectID
 * 
 */
  }

  /* "enet.pyx":557
 * 
 *     property connectID:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.connectID
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.connectID.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":562
 * 
 *     property outgoingSessionID:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingSessionID
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_17outgoingSessionID_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_17outgoingSessionID_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_17outgoingSessionID___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_17outgoingSessionID___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":563
 *     property outgoingSessionID:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.outgoingSessionID
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 563, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 563, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 563, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":564
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingSessionID             # <<<<<<<<<<<<<<
 * 
 *     property incomingSessionID:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint8(__pyx_v_self->_enet_peer->outgoingSessionID); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 564, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":563
 *     property outgoingSessionID:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.outgoingSessionID
 * 
 */
  }

  /* "enet.pyx":562
 * 
 *     property outgoingSessionID:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingSessionID
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.outgoingSessionID.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":567
 * 
 *     property incomingSessionID:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.incomingSessionID
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_17incomingSessionID_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_17incomingSessionID_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_17incomingSessionID___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_17incomingSessionID___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":568
 *     property incomingSessionID:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.incomingSessionID
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":569
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.incomingSessionID             # <<<<<<<<<<<<<<
 * 
 *     property address:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint8(__pyx_v_self->_enet_peer->incomingSessionID); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 569, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":568
 *     property incomingSessionID:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.incomingSessionID
 * 
 */
  }

  /* "enet.pyx":567
 * 
 *     property incomingSessionID:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.incomingSessionID
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.incomingSessionID.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":572
 * 
 *     property address:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 a = Address(None, 0)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_7address_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_7address_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_7address___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_7address___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  struct __pyx_obj_6bzenet_Address *__pyx_v_a = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  ENetAddress __pyx_t_5;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":573
 *     property address:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 a = Address(None, 0)
 *                 (<Address> a)._enet_address = self._enet_peer.address
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 573, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":574
 *         def __get__(self):
 *             if self.check_valid():
 *                 a = Address(None, 0)             # <<<<<<<<<<<<<<
 *                 (<Address> a)._enet_address = self._enet_peer.address
 *                 return a
 */
    __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6bzenet_Address), __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_a = ((struct __pyx_obj_6bzenet_Address *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "enet.pyx":575
 *             if self.check_valid():
 *                 a = Address(None, 0)
 *                 (<Address> a)._enet_address = self._enet_peer.address             # <<<<<<<<<<<<<<
 *                 return a
 * 
 */
    __pyx_t_5 = __pyx_v_self->_enet_peer->address;
    __pyx_v_a->_enet_address = __pyx_t_5;

    /* "enet.pyx":576
 *                 a = Address(None, 0)
 *                 (<Address> a)._enet_address = self._enet_peer.address
 *                 return a             # <<<<<<<<<<<<<<
 * 
 *     property data:
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_a));
    __pyx_r = ((PyObject *)__pyx_v_a);
    goto __pyx_L0;

    /* "enet.pyx":573
 *     property address:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 a = Address(None, 0)
 *                 (<Address> a)._enet_address = self._enet_peer.address
 */
  }

  /* "enet.pyx":572
 * 
 *     property address:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 a = Address(None, 0)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.address.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_a);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":579
 * 
 *     property data:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.data
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_4data_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_4data_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_4data___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_4data___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":580
 *     property data:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.data
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 580, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 580, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 580, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":581
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.data             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, value):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_self->_enet_peer->data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 581, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":580
 *     property data:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.data
 * 
 */
  }

  /* "enet.pyx":579
 * 
 *     property data:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.data
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.data.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":583
 *                 return self._enet_peer.data
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 self._enet_peer.data = value
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_4Peer_4data_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6bzenet_4Peer_4data_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_4data_2__set__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_4Peer_4data_2__set__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  char *__pyx_t_5;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "enet.pyx":584
 * 
 *         def __set__(self, value):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 self._enet_peer.data = value
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 584, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":585
 *         def __set__(self, value):
 *             if self.check_valid():
 *                 self._enet_peer.data = value             # <<<<<<<<<<<<<<
 * 
 *     property state:
 */
    __pyx_t_5 = __Pyx_PyObject_AsWritableString(__pyx_v_value); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 585, __pyx_L1_error)
    __pyx_v_self->_enet_peer->data = __pyx_t_5;

    /* "enet.pyx":584
 * 
 *         def __set__(self, value):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 self._enet_peer.data = value
 * 
 */
  }

  /* "enet.pyx":583
 *                 return self._enet_peer.data
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 self._enet_peer.data = value
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.data.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":588
 * 
 *     property state:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.state
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_5state_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_5state_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_5state___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_5state___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":589
 *     property state:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.state
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 589, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 589, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 589, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":590
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.state             # <<<<<<<<<<<<<<
 * 
 *     property channelCount:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_ENetPeerState(__pyx_v_self->_enet_peer->state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 590, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":589
 *     property state:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.state
 * 
 */
  }

  /* "enet.pyx":588
 * 
 *     property state:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.state
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.state.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":593
 * 
 *     property channelCount:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.channelCount
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_12channelCount_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_12channelCount_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_12channelCount___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_12channelCount___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":594
 *     property channelCount:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.channelCount
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 594, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 594, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 594, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":595
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.channelCount             # <<<<<<<<<<<<<<
 * 
 *     property incomingBandwidth:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->_enet_peer->channelCount); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 595, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":594
 *     property channelCount:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.channelCount
 * 
 */
  }

  /* "enet.pyx":593
 * 
 *     property channelCount:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.channelCount
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.channelCount.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":598
 * 
 *     property incomingBandwidth:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.incomingBandwidth
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_17incomingBandwidth_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_17incomingBandwidth_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_17incomingBandwidth___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_17incomingBandwidth___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":599
 *     property incomingBandwidth:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.incomingBandwidth
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":600
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.incomingBandwidth             # <<<<<<<<<<<<<<
 * 
 *     property outgoingBandwidth:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->incomingBandwidth); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 600, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":599
 *     property incomingBandwidth:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.incomingBandwidth
 * 
 */
  }

  /* "enet.pyx":598
 * 
 *     property incomingBandwidth:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.incomingBandwidth
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.incomingBandwidth.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":603
 * 
 *     property outgoingBandwidth:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingBandwidth
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_17outgoingBandwidth_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_17outgoingBandwidth_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_17outgoingBandwidth___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_17outgoingBandwidth___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":604
 *     property outgoingBandwidth:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.outgoingBandwidth
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 604, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 604, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 604, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":605
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingBandwidth             # <<<<<<<<<<<<<<
 * 
 *     property incomingBandwidthThrottleEpoch:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->outgoingBandwidth); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":604
 *     property outgoingBandwidth:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.outgoingBandwidth
 * 
 */
  }

  /* "enet.pyx":603
 * 
 *     property outgoingBandwidth:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingBandwidth
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.outgoingBandwidth.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":608
 * 
 *     property incomingBandwidthThrottleEpoch:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.incomingBandwidthThrottleEpoch
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_30incomingBandwidthThrottleEpoch_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_30incomingBandwidthThrottleEpoch_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_30incomingBandwidthThrottleEpoch___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_30incomingBandwidthThrottleEpoch___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":609
 *     property incomingBandwidthThrottleEpoch:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.incomingBandwidthThrottleEpoch
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 609, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 609, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 609, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":610
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.incomingBandwidthThrottleEpoch             # <<<<<<<<<<<<<<
 * 
 *     property outgoingBandwidthThrottleEpoch:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->incomingBandwidthThrottleEpoch); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 610, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":609
 *     property incomingBandwidthThrottleEpoch:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.incomingBandwidthThrottleEpoch
 * 
 */
  }

  /* "enet.pyx":608
 * 
 *     property incomingBandwidthThrottleEpoch:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.incomingBandwidthThrottleEpoch
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.incomingBandwidthThrottleEpoch.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":613
 * 
 *     property outgoingBandwidthThrottleEpoch:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingBandwidthThrottleEpoch
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_30outgoingBandwidthThrottleEpoch_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_30outgoingBandwidthThrottleEpoch_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_30outgoingBandwidthThrottleEpoch___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_30outgoingBandwidthThrottleEpoch___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":614
 *     property outgoingBandwidthThrottleEpoch:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.outgoingBandwidthThrottleEpoch
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 614, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 614, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 614, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":615
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingBandwidthThrottleEpoch             # <<<<<<<<<<<<<<
 * 
 *     property incomingDataTotal:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->outgoingBandwidthThrottleEpoch); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 615, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":614
 *     property outgoingBandwidthThrottleEpoch:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.outgoingBandwidthThrottleEpoch
 * 
 */
  }

  /* "enet.pyx":613
 * 
 *     property outgoingBandwidthThrottleEpoch:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingBandwidthThrottleEpoch
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.outgoingBandwidthThrottleEpoch.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":618
 * 
 *     property incomingDataTotal:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.incomingDataTotal
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_17incomingDataTotal_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_17incomingDataTotal_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_17incomingDataTotal___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_17incomingDataTotal___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":619
 *     property incomingDataTotal:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.incomingDataTotal
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 619, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 619, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 619, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":620
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.incomingDataTotal             # <<<<<<<<<<<<<<
 * 
 *     property outgoingDataTotal:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->incomingDataTotal); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 620, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":619
 *     property incomingDataTotal:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.incomingDataTotal
 * 
 */
  }

  /* "enet.pyx":618
 * 
 *     property incomingDataTotal:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.incomingDataTotal
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.incomingDataTotal.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":623
 * 
 *     property outgoingDataTotal:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingDataTotal
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_17outgoingDataTotal_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_17outgoingDataTotal_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_17outgoingDataTotal___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_17outgoingDataTotal___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":624
 *     property outgoingDataTotal:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.outgoingDataTotal
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 624, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 624, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 624, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":625
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingDataTotal             # <<<<<<<<<<<<<<
 * 
 *     property lastSendTime:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->outgoingDataTotal); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 625, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":624
 *     property outgoingDataTotal:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.outgoingDataTotal
 * 
 */
  }

  /* "enet.pyx":623
 * 
 *     property outgoingDataTotal:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingDataTotal
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.outgoingDataTotal.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":628
 * 
 *     property lastSendTime:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.lastSendTime
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_12lastSendTime_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_12lastSendTime_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_12lastSendTime___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_12lastSendTime___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":629
 *     property lastSendTime:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.lastSendTime
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":630
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.lastSendTime             # <<<<<<<<<<<<<<
 * 
 *     property lastReceiveTime:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->lastSendTime); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 630, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":629
 *     property lastSendTime:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.lastSendTime
 * 
 */
  }

  /* "enet.pyx":628
 * 
 *     property lastSendTime:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.lastSendTime
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.lastSendTime.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":633
 * 
 *     property lastReceiveTime:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.lastReceiveTime
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_15lastReceiveTime_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_15lastReceiveTime_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_15lastReceiveTime___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_15lastReceiveTime___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":634
 *     property lastReceiveTime:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.lastReceiveTime
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":635
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.lastReceiveTime             # <<<<<<<<<<<<<<
 * 
 *     property nextTimeout:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->lastReceiveTime); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 635, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":634
 *     property lastReceiveTime:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.lastReceiveTime
 * 
 */
  }

  /* "enet.pyx":633
 * 
 *     property lastReceiveTime:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.lastReceiveTime
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.lastReceiveTime.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":638
 * 
 *     property nextTimeout:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.nextTimeout
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_11nextTimeout_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_11nextTimeout_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_11nextTimeout___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_11nextTimeout___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":639
 *     property nextTimeout:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.nextTimeout
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 639, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 639, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 639, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":640
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.nextTimeout             # <<<<<<<<<<<<<<
 * 
 *     property earliestTimeout:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->nextTimeout); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 640, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":639
 *     property nextTimeout:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.nextTimeout
 * 
 */
  }

  /* "enet.pyx":638
 * 
 *     property nextTimeout:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.nextTimeout
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.nextTimeout.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":643
 * 
 *     property earliestTimeout:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.earliestTimeout
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_15earliestTimeout_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_15earliestTimeout_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_15earliestTimeout___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_15earliestTimeout___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":644
 *     property earliestTimeout:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.earliestTimeout
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 644, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 644, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 644, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":645
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.earliestTimeout             # <<<<<<<<<<<<<<
 * 
 *     property packetLossEpoch:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->earliestTimeout); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 645, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":644
 *     property earliestTimeout:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.earliestTimeout
 * 
 */
  }

  /* "enet.pyx":643
 * 
 *     property earliestTimeout:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.earliestTimeout
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.earliestTimeout.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":648
 * 
 *     property packetLossEpoch:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetLossEpoch
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_15packetLossEpoch_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_15packetLossEpoch_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_15packetLossEpoch___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_15packetLossEpoch___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":649
 *     property packetLossEpoch:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetLossEpoch
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 649, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 649, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 649, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":650
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.packetLossEpoch             # <<<<<<<<<<<<<<
 * 
 *     property packetsSent:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->packetLossEpoch); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 650, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":649
 *     property packetLossEpoch:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetLossEpoch
 * 
 */
  }

  /* "enet.pyx":648
 * 
 *     property packetLossEpoch:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetLossEpoch
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.packetLossEpoch.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":653
 * 
 *     property packetsSent:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetsSent
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_11packetsSent_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_11packetsSent_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_11packetsSent___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_11packetsSent___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":654
 *     property packetsSent:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetsSent
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 654, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 654, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 654, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":655
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.packetsSent             # <<<<<<<<<<<<<<
 * 
 *     property packetsLost:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->packetsSent); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 655, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":654
 *     property packetsSent:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetsSent
 * 
 */
  }

  /* "enet.pyx":653
 * 
 *     property packetsSent:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetsSent
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.packetsSent.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":658
 * 
 *     property packetsLost:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetsLost
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_11packetsLost_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_11packetsLost_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_11packetsLost___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_11packetsLost___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":659
 *     property packetsLost:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetsLost
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 659, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 659, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 659, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":660
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.packetsLost             # <<<<<<<<<<<<<<
 * 
 *     property packetLoss:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->packetsLost); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 660, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":659
 *     property packetsLost:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetsLost
 * 
 */
  }

  /* "enet.pyx":658
 * 
 *     property packetsLost:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetsLost
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.packetsLost.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":663
 * 
 *     property packetLoss:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetLoss
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_10packetLoss_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_10packetLoss_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_10packetLoss___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_10packetLoss___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":664
 *     property packetLoss:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetLoss
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 664, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 664, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 664, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":665
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.packetLoss             # <<<<<<<<<<<<<<
 * 
 *     property packetLossVariance:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->packetLoss); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 665, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":664
 *     property packetLoss:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetLoss
 * 
 */
  }

  /* "enet.pyx":663
 * 
 *     property packetLoss:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetLoss
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.packetLoss.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":668
 * 
 *     property packetLossVariance:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetLossVariance
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_18packetLossVariance_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_18packetLossVariance_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_18packetLossVariance___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_18packetLossVariance___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":669
 *     property packetLossVariance:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetLossVariance
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 669, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 669, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 669, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":670
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.packetLossVariance             # <<<<<<<<<<<<<<
 * 
 *     property packetThrottle:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->packetLossVariance); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 670, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":669
 *     property packetLossVariance:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetLossVariance
 * 
 */
  }

  /* "enet.pyx":668
 * 
 *     property packetLossVariance:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetLossVariance
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.packetLossVariance.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":673
 * 
 *     property packetThrottle:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottle
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_14packetThrottle_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_14packetThrottle_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_14packetThrottle___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_14packetThrottle___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":674
 *     property packetThrottle:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetThrottle
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 674, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 674, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 674, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":675
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottle             # <<<<<<<<<<<<<<
 * 
 *     property packetThrottleLimit:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->packetThrottle); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 675, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":674
 *     property packetThrottle:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetThrottle
 * 
 */
  }

  /* "enet.pyx":673
 * 
 *     property packetThrottle:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottle
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.packetThrottle.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":678
 * 
 *     property packetThrottleLimit:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleLimit
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_19packetThrottleLimit_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_19packetThrottleLimit_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_19packetThrottleLimit___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_19packetThrottleLimit___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":679
 *     property packetThrottleLimit:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetThrottleLimit
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 679, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 679, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 679, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":680
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleLimit             # <<<<<<<<<<<<<<
 * 
 *     property packetThrottleCounter:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->packetThrottleLimit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 680, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":679
 *     property packetThrottleLimit:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetThrottleLimit
 * 
 */
  }

  /* "enet.pyx":678
 * 
 *     property packetThrottleLimit:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleLimit
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.packetThrottleLimit.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":683
 * 
 *     property packetThrottleCounter:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleCounter
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_21packetThrottleCounter_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_21packetThrottleCounter_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_21packetThrottleCounter___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_21packetThrottleCounter___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":684
 *     property packetThrottleCounter:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetThrottleCounter
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 684, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 684, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 684, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":685
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleCounter             # <<<<<<<<<<<<<<
 * 
 *     property packetThrottleEpoch:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->packetThrottleCounter); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 685, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":684
 *     property packetThrottleCounter:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetThrottleCounter
 * 
 */
  }

  /* "enet.pyx":683
 * 
 *     property packetThrottleCounter:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleCounter
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.packetThrottleCounter.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":688
 * 
 *     property packetThrottleEpoch:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleEpoch
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_19packetThrottleEpoch_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_19packetThrottleEpoch_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_19packetThrottleEpoch___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_19packetThrottleEpoch___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":689
 *     property packetThrottleEpoch:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetThrottleEpoch
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 689, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 689, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 689, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":690
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleEpoch             # <<<<<<<<<<<<<<
 * 
 *     property packetThrottleAcceleration:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->packetThrottleEpoch); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 690, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":689
 *     property packetThrottleEpoch:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetThrottleEpoch
 * 
 */
  }

  /* "enet.pyx":688
 * 
 *     property packetThrottleEpoch:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleEpoch
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.packetThrottleEpoch.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":693
 * 
 *     property packetThrottleAcceleration:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleAcceleration
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_26packetThrottleAcceleration_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_26packetThrottleAcceleration_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_26packetThrottleAcceleration___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_26packetThrottleAcceleration___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":694
 *     property packetThrottleAcceleration:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetThrottleAcceleration
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 694, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 694, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 694, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":695
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleAcceleration             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, value):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->packetThrottleAcceleration); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 695, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":694
 *     property packetThrottleAcceleration:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetThrottleAcceleration
 * 
 */
  }

  /* "enet.pyx":693
 * 
 *     property packetThrottleAcceleration:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleAcceleration
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.packetThrottleAcceleration.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":697
 *                 return self._enet_peer.packetThrottleAcceleration
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 enet_peer_throttle_configure(self._enet_peer,
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_4Peer_26packetThrottleAcceleration_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6bzenet_4Peer_26packetThrottleAcceleration_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_26packetThrottleAcceleration_2__set__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_4Peer_26packetThrottleAcceleration_2__set__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  enet_uint32 __pyx_t_5;
  enet_uint32 __pyx_t_6;
  enet_uint32 __pyx_t_7;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "enet.pyx":698
 * 
 *         def __set__(self, value):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 enet_peer_throttle_configure(self._enet_peer,
 *                     self.packetThrottleInterval, value,
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 698, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 698, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 698, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":700
 *             if self.check_valid():
 *                 enet_peer_throttle_configure(self._enet_peer,
 *                     self.packetThrottleInterval, value,             # <<<<<<<<<<<<<<
 *                     self.packetThrottleDeceleration)
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_packetThrottleInterval); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 700, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyInt_As_enet_uint32(__pyx_t_1); if (unlikely((__pyx_t_5 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 700, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = __Pyx_PyInt_As_enet_uint32(__pyx_v_value); if (unlikely((__pyx_t_6 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 700, __pyx_L1_error)

    /* "enet.pyx":701
 *                 enet_peer_throttle_configure(self._enet_peer,
 *                     self.packetThrottleInterval, value,
 *                     self.packetThrottleDeceleration)             # <<<<<<<<<<<<<<
 * 
 *     property packetThrottleDeceleration:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_packetThrottleDeceleration); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 701, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PyInt_As_enet_uint32(__pyx_t_1); if (unlikely((__pyx_t_7 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 701, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "enet.pyx":699
 *         def __set__(self, value):
 *             if self.check_valid():
 *                 enet_peer_throttle_configure(self._enet_peer,             # <<<<<<<<<<<<<<
 *                     self.packetThrottleInterval, value,
 *                     self.packetThrottleDeceleration)
 */
    enet_peer_throttle_configure(__pyx_v_self->_enet_peer, __pyx_t_5, __pyx_t_6, __pyx_t_7);

    /* "enet.pyx":698
 * 
 *         def __set__(self, value):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 enet_peer_throttle_configure(self._enet_peer,
 *                     self.packetThrottleInterval, value,
 */
  }

  /* "enet.pyx":697
 *                 return self._enet_peer.packetThrottleAcceleration
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 enet_peer_throttle_configure(self._enet_peer,
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.packetThrottleAcceleration.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":704
 * 
 *     property packetThrottleDeceleration:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleDeceleration
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_26packetThrottleDeceleration_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_26packetThrottleDeceleration_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_26packetThrottleDeceleration___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_26packetThrottleDeceleration___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":705
 *     property packetThrottleDeceleration:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetThrottleDeceleration
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":706
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleDeceleration             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, value):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->packetThrottleDeceleration); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 706, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":705
 *     property packetThrottleDeceleration:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetThrottleDeceleration
 * 
 */
  }

  /* "enet.pyx":704
 * 
 *     property packetThrottleDeceleration:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleDeceleration
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.packetThrottleDeceleration.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":708
 *                 return self._enet_peer.packetThrottleDeceleration
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 enet_peer_throttle_configure(self._enet_peer,
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_4Peer_26packetThrottleDeceleration_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6bzenet_4Peer_26packetThrottleDeceleration_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_26packetThrottleDeceleration_2__set__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_4Peer_26packetThrottleDeceleration_2__set__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  enet_uint32 __pyx_t_5;
  enet_uint32 __pyx_t_6;
  enet_uint32 __pyx_t_7;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "enet.pyx":709
 * 
 *         def __set__(self, value):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 enet_peer_throttle_configure(self._enet_peer,
 *                     self.packetThrottleInterval,
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 709, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 709, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 709, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":711
 *             if self.check_valid():
 *                 enet_peer_throttle_configure(self._enet_peer,
 *                     self.packetThrottleInterval,             # <<<<<<<<<<<<<<
 *                     self.packetThrottleAcceleration, value)
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_packetThrottleInterval); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 711, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyInt_As_enet_uint32(__pyx_t_1); if (unlikely((__pyx_t_5 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 711, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "enet.pyx":712
 *                 enet_peer_throttle_configure(self._enet_peer,
 *                     self.packetThrottleInterval,
 *                     self.packetThrottleAcceleration, value)             # <<<<<<<<<<<<<<
 * 
 *     property packetThrottleInterval:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_packetThrottleAcceleration); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 712, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyInt_As_enet_uint32(__pyx_t_1); if (unlikely((__pyx_t_6 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 712, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = __Pyx_PyInt_As_enet_uint32(__pyx_v_value); if (unlikely((__pyx_t_7 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 712, __pyx_L1_error)

    /* "enet.pyx":710
 *         def __set__(self, value):
 *             if self.check_valid():
 *                 enet_peer_throttle_configure(self._enet_peer,             # <<<<<<<<<<<<<<
 *                     self.packetThrottleInterval,
 *                     self.packetThrottleAcceleration, value)
 */
    enet_peer_throttle_configure(__pyx_v_self->_enet_peer, __pyx_t_5, __pyx_t_6, __pyx_t_7);

    /* "enet.pyx":709
 * 
 *         def __set__(self, value):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 enet_peer_throttle_configure(self._enet_peer,
 *                     self.packetThrottleInterval,
 */
  }

  /* "enet.pyx":708
 *                 return self._enet_peer.packetThrottleDeceleration
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 enet_peer_throttle_configure(self._enet_peer,
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.packetThrottleDeceleration.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":715
 * 
 *     property packetThrottleInterval:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleInterval
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_22packetThrottleInterval_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_22packetThrottleInterval_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_22packetThrottleInterval___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_22packetThrottleInterval___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":716
 *     property packetThrottleInterval:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetThrottleInterval
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":717
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleInterval             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, value):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->packetThrottleInterval); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 717, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":716
 *     property packetThrottleInterval:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.packetThrottleInterval
 * 
 */
  }

  /* "enet.pyx":715
 * 
 *     property packetThrottleInterval:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.packetThrottleInterval
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.packetThrottleInterval.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":719
 *                 return self._enet_peer.packetThrottleInterval
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 enet_peer_throttle_configure(
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_4Peer_22packetThrottleInterval_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6bzenet_4Peer_22packetThrottleInterval_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_22packetThrottleInterval_2__set__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_4Peer_22packetThrottleInterval_2__set__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  enet_uint32 __pyx_t_5;
  enet_uint32 __pyx_t_6;
  enet_uint32 __pyx_t_7;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "enet.pyx":720
 * 
 *         def __set__(self, value):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 enet_peer_throttle_configure(
 *                     self._enet_peer, value, self.packetThrottleAcceleration,
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 720, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 720, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 720, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":722
 *             if self.check_valid():
 *                 enet_peer_throttle_configure(
 *                     self._enet_peer, value, self.packetThrottleAcceleration,             # <<<<<<<<<<<<<<
 *                     self.packetThrottleDeceleration)
 * 
 */
    __pyx_t_5 = __Pyx_PyInt_As_enet_uint32(__pyx_v_value); if (unlikely((__pyx_t_5 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 722, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_packetThrottleAcceleration); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 722, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyInt_As_enet_uint32(__pyx_t_1); if (unlikely((__pyx_t_6 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 722, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "enet.pyx":723
 *                 enet_peer_throttle_configure(
 *                     self._enet_peer, value, self.packetThrottleAcceleration,
 *                     self.packetThrottleDeceleration)             # <<<<<<<<<<<<<<
 * 
 *     property lastRoundTripTime:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_packetThrottleDeceleration); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 723, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PyInt_As_enet_uint32(__pyx_t_1); if (unlikely((__pyx_t_7 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 723, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "enet.pyx":721
 *         def __set__(self, value):
 *             if self.check_valid():
 *                 enet_peer_throttle_configure(             # <<<<<<<<<<<<<<
 *                     self._enet_peer, value, self.packetThrottleAcceleration,
 *                     self.packetThrottleDeceleration)
 */
    enet_peer_throttle_configure(__pyx_v_self->_enet_peer, __pyx_t_5, __pyx_t_6, __pyx_t_7);

    /* "enet.pyx":720
 * 
 *         def __set__(self, value):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 enet_peer_throttle_configure(
 *                     self._enet_peer, value, self.packetThrottleAcceleration,
 */
  }

  /* "enet.pyx":719
 *                 return self._enet_peer.packetThrottleInterval
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 enet_peer_throttle_configure(
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.packetThrottleInterval.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":726
 * 
 *     property lastRoundTripTime:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.lastRoundTripTime
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_17lastRoundTripTime_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_17lastRoundTripTime_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_17lastRoundTripTime___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_17lastRoundTripTime___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":727
 *     property lastRoundTripTime:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.lastRoundTripTime
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 727, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 727, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 727, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":728
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.lastRoundTripTime             # <<<<<<<<<<<<<<
 * 
 *     property lowestRoundTripTime:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->lastRoundTripTime); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":727
 *     property lastRoundTripTime:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.lastRoundTripTime
 * 
 */
  }

  /* "enet.pyx":726
 * 
 *     property lastRoundTripTime:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.lastRoundTripTime
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.lastRoundTripTime.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":731
 * 
 *     property lowestRoundTripTime:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.lowestRoundTripTime
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_19lowestRoundTripTime_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_19lowestRoundTripTime_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_19lowestRoundTripTime___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_19lowestRoundTripTime___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":732
 *     property lowestRoundTripTime:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.lowestRoundTripTime
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 732, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 732, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 732, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":733
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.lowestRoundTripTime             # <<<<<<<<<<<<<<
 * 
 *     property lastRoundTripTimeVariance:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->lowestRoundTripTime); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 733, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":732
 *     property lowestRoundTripTime:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.lowestRoundTripTime
 * 
 */
  }

  /* "enet.pyx":731
 * 
 *     property lowestRoundTripTime:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.lowestRoundTripTime
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.lowestRoundTripTime.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":736
 * 
 *     property lastRoundTripTimeVariance:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.lastRoundTripTimeVariance
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_25lastRoundTripTimeVariance_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_25lastRoundTripTimeVariance_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_25lastRoundTripTimeVariance___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_25lastRoundTripTimeVariance___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":737
 *     property lastRoundTripTimeVariance:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.lastRoundTripTimeVariance
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 737, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 737, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 737, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":738
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.lastRoundTripTimeVariance             # <<<<<<<<<<<<<<
 * 
 *     property highestRoundTripTimeVariance:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->lastRoundTripTimeVariance); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 738, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":737
 *     property lastRoundTripTimeVariance:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.lastRoundTripTimeVariance
 * 
 */
  }

  /* "enet.pyx":736
 * 
 *     property lastRoundTripTimeVariance:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.lastRoundTripTimeVariance
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.lastRoundTripTimeVariance.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":741
 * 
 *     property highestRoundTripTimeVariance:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.highestRoundTripTimeVariance
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_28highestRoundTripTimeVariance_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_28highestRoundTripTimeVariance_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_28highestRoundTripTimeVariance___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_28highestRoundTripTimeVariance___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":742
 *     property highestRoundTripTimeVariance:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.highestRoundTripTimeVariance
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 742, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":743
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.highestRoundTripTimeVariance             # <<<<<<<<<<<<<<
 * 
 *     property roundTripTime:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->highestRoundTripTimeVariance); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 743, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":742
 *     property highestRoundTripTimeVariance:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.highestRoundTripTimeVariance
 * 
 */
  }

  /* "enet.pyx":741
 * 
 *     property highestRoundTripTimeVariance:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.highestRoundTripTimeVariance
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.highestRoundTripTimeVariance.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":746
 * 
 *     property roundTripTime:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.roundTripTime
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_13roundTripTime_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_13roundTripTime_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_13roundTripTime___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_13roundTripTime___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":747
 *     property roundTripTime:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.roundTripTime
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 747, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 747, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 747, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":748
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.roundTripTime             # <<<<<<<<<<<<<<
 * 
 *     property roundTripTimeVariance:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->roundTripTime); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 748, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":747
 *     property roundTripTime:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.roundTripTime
 * 
 */
  }

  /* "enet.pyx":746
 * 
 *     property roundTripTime:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.roundTripTime
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.roundTripTime.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":751
 * 
 *     property roundTripTimeVariance:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.roundTripTimeVariance
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_21roundTripTimeVariance_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_21roundTripTimeVariance_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_21roundTripTimeVariance___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_21roundTripTimeVariance___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":752
 *     property roundTripTimeVariance:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.roundTripTimeVariance
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 752, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 752, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 752, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":753
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.roundTripTimeVariance             # <<<<<<<<<<<<<<
 * 
 *     property mtu:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->roundTripTimeVariance); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 753, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":752
 *     property roundTripTimeVariance:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.roundTripTimeVariance
 * 
 */
  }

  /* "enet.pyx":751
 * 
 *     property roundTripTimeVariance:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.roundTripTimeVariance
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.roundTripTimeVariance.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":756
 * 
 *     property mtu:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.mtu
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_3mtu_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_3mtu_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_3mtu___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_3mtu___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":757
 *     property mtu:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.mtu
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 757, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 757, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 757, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":758
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.mtu             # <<<<<<<<<<<<<<
 * 
 *     property windowSize:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->mtu); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 758, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":757
 *     property mtu:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.mtu
 * 
 */
  }

  /* "enet.pyx":756
 * 
 *     property mtu:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.mtu
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.mtu.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":761
 * 
 *     property windowSize:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.windowSize
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_10windowSize_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_10windowSize_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_10windowSize___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_10windowSize___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":762
 *     property windowSize:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.windowSize
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 762, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 762, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 762, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":763
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.windowSize             # <<<<<<<<<<<<<<
 * 
 *     property reliableDataInTransit:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->windowSize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 763, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":762
 *     property windowSize:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.windowSize
 * 
 */
  }

  /* "enet.pyx":761
 * 
 *     property windowSize:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.windowSize
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.windowSize.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":766
 * 
 *     property reliableDataInTransit:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.reliableDataInTransit
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_21reliableDataInTransit_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_21reliableDataInTransit_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_21reliableDataInTransit___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_21reliableDataInTransit___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":767
 *     property reliableDataInTransit:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.reliableDataInTransit
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 767, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 767, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 767, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":768
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.reliableDataInTransit             # <<<<<<<<<<<<<<
 * 
 *     property outgoingReliableSequenceNumber:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->reliableDataInTransit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 768, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":767
 *     property reliableDataInTransit:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.reliableDataInTransit
 * 
 */
  }

  /* "enet.pyx":766
 * 
 *     property reliableDataInTransit:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.reliableDataInTransit
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.reliableDataInTransit.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":771
 * 
 *     property outgoingReliableSequenceNumber:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingReliableSequenceNumber
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_30outgoingReliableSequenceNumber_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_30outgoingReliableSequenceNumber_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_30outgoingReliableSequenceNumber___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_30outgoingReliableSequenceNumber___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":772
 *     property outgoingReliableSequenceNumber:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.outgoingReliableSequenceNumber
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 772, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 772, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 772, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":773
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingReliableSequenceNumber             # <<<<<<<<<<<<<<
 * 
 *     property needsDispatch:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint16(__pyx_v_self->_enet_peer->outgoingReliableSequenceNumber); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 773, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":772
 *     property outgoingReliableSequenceNumber:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.outgoingReliableSequenceNumber
 * 
 */
  }

  /* "enet.pyx":771
 * 
 *     property outgoingReliableSequenceNumber:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingReliableSequenceNumber
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.outgoingReliableSequenceNumber.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":776
 * 
 *     property needsDispatch:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.needsDispatch
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_13needsDispatch_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_13needsDispatch_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_13needsDispatch___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_13needsDispatch___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":777
 *     property needsDispatch:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.needsDispatch
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":778
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.needsDispatch             # <<<<<<<<<<<<<<
 * 
 *     property incomingUnsequencedGroup:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_enet_peer->needsDispatch); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 778, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":777
 *     property needsDispatch:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.needsDispatch
 * 
 */
  }

  /* "enet.pyx":776
 * 
 *     property needsDispatch:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.needsDispatch
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.needsDispatch.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":781
 * 
 *     property incomingUnsequencedGroup:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.incomingUnsequencedGroup
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_24incomingUnsequencedGroup_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_24incomingUnsequencedGroup_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_24incomingUnsequencedGroup___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_24incomingUnsequencedGroup___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":782
 *     property incomingUnsequencedGroup:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.incomingUnsequencedGroup
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 782, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 782, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 782, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":783
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.incomingUnsequencedGroup             # <<<<<<<<<<<<<<
 * 
 *     property outgoingUnsequencedGroup:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint16(__pyx_v_self->_enet_peer->incomingUnsequencedGroup); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 783, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":782
 *     property incomingUnsequencedGroup:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.incomingUnsequencedGroup
 * 
 */
  }

  /* "enet.pyx":781
 * 
 *     property incomingUnsequencedGroup:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.incomingUnsequencedGroup
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.incomingUnsequencedGroup.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":786
 * 
 *     property outgoingUnsequencedGroup:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingUnsequencedGroup
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_24outgoingUnsequencedGroup_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_24outgoingUnsequencedGroup_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_24outgoingUnsequencedGroup___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_24outgoingUnsequencedGroup___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":787
 *     property outgoingUnsequencedGroup:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.outgoingUnsequencedGroup
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 787, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 787, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 787, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":788
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingUnsequencedGroup             # <<<<<<<<<<<<<<
 * 
 *     property eventData:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint16(__pyx_v_self->_enet_peer->outgoingUnsequencedGroup); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 788, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":787
 *     property outgoingUnsequencedGroup:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.outgoingUnsequencedGroup
 * 
 */
  }

  /* "enet.pyx":786
 * 
 *     property outgoingUnsequencedGroup:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.outgoingUnsequencedGroup
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.outgoingUnsequencedGroup.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":791
 * 
 *     property eventData:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.eventData
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_9eventData_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_9eventData_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_9eventData___get__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_9eventData___get__(struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":792
 *     property eventData:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.eventData
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_valid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 792, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 792, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 792, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "enet.pyx":793
 *         def __get__(self):
 *             if self.check_valid():
 *                 return self._enet_peer.eventData             # <<<<<<<<<<<<<<
 * 
 * cdef class Event:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_peer->eventData); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 793, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "enet.pyx":792
 *     property eventData:
 *         def __get__(self):
 *             if self.check_valid():             # <<<<<<<<<<<<<<
 *                 return self._enet_peer.eventData
 * 
 */
  }

  /* "enet.pyx":791
 * 
 *     property eventData:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.check_valid():
 *                 return self._enet_peer.eventData
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Peer.eventData.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._enet_peer cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_23__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_23__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_22__reduce_cython__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_22__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Peer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._enet_peer cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._enet_peer cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._enet_peer cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Peer.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._enet_peer cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._enet_peer cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Peer_25__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Peer_25__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Peer_24__setstate_cython__(((struct __pyx_obj_6bzenet_Peer *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Peer_24__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Peer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self._enet_peer cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._enet_peer cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._enet_peer cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._enet_peer cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Peer.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":816
 *     cdef Packet _packet
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self._packet = None
 * 
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_5Event_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6bzenet_5Event_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 0))) return -1;
  __pyx_r = __pyx_pf_6bzenet_5Event___init__(((struct __pyx_obj_6bzenet_Event *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_5Event___init__(struct __pyx_obj_6bzenet_Event *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "enet.pyx":817
 * 
 *     def __init__(self):
 *         self._packet = None             # <<<<<<<<<<<<<<
 * 
 *     property type:
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_packet);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_packet));
  __pyx_v_self->_packet = ((struct __pyx_obj_6bzenet_Packet *)Py_None);

  /* "enet.pyx":816
 *     cdef Packet _packet
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self._packet = None
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":820
 * 
 *     property type:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_event.type
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_5Event_4type_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_5Event_4type_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_5Event_4type___get__(((struct __pyx_obj_6bzenet_Event *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_5Event_4type___get__(struct __pyx_obj_6bzenet_Event *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":821
 *     property type:
 *         def __get__(self):
 *             return self._enet_event.type             # <<<<<<<<<<<<<<
 * 
 *     property peer:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_ENetEventType(__pyx_v_self->_enet_event.type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 821, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "enet.pyx":820
 * 
 *     property type:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_event.type
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Event.type.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":824
 * 
 *     property peer:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             peer = Peer()
 *             (<Peer> peer)._enet_peer = self._enet_event.peer
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_5Event_4peer_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_5Event_4peer_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_5Event_4peer___get__(((struct __pyx_obj_6bzenet_Event *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_5Event_4peer___get__(struct __pyx_obj_6bzenet_Event *__pyx_v_self) {
  struct __pyx_obj_6bzenet_Peer *__pyx_v_peer = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  ENetPeer *__pyx_t_2;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":825
 *     property peer:
 *         def __get__(self):
 *             peer = Peer()             # <<<<<<<<<<<<<<
 *             (<Peer> peer)._enet_peer = self._enet_event.peer
 *             return peer
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_6bzenet_Peer)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 825, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_peer = ((struct __pyx_obj_6bzenet_Peer *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "enet.pyx":826
 *         def __get__(self):
 *             peer = Peer()
 *             (<Peer> peer)._enet_peer = self._enet_event.peer             # <<<<<<<<<<<<<<
 *             return peer
 * 
 */
  __pyx_t_2 = __pyx_v_self->_enet_event.peer;
  __pyx_v_peer->_enet_peer = __pyx_t_2;

  /* "enet.pyx":827
 *             peer = Peer()
 *             (<Peer> peer)._enet_peer = self._enet_event.peer
 *             return peer             # <<<<<<<<<<<<<<
 * 
 *     property channelID:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_peer));
  __pyx_r = ((PyObject *)__pyx_v_peer);
  goto __pyx_L0;

  /* "enet.pyx":824
 * 
 *     property peer:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             peer = Peer()
 *             (<Peer> peer)._enet_peer = self._enet_event.peer
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Event.peer.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_peer);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":830
 * 
 *     property channelID:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_event.channelID
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_5Event_9channelID_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_5Event_9channelID_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_5Event_9channelID___get__(((struct __pyx_obj_6bzenet_Event *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_5Event_9channelID___get__(struct __pyx_obj_6bzenet_Event *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":831
 *     property channelID:
 *         def __get__(self):
 *             return self._enet_event.channelID             # <<<<<<<<<<<<<<
 * 
 *     property data:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_enet_uint8(__pyx_v_self->_enet_event.channelID); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "enet.pyx":830
 * 
 *     property channelID:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_event.channelID
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Event.channelID.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":834
 * 
 *     property data:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_event.data
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_5Event_4data_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_5Event_4data_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_5Event_4data___get__(((struct __pyx_obj_6bzenet_Event *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_5Event_4data___get__(struct __pyx_obj_6bzenet_Event *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":835
 *     property data:
 *         def __get__(self):
 *             return self._enet_event.data             # <<<<<<<<<<<<<<
 * 
 *     property packet:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_event.data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 835, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "enet.pyx":834
 * 
 *     property data:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_event.data
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Event.data.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":838
 * 
 *     property packet:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if not self._packet:
 *                 self._packet = Packet()
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_5Event_6packet_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_5Event_6packet_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_5Event_6packet___get__(((struct __pyx_obj_6bzenet_Event *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_5Event_6packet___get__(struct __pyx_obj_6bzenet_Event *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  ENetPacket *__pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":839
 *     property packet:
 *         def __get__(self):
 *             if not self._packet:             # <<<<<<<<<<<<<<
 *                 self._packet = Packet()
 *                 (<Packet> self._packet)._enet_packet = self._enet_event.packet
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->_packet)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 839, __pyx_L1_error)
  __pyx_t_2 = ((!__pyx_t_1) != 0);
  if (__pyx_t_2) {

    /* "enet.pyx":840
 *         def __get__(self):
 *             if not self._packet:
 *                 self._packet = Packet()             # <<<<<<<<<<<<<<
 *                 (<Packet> self._packet)._enet_packet = self._enet_event.packet
 *             return self._packet
 */
    __pyx_t_3 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_6bzenet_Packet)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 840, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->_packet);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->_packet));
    __pyx_v_self->_packet = ((struct __pyx_obj_6bzenet_Packet *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "enet.pyx":841
 *             if not self._packet:
 *                 self._packet = Packet()
 *                 (<Packet> self._packet)._enet_packet = self._enet_event.packet             # <<<<<<<<<<<<<<
 *             return self._packet
 * 
 */
    __pyx_t_4 = __pyx_v_self->_enet_event.packet;
    __pyx_v_self->_packet->_enet_packet = __pyx_t_4;

    /* "enet.pyx":839
 *     property packet:
 *         def __get__(self):
 *             if not self._packet:             # <<<<<<<<<<<<<<
 *                 self._packet = Packet()
 *                 (<Packet> self._packet)._enet_packet = self._enet_event.packet
 */
  }

  /* "enet.pyx":842
 *                 self._packet = Packet()
 *                 (<Packet> self._packet)._enet_packet = self._enet_event.packet
 *             return self._packet             # <<<<<<<<<<<<<<
 * 
 * from weakref import WeakValueDictionary
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_packet));
  __pyx_r = ((PyObject *)__pyx_v_self->_packet);
  goto __pyx_L0;

  /* "enet.pyx":838
 * 
 *     property packet:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if not self._packet:
 *                 self._packet = Packet()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("bzenet.Event.packet.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._enet_event cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_5Event_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6bzenet_5Event_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_5Event_2__reduce_cython__(((struct __pyx_obj_6bzenet_Event *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_5Event_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Event *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._enet_event cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._enet_event cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._enet_event cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Event.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._enet_event cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._enet_event cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_5Event_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6bzenet_5Event_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_5Event_4__setstate_cython__(((struct __pyx_obj_6bzenet_Event *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_5Event_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Event *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self._enet_event cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._enet_event cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._enet_event cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._enet_event cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Event.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":871
 *     cdef object __weakref__
 * 
 *     def __init__ (self, Address address=None, peerCount=0, channelLimit=0,             # <<<<<<<<<<<<<<
 *         incomingBandwidth=0, outgoingBandwidth=0):
 * 
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_4Host_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6bzenet_4Host_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_6bzenet_Address *__pyx_v_address = 0;
  PyObject *__pyx_v_peerCount = 0;
  PyObject *__pyx_v_channelLimit = 0;
  PyObject *__pyx_v_incomingBandwidth = 0;
  PyObject *__pyx_v_outgoingBandwidth = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_address,&__pyx_n_s_peerCount,&__pyx_n_s_channelLimit,&__pyx_n_s_incomingBandwidth,&__pyx_n_s_outgoingBandwidth,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[0] = (PyObject *)((struct __pyx_obj_6bzenet_Address *)Py_None);
    values[1] = ((PyObject *)__pyx_int_0);
    values[2] = ((PyObject *)__pyx_int_0);
    values[3] = ((PyObject *)__pyx_int_0);
    values[4] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_address);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_peerCount);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_channelLimit);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_incomingBandwidth);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_outgoingBandwidth);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 871, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_address = ((struct __pyx_obj_6bzenet_Address *)values[0]);
    __pyx_v_peerCount = values[1];
    __pyx_v_channelLimit = values[2];
    __pyx_v_incomingBandwidth = values[3];
    __pyx_v_outgoingBandwidth = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 871, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzenet.Host.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_address), __pyx_ptype_6bzenet_Address, 1, "address", 0))) __PYX_ERR(0, 871, __pyx_L1_error)
  __pyx_r = __pyx_pf_6bzenet_4Host___init__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self), __pyx_v_address, __pyx_v_peerCount, __pyx_v_channelLimit, __pyx_v_incomingBandwidth, __pyx_v_outgoingBandwidth);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_4Host___init__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, struct __pyx_obj_6bzenet_Address *__pyx_v_address, PyObject *__pyx_v_peerCount, PyObject *__pyx_v_channelLimit, PyObject *__pyx_v_incomingBandwidth, PyObject *__pyx_v_outgoingBandwidth) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  enet_uint32 __pyx_t_4;
  enet_uint32 __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "enet.pyx":874
 *         incomingBandwidth=0, outgoingBandwidth=0):
 * 
 *         if address:             # <<<<<<<<<<<<<<
 *             self._enet_host = enet_host_create(&address._enet_address, peerCount, channelLimit, incomingBandwidth, outgoingBandwidth)
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_address)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 874, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "enet.pyx":875
 * 
 *         if address:
 *             self._enet_host = enet_host_create(&address._enet_address, peerCount, channelLimit, incomingBandwidth, outgoingBandwidth)             # <<<<<<<<<<<<<<
 *         else:
 *             self._enet_host = enet_host_create(NULL, peerCount, channelLimit, incomingBandwidth, outgoingBandwidth)
 */
    __pyx_t_2 = __Pyx_PyInt_As_size_t(__pyx_v_peerCount); if (unlikely((__pyx_t_2 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 875, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_v_channelLimit); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 875, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyInt_As_enet_uint32(__pyx_v_incomingBandwidth); if (unlikely((__pyx_t_4 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 875, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyInt_As_enet_uint32(__pyx_v_outgoingBandwidth); if (unlikely((__pyx_t_5 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 875, __pyx_L1_error)
    __pyx_v_self->_enet_host = enet_host_create((&__pyx_v_address->_enet_address), __pyx_t_2, __pyx_t_3, __pyx_t_4, __pyx_t_5);

    /* "enet.pyx":874
 *         incomingBandwidth=0, outgoingBandwidth=0):
 * 
 *         if address:             # <<<<<<<<<<<<<<
 *             self._enet_host = enet_host_create(&address._enet_address, peerCount, channelLimit, incomingBandwidth, outgoingBandwidth)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "enet.pyx":877
 *             self._enet_host = enet_host_create(&address._enet_address, peerCount, channelLimit, incomingBandwidth, outgoingBandwidth)
 *         else:
 *             self._enet_host = enet_host_create(NULL, peerCount, channelLimit, incomingBandwidth, outgoingBandwidth)             # <<<<<<<<<<<<<<
 * 
 *         if not self._enet_host:
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_v_peerCount); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 877, __pyx_L1_error)
    __pyx_t_2 = __Pyx_PyInt_As_size_t(__pyx_v_channelLimit); if (unlikely((__pyx_t_2 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 877, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyInt_As_enet_uint32(__pyx_v_incomingBandwidth); if (unlikely((__pyx_t_5 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 877, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyInt_As_enet_uint32(__pyx_v_outgoingBandwidth); if (unlikely((__pyx_t_4 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 877, __pyx_L1_error)
    __pyx_v_self->_enet_host = enet_host_create(NULL, __pyx_t_3, __pyx_t_2, __pyx_t_5, __pyx_t_4);
  }
  __pyx_L3:;

  /* "enet.pyx":879
 *             self._enet_host = enet_host_create(NULL, peerCount, channelLimit, incomingBandwidth, outgoingBandwidth)
 * 
 *         if not self._enet_host:             # <<<<<<<<<<<<<<
 *             raise MemoryError("Unable to create host structure!")
 *         self.dealloc = True
 */
  __pyx_t_1 = ((!(__pyx_v_self->_enet_host != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "enet.pyx":880
 * 
 *         if not self._enet_host:
 *             raise MemoryError("Unable to create host structure!")             # <<<<<<<<<<<<<<
 *         self.dealloc = True
 * 
 */
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 880, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 880, __pyx_L1_error)

    /* "enet.pyx":879
 *             self._enet_host = enet_host_create(NULL, peerCount, channelLimit, incomingBandwidth, outgoingBandwidth)
 * 
 *         if not self._enet_host:             # <<<<<<<<<<<<<<
 *             raise MemoryError("Unable to create host structure!")
 *         self.dealloc = True
 */
  }

  /* "enet.pyx":881
 *         if not self._enet_host:
 *             raise MemoryError("Unable to create host structure!")
 *         self.dealloc = True             # <<<<<<<<<<<<<<
 * 
 *         global host_static_instances
 */
  __Pyx_INCREF(Py_True);
  __Pyx_GIVEREF(Py_True);
  __Pyx_GOTREF(__pyx_v_self->dealloc);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->dealloc));
  __pyx_v_self->dealloc = ((PyBoolObject *)Py_True);

  /* "enet.pyx":884
 * 
 *         global host_static_instances
 *         host_static_instances[<uintptr_t>self._enet_host] = self             # <<<<<<<<<<<<<<
 * 
 *     def __hash__(self):
 */
  if (unlikely(__Pyx_SetItemInt(__pyx_v_6bzenet_host_static_instances, ((uintptr_t)__pyx_v_self->_enet_host), ((PyObject *)__pyx_v_self), uintptr_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1) < 0)) __PYX_ERR(0, 884, __pyx_L1_error)

  /* "enet.pyx":871
 *     cdef object __weakref__
 * 
 *     def __init__ (self, Address address=None, peerCount=0, channelLimit=0,             # <<<<<<<<<<<<<<
 *         incomingBandwidth=0, outgoingBandwidth=0):
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("bzenet.Host.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":886
 *         host_static_instances[<uintptr_t>self._enet_host] = self
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         return <uintptr_t>self._enet_host
 * 
 */

/* Python wrapper */
static Py_hash_t __pyx_pw_6bzenet_4Host_3__hash__(PyObject *__pyx_v_self); /*proto*/
static Py_hash_t __pyx_pw_6bzenet_4Host_3__hash__(PyObject *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__hash__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_2__hash__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_hash_t __pyx_pf_6bzenet_4Host_2__hash__(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__hash__", 0);

  /* "enet.pyx":887
 * 
 *     def __hash__(self):
 *         return <uintptr_t>self._enet_host             # <<<<<<<<<<<<<<
 * 
 *     def __cinit__(self):
 */
  __pyx_r = ((uintptr_t)__pyx_v_self->_enet_host);
  goto __pyx_L0;

  /* "enet.pyx":886
 *         host_static_instances[<uintptr_t>self._enet_host] = self
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         return <uintptr_t>self._enet_host
 * 
 */

  /* function exit code */
  __pyx_L0:;
  if (unlikely(__pyx_r == -1) && !PyErr_Occurred()) __pyx_r = -2;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":889
 *         return <uintptr_t>self._enet_host
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         self.dealloc = False
 *         self._enet_host = NULL
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_4Host_5__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6bzenet_4Host_5__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_6bzenet_4Host_4__cinit__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_4Host_4__cinit__(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "enet.pyx":890
 * 
 *     def __cinit__(self):
 *         self.dealloc = False             # <<<<<<<<<<<<<<
 *         self._enet_host = NULL
 * 
 */
  __Pyx_INCREF(Py_False);
  __Pyx_GIVEREF(Py_False);
  __Pyx_GOTREF(__pyx_v_self->dealloc);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->dealloc));
  __pyx_v_self->dealloc = ((PyBoolObject *)Py_False);

  /* "enet.pyx":891
 *     def __cinit__(self):
 *         self.dealloc = False
 *         self._enet_host = NULL             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_v_self->_enet_host = NULL;

  /* "enet.pyx":889
 *         return <uintptr_t>self._enet_host
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         self.dealloc = False
 *         self._enet_host = NULL
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":893
 *         self._enet_host = NULL
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.dealloc:
 *             enet_host_destroy(self._enet_host)
 */

/* Python wrapper */
static void __pyx_pw_6bzenet_4Host_7__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_6bzenet_4Host_7__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_6bzenet_4Host_6__dealloc__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_6bzenet_4Host_6__dealloc__(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "enet.pyx":894
 * 
 *     def __dealloc__(self):
 *         if self.dealloc:             # <<<<<<<<<<<<<<
 *             enet_host_destroy(self._enet_host)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->dealloc)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 894, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "enet.pyx":895
 *     def __dealloc__(self):
 *         if self.dealloc:
 *             enet_host_destroy(self._enet_host)             # <<<<<<<<<<<<<<
 * 
 *     def connect(self, Address address, channelCount, data=0):
 */
    enet_host_destroy(__pyx_v_self->_enet_host);

    /* "enet.pyx":894
 * 
 *     def __dealloc__(self):
 *         if self.dealloc:             # <<<<<<<<<<<<<<
 *             enet_host_destroy(self._enet_host)
 * 
 */
  }

  /* "enet.pyx":893
 *         self._enet_host = NULL
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.dealloc:
 *             enet_host_destroy(self._enet_host)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("bzenet.Host.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "enet.pyx":897
 *             enet_host_destroy(self._enet_host)
 * 
 *     def connect(self, Address address, channelCount, data=0):             # <<<<<<<<<<<<<<
 *         """
 *         Peer connect (Address address, int channelCount, int data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_9connect(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6bzenet_4Host_8connect[] = "\n        Peer connect (Address address, int channelCount, int data)\n\n        Initiates a connection to a foreign host and returns a Peer.\n        ";
static PyObject *__pyx_pw_6bzenet_4Host_9connect(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_6bzenet_Address *__pyx_v_address = 0;
  PyObject *__pyx_v_channelCount = 0;
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("connect (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_address,&__pyx_n_s_channelCount,&__pyx_n_s_data,0};
    PyObject* values[3] = {0,0,0};
    values[2] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_address)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_channelCount)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("connect", 0, 2, 3, 1); __PYX_ERR(0, 897, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "connect") < 0)) __PYX_ERR(0, 897, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_address = ((struct __pyx_obj_6bzenet_Address *)values[0]);
    __pyx_v_channelCount = values[1];
    __pyx_v_data = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("connect", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 897, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzenet.Host.connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_address), __pyx_ptype_6bzenet_Address, 1, "address", 0))) __PYX_ERR(0, 897, __pyx_L1_error)
  __pyx_r = __pyx_pf_6bzenet_4Host_8connect(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self), __pyx_v_address, __pyx_v_channelCount, __pyx_v_data);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_8connect(struct __pyx_obj_6bzenet_Host *__pyx_v_self, struct __pyx_obj_6bzenet_Address *__pyx_v_address, PyObject *__pyx_v_channelCount, PyObject *__pyx_v_data) {
  struct __pyx_obj_6bzenet_Peer *__pyx_v_peer = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  enet_uint32 __pyx_t_4;
  __Pyx_RefNannySetupContext("connect", 0);

  /* "enet.pyx":904
 *         """
 * 
 *         if self._enet_host:             # <<<<<<<<<<<<<<
 *             peer = Peer()
 *             (<Peer> peer)._enet_peer = enet_host_connect(
 */
  __pyx_t_1 = (__pyx_v_self->_enet_host != 0);
  if (__pyx_t_1) {

    /* "enet.pyx":905
 * 
 *         if self._enet_host:
 *             peer = Peer()             # <<<<<<<<<<<<<<
 *             (<Peer> peer)._enet_peer = enet_host_connect(
 *                 self._enet_host, &address._enet_address, channelCount, data)
 */
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_6bzenet_Peer)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 905, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_peer = ((struct __pyx_obj_6bzenet_Peer *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "enet.pyx":907
 *             peer = Peer()
 *             (<Peer> peer)._enet_peer = enet_host_connect(
 *                 self._enet_host, &address._enet_address, channelCount, data)             # <<<<<<<<<<<<<<
 * 
 *             if not (<Peer> peer)._enet_peer:
 */
    __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_v_channelCount); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 907, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyInt_As_enet_uint32(__pyx_v_data); if (unlikely((__pyx_t_4 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 907, __pyx_L1_error)

    /* "enet.pyx":906
 *         if self._enet_host:
 *             peer = Peer()
 *             (<Peer> peer)._enet_peer = enet_host_connect(             # <<<<<<<<<<<<<<
 *                 self._enet_host, &address._enet_address, channelCount, data)
 * 
 */
    __pyx_v_peer->_enet_peer = enet_host_connect(__pyx_v_self->_enet_host, (&__pyx_v_address->_enet_address), __pyx_t_3, __pyx_t_4);

    /* "enet.pyx":909
 *                 self._enet_host, &address._enet_address, channelCount, data)
 * 
 *             if not (<Peer> peer)._enet_peer:             # <<<<<<<<<<<<<<
 *                 raise IOError("Connection failure!")
 * 
 */
    __pyx_t_1 = ((!(__pyx_v_peer->_enet_peer != 0)) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "enet.pyx":910
 * 
 *             if not (<Peer> peer)._enet_peer:
 *                 raise IOError("Connection failure!")             # <<<<<<<<<<<<<<
 * 
 *             return peer
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_IOError, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 910, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 910, __pyx_L1_error)

      /* "enet.pyx":909
 *                 self._enet_host, &address._enet_address, channelCount, data)
 * 
 *             if not (<Peer> peer)._enet_peer:             # <<<<<<<<<<<<<<
 *                 raise IOError("Connection failure!")
 * 
 */
    }

    /* "enet.pyx":912
 *                 raise IOError("Connection failure!")
 * 
 *             return peer             # <<<<<<<<<<<<<<
 * 
 *     def check_events(self):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_peer));
    __pyx_r = ((PyObject *)__pyx_v_peer);
    goto __pyx_L0;

    /* "enet.pyx":904
 *         """
 * 
 *         if self._enet_host:             # <<<<<<<<<<<<<<
 *             peer = Peer()
 *             (<Peer> peer)._enet_peer = enet_host_connect(
 */
  }

  /* "enet.pyx":897
 *             enet_host_destroy(self._enet_host)
 * 
 *     def connect(self, Address address, channelCount, data=0):             # <<<<<<<<<<<<<<
 *         """
 *         Peer connect (Address address, int channelCount, int data)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("bzenet.Host.connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_peer);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":914
 *             return peer
 * 
 *     def check_events(self):             # <<<<<<<<<<<<<<
 *         """
 *         Checks for any queued events on the host and dispatches one if available
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_11check_events(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6bzenet_4Host_10check_events[] = "\n        Checks for any queued events on the host and dispatches one if available\n        ";
static PyObject *__pyx_pw_6bzenet_4Host_11check_events(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("check_events (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_10check_events(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_10check_events(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  struct __pyx_obj_6bzenet_Event *__pyx_v_event = NULL;
  int __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("check_events", 0);

  /* "enet.pyx":919
 *         """
 * 
 *         if self._enet_host:             # <<<<<<<<<<<<<<
 *             event = Event()
 *             result = enet_host_check_events(
 */
  __pyx_t_1 = (__pyx_v_self->_enet_host != 0);
  if (__pyx_t_1) {

    /* "enet.pyx":920
 * 
 *         if self._enet_host:
 *             event = Event()             # <<<<<<<<<<<<<<
 *             result = enet_host_check_events(
 *                 self._enet_host, &(<Event> event)._enet_event)
 */
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_6bzenet_Event)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 920, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_event = ((struct __pyx_obj_6bzenet_Event *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "enet.pyx":921
 *         if self._enet_host:
 *             event = Event()
 *             result = enet_host_check_events(             # <<<<<<<<<<<<<<
 *                 self._enet_host, &(<Event> event)._enet_event)
 * 
 */
    __pyx_v_result = enet_host_check_events(__pyx_v_self->_enet_host, (&__pyx_v_event->_enet_event));

    /* "enet.pyx":924
 *                 self._enet_host, &(<Event> event)._enet_event)
 * 
 *             if result < 0:             # <<<<<<<<<<<<<<
 *                 raise IOError("Servicing error - probably disconnected.")
 *             elif result == 0:
 */
    __pyx_t_1 = ((__pyx_v_result < 0) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "enet.pyx":925
 * 
 *             if result < 0:
 *                 raise IOError("Servicing error - probably disconnected.")             # <<<<<<<<<<<<<<
 *             elif result == 0:
 *                 return None
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_IOError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 925, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 925, __pyx_L1_error)

      /* "enet.pyx":924
 *                 self._enet_host, &(<Event> event)._enet_event)
 * 
 *             if result < 0:             # <<<<<<<<<<<<<<
 *                 raise IOError("Servicing error - probably disconnected.")
 *             elif result == 0:
 */
    }

    /* "enet.pyx":926
 *             if result < 0:
 *                 raise IOError("Servicing error - probably disconnected.")
 *             elif result == 0:             # <<<<<<<<<<<<<<
 *                 return None
 *             else:
 */
    __pyx_t_1 = ((__pyx_v_result == 0) != 0);
    if (__pyx_t_1) {

      /* "enet.pyx":927
 *                 raise IOError("Servicing error - probably disconnected.")
 *             elif result == 0:
 *                 return None             # <<<<<<<<<<<<<<
 *             else:
 *                 return event
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      goto __pyx_L0;

      /* "enet.pyx":926
 *             if result < 0:
 *                 raise IOError("Servicing error - probably disconnected.")
 *             elif result == 0:             # <<<<<<<<<<<<<<
 *                 return None
 *             else:
 */
    }

    /* "enet.pyx":929
 *                 return None
 *             else:
 *                 return event             # <<<<<<<<<<<<<<
 * 
 *     def service(self, timeout, fast_drop=False):
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(((PyObject *)__pyx_v_event));
      __pyx_r = ((PyObject *)__pyx_v_event);
      goto __pyx_L0;
    }

    /* "enet.pyx":919
 *         """
 * 
 *         if self._enet_host:             # <<<<<<<<<<<<<<
 *             event = Event()
 *             result = enet_host_check_events(
 */
  }

  /* "enet.pyx":914
 *             return peer
 * 
 *     def check_events(self):             # <<<<<<<<<<<<<<
 *         """
 *         Checks for any queued events on the host and dispatches one if available
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("bzenet.Host.check_events", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_event);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":931
 *                 return event
 * 
 *     def service(self, timeout, fast_drop=False):             # <<<<<<<<<<<<<<
 *         """
 *         Event service (int timeout)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_13service(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6bzenet_4Host_12service[] = "\n        Event service (int timeout)\n\n        Waits for events on the host specified and shuttles packets between\n        the host and its peers. The timeout is in milliseconds.\n\n        if fast_drop is set, None can be returned instead\n        ";
static PyObject *__pyx_pw_6bzenet_4Host_13service(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_timeout = 0;
  PyObject *__pyx_v_fast_drop = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("service (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_timeout,&__pyx_n_s_fast_drop,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_timeout)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fast_drop);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "service") < 0)) __PYX_ERR(0, 931, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_timeout = values[0];
    __pyx_v_fast_drop = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("service", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 931, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzenet.Host.service", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6bzenet_4Host_12service(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self), __pyx_v_timeout, __pyx_v_fast_drop);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_12service(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_timeout, PyObject *__pyx_v_fast_drop) {
  struct __pyx_obj_6bzenet_Event *__pyx_v_event = NULL;
  int __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  enet_uint32 __pyx_t_3;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("service", 0);

  /* "enet.pyx":941
 *         """
 * 
 *         if self._enet_host:             # <<<<<<<<<<<<<<
 *             event = Event()
 *             result = enet_host_service(
 */
  __pyx_t_1 = (__pyx_v_self->_enet_host != 0);
  if (__pyx_t_1) {

    /* "enet.pyx":942
 * 
 *         if self._enet_host:
 *             event = Event()             # <<<<<<<<<<<<<<
 *             result = enet_host_service(
 *                 self._enet_host, &(<Event> event)._enet_event, timeout)
 */
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_6bzenet_Event)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 942, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_event = ((struct __pyx_obj_6bzenet_Event *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "enet.pyx":944
 *             event = Event()
 *             result = enet_host_service(
 *                 self._enet_host, &(<Event> event)._enet_event, timeout)             # <<<<<<<<<<<<<<
 * 
 *             if result < 0:
 */
    __pyx_t_3 = __Pyx_PyInt_As_enet_uint32(__pyx_v_timeout); if (unlikely((__pyx_t_3 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 944, __pyx_L1_error)

    /* "enet.pyx":943
 *         if self._enet_host:
 *             event = Event()
 *             result = enet_host_service(             # <<<<<<<<<<<<<<
 *                 self._enet_host, &(<Event> event)._enet_event, timeout)
 * 
 */
    __pyx_v_result = enet_host_service(__pyx_v_self->_enet_host, (&__pyx_v_event->_enet_event), __pyx_t_3);

    /* "enet.pyx":946
 *                 self._enet_host, &(<Event> event)._enet_event, timeout)
 * 
 *             if result < 0:             # <<<<<<<<<<<<<<
 *                 raise IOError("Servicing error - probably disconnected.")
 *             if result == 0 and fast_drop:
 */
    __pyx_t_1 = ((__pyx_v_result < 0) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "enet.pyx":947
 * 
 *             if result < 0:
 *                 raise IOError("Servicing error - probably disconnected.")             # <<<<<<<<<<<<<<
 *             if result == 0 and fast_drop:
 *                 return None
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_IOError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 947, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 947, __pyx_L1_error)

      /* "enet.pyx":946
 *                 self._enet_host, &(<Event> event)._enet_event, timeout)
 * 
 *             if result < 0:             # <<<<<<<<<<<<<<
 *                 raise IOError("Servicing error - probably disconnected.")
 *             if result == 0 and fast_drop:
 */
    }

    /* "enet.pyx":948
 *             if result < 0:
 *                 raise IOError("Servicing error - probably disconnected.")
 *             if result == 0 and fast_drop:             # <<<<<<<<<<<<<<
 *                 return None
 *             else:
 */
    __pyx_t_4 = ((__pyx_v_result == 0) != 0);
    if (__pyx_t_4) {
    } else {
      __pyx_t_1 = __pyx_t_4;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_fast_drop); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 948, __pyx_L1_error)
    __pyx_t_1 = __pyx_t_4;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_1) {

      /* "enet.pyx":949
 *                 raise IOError("Servicing error - probably disconnected.")
 *             if result == 0 and fast_drop:
 *                 return None             # <<<<<<<<<<<<<<
 *             else:
 *                 return event
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      goto __pyx_L0;

      /* "enet.pyx":948
 *             if result < 0:
 *                 raise IOError("Servicing error - probably disconnected.")
 *             if result == 0 and fast_drop:             # <<<<<<<<<<<<<<
 *                 return None
 *             else:
 */
    }

    /* "enet.pyx":951
 *                 return None
 *             else:
 *                 return event             # <<<<<<<<<<<<<<
 * 
 *     def flush(self):
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(((PyObject *)__pyx_v_event));
      __pyx_r = ((PyObject *)__pyx_v_event);
      goto __pyx_L0;
    }

    /* "enet.pyx":941
 *         """
 * 
 *         if self._enet_host:             # <<<<<<<<<<<<<<
 *             event = Event()
 *             result = enet_host_service(
 */
  }

  /* "enet.pyx":931
 *                 return event
 * 
 *     def service(self, timeout, fast_drop=False):             # <<<<<<<<<<<<<<
 *         """
 *         Event service (int timeout)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("bzenet.Host.service", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_event);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":953
 *                 return event
 * 
 *     def flush(self):             # <<<<<<<<<<<<<<
 *         """
 *         flush ()
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_15flush(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6bzenet_4Host_14flush[] = "\n        flush ()\n\n        Sends any queued packets on the host specified to its designated peers.\n        ";
static PyObject *__pyx_pw_6bzenet_4Host_15flush(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("flush (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_14flush(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_14flush(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("flush", 0);

  /* "enet.pyx":960
 *         """
 * 
 *         if self._enet_host:             # <<<<<<<<<<<<<<
 *             enet_host_flush(self._enet_host)
 * 
 */
  __pyx_t_1 = (__pyx_v_self->_enet_host != 0);
  if (__pyx_t_1) {

    /* "enet.pyx":961
 * 
 *         if self._enet_host:
 *             enet_host_flush(self._enet_host)             # <<<<<<<<<<<<<<
 * 
 *     def broadcast(self, channelID, Packet packet):
 */
    enet_host_flush(__pyx_v_self->_enet_host);

    /* "enet.pyx":960
 *         """
 * 
 *         if self._enet_host:             # <<<<<<<<<<<<<<
 *             enet_host_flush(self._enet_host)
 * 
 */
  }

  /* "enet.pyx":953
 *                 return event
 * 
 *     def flush(self):             # <<<<<<<<<<<<<<
 *         """
 *         flush ()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":963
 *             enet_host_flush(self._enet_host)
 * 
 *     def broadcast(self, channelID, Packet packet):             # <<<<<<<<<<<<<<
 *         """
 *         broadcast (int channelID, Packet packet)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_17broadcast(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6bzenet_4Host_16broadcast[] = "\n        broadcast (int channelID, Packet packet)\n\n        Queues a packet to be sent to all peers associated with the host.\n        ";
static PyObject *__pyx_pw_6bzenet_4Host_17broadcast(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_channelID = 0;
  struct __pyx_obj_6bzenet_Packet *__pyx_v_packet = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("broadcast (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_channelID,&__pyx_n_s_packet,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_channelID)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_packet)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("broadcast", 1, 2, 2, 1); __PYX_ERR(0, 963, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "broadcast") < 0)) __PYX_ERR(0, 963, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_channelID = values[0];
    __pyx_v_packet = ((struct __pyx_obj_6bzenet_Packet *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("broadcast", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 963, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzenet.Host.broadcast", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_packet), __pyx_ptype_6bzenet_Packet, 1, "packet", 0))) __PYX_ERR(0, 963, __pyx_L1_error)
  __pyx_r = __pyx_pf_6bzenet_4Host_16broadcast(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self), __pyx_v_channelID, __pyx_v_packet);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_16broadcast(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_channelID, struct __pyx_obj_6bzenet_Packet *__pyx_v_packet) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  enet_uint8 __pyx_t_5;
  __Pyx_RefNannySetupContext("broadcast", 0);

  /* "enet.pyx":970
 *         """
 * 
 *         if self._enet_host:             # <<<<<<<<<<<<<<
 *             if packet.is_valid():
 *                 packet.sent = True
 */
  __pyx_t_1 = (__pyx_v_self->_enet_host != 0);
  if (__pyx_t_1) {

    /* "enet.pyx":971
 * 
 *         if self._enet_host:
 *             if packet.is_valid():             # <<<<<<<<<<<<<<
 *                 packet.sent = True
 *                 enet_host_broadcast(self._enet_host, channelID, packet._enet_packet)
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_packet), __pyx_n_s_is_valid); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 971, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 971, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 971, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_1) {

      /* "enet.pyx":972
 *         if self._enet_host:
 *             if packet.is_valid():
 *                 packet.sent = True             # <<<<<<<<<<<<<<
 *                 enet_host_broadcast(self._enet_host, channelID, packet._enet_packet)
 * 
 */
      __Pyx_INCREF(Py_True);
      __Pyx_GIVEREF(Py_True);
      __Pyx_GOTREF(__pyx_v_packet->sent);
      __Pyx_DECREF(((PyObject *)__pyx_v_packet->sent));
      __pyx_v_packet->sent = ((PyBoolObject *)Py_True);

      /* "enet.pyx":973
 *             if packet.is_valid():
 *                 packet.sent = True
 *                 enet_host_broadcast(self._enet_host, channelID, packet._enet_packet)             # <<<<<<<<<<<<<<
 * 
 *     def compress_with_range_coder(self):
 */
      __pyx_t_5 = __Pyx_PyInt_As_enet_uint8(__pyx_v_channelID); if (unlikely((__pyx_t_5 == ((enet_uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 973, __pyx_L1_error)
      enet_host_broadcast(__pyx_v_self->_enet_host, __pyx_t_5, __pyx_v_packet->_enet_packet);

      /* "enet.pyx":971
 * 
 *         if self._enet_host:
 *             if packet.is_valid():             # <<<<<<<<<<<<<<
 *                 packet.sent = True
 *                 enet_host_broadcast(self._enet_host, channelID, packet._enet_packet)
 */
    }

    /* "enet.pyx":970
 *         """
 * 
 *         if self._enet_host:             # <<<<<<<<<<<<<<
 *             if packet.is_valid():
 *                 packet.sent = True
 */
  }

  /* "enet.pyx":963
 *             enet_host_flush(self._enet_host)
 * 
 *     def broadcast(self, channelID, Packet packet):             # <<<<<<<<<<<<<<
 *         """
 *         broadcast (int channelID, Packet packet)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("bzenet.Host.broadcast", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":975
 *                 enet_host_broadcast(self._enet_host, channelID, packet._enet_packet)
 * 
 *     def compress_with_range_coder(self):             # <<<<<<<<<<<<<<
 *         """
 *         Sets the packet compressor the host should use to the default range coder
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_19compress_with_range_coder(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6bzenet_4Host_18compress_with_range_coder[] = "\n        Sets the packet compressor the host should use to the default range coder\n        ";
static PyObject *__pyx_pw_6bzenet_4Host_19compress_with_range_coder(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compress_with_range_coder (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_18compress_with_range_coder(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_18compress_with_range_coder(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("compress_with_range_coder", 0);

  /* "enet.pyx":980
 *         """
 * 
 *         if self._enet_host:             # <<<<<<<<<<<<<<
 *             return enet_host_compress_with_range_coder(self._enet_host);
 * 
 */
  __pyx_t_1 = (__pyx_v_self->_enet_host != 0);
  if (__pyx_t_1) {

    /* "enet.pyx":981
 * 
 *         if self._enet_host:
 *             return enet_host_compress_with_range_coder(self._enet_host);             # <<<<<<<<<<<<<<
 * 
 *     property socket:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_int(enet_host_compress_with_range_coder(__pyx_v_self->_enet_host)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 981, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "enet.pyx":980
 *         """
 * 
 *         if self._enet_host:             # <<<<<<<<<<<<<<
 *             return enet_host_compress_with_range_coder(self._enet_host);
 * 
 */
  }

  /* "enet.pyx":975
 *                 enet_host_broadcast(self._enet_host, channelID, packet._enet_packet)
 * 
 *     def compress_with_range_coder(self):             # <<<<<<<<<<<<<<
 *         """
 *         Sets the packet compressor the host should use to the default range coder
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("bzenet.Host.compress_with_range_coder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":984
 * 
 *     property socket:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             socket = Socket()
 *             (<Socket> socket)._enet_socket = self._enet_host.socket
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_6socket_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Host_6socket_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_6socket___get__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_6socket___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  struct __pyx_obj_6bzenet_Socket *__pyx_v_socket = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  ENetSocket __pyx_t_2;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":985
 *     property socket:
 *         def __get__(self):
 *             socket = Socket()             # <<<<<<<<<<<<<<
 *             (<Socket> socket)._enet_socket = self._enet_host.socket
 *             return socket
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_6bzenet_Socket)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 985, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_socket = ((struct __pyx_obj_6bzenet_Socket *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "enet.pyx":986
 *         def __get__(self):
 *             socket = Socket()
 *             (<Socket> socket)._enet_socket = self._enet_host.socket             # <<<<<<<<<<<<<<
 *             return socket
 * 
 */
  __pyx_t_2 = __pyx_v_self->_enet_host->socket;
  __pyx_v_socket->_enet_socket = __pyx_t_2;

  /* "enet.pyx":987
 *             socket = Socket()
 *             (<Socket> socket)._enet_socket = self._enet_host.socket
 *             return socket             # <<<<<<<<<<<<<<
 * 
 *     property address:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_socket));
  __pyx_r = ((PyObject *)__pyx_v_socket);
  goto __pyx_L0;

  /* "enet.pyx":984
 * 
 *     property socket:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             socket = Socket()
 *             (<Socket> socket)._enet_socket = self._enet_host.socket
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Host.socket.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_socket);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":990
 * 
 *     property address:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self._enet_host:
 *                 a = Address(None, 0)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_7address_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Host_7address_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_7address___get__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_7address___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  struct __pyx_obj_6bzenet_Address *__pyx_v_a = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  ENetAddress __pyx_t_3;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":991
 *     property address:
 *         def __get__(self):
 *             if self._enet_host:             # <<<<<<<<<<<<<<
 *                 a = Address(None, 0)
 *                 (<Address> a)._enet_address = self._enet_host.address
 */
  __pyx_t_1 = (__pyx_v_self->_enet_host != 0);
  if (__pyx_t_1) {

    /* "enet.pyx":992
 *         def __get__(self):
 *             if self._enet_host:
 *                 a = Address(None, 0)             # <<<<<<<<<<<<<<
 *                 (<Address> a)._enet_address = self._enet_host.address
 *                 return a
 */
    __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6bzenet_Address), __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 992, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_a = ((struct __pyx_obj_6bzenet_Address *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "enet.pyx":993
 *             if self._enet_host:
 *                 a = Address(None, 0)
 *                 (<Address> a)._enet_address = self._enet_host.address             # <<<<<<<<<<<<<<
 *                 return a
 * 
 */
    __pyx_t_3 = __pyx_v_self->_enet_host->address;
    __pyx_v_a->_enet_address = __pyx_t_3;

    /* "enet.pyx":994
 *                 a = Address(None, 0)
 *                 (<Address> a)._enet_address = self._enet_host.address
 *                 return a             # <<<<<<<<<<<<<<
 * 
 *     property incomingBandwidth:
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_a));
    __pyx_r = ((PyObject *)__pyx_v_a);
    goto __pyx_L0;

    /* "enet.pyx":991
 *     property address:
 *         def __get__(self):
 *             if self._enet_host:             # <<<<<<<<<<<<<<
 *                 a = Address(None, 0)
 *                 (<Address> a)._enet_address = self._enet_host.address
 */
  }

  /* "enet.pyx":990
 * 
 *     property address:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self._enet_host:
 *                 a = Address(None, 0)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("bzenet.Host.address.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_a);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":997
 * 
 *     property incomingBandwidth:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_host.incomingBandwidth
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_17incomingBandwidth_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Host_17incomingBandwidth_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_17incomingBandwidth___get__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_17incomingBandwidth___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":998
 *     property incomingBandwidth:
 *         def __get__(self):
 *             return self._enet_host.incomingBandwidth             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_host->incomingBandwidth); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 998, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "enet.pyx":997
 * 
 *     property incomingBandwidth:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_host.incomingBandwidth
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Host.incomingBandwidth.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1000
 *             return self._enet_host.incomingBandwidth
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             enet_host_bandwidth_limit(self._enet_host,
 *                 value, self.outgoingBandwidth)
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_4Host_17incomingBandwidth_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6bzenet_4Host_17incomingBandwidth_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_17incomingBandwidth_2__set__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_4Host_17incomingBandwidth_2__set__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  enet_uint32 __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  enet_uint32 __pyx_t_3;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "enet.pyx":1002
 *         def __set__(self, value):
 *             enet_host_bandwidth_limit(self._enet_host,
 *                 value, self.outgoingBandwidth)             # <<<<<<<<<<<<<<
 * 
 *     property outgoingBandwidth:
 */
  __pyx_t_1 = __Pyx_PyInt_As_enet_uint32(__pyx_v_value); if (unlikely((__pyx_t_1 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1002, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_outgoingBandwidth); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1002, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_enet_uint32(__pyx_t_2); if (unlikely((__pyx_t_3 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1002, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "enet.pyx":1001
 * 
 *         def __set__(self, value):
 *             enet_host_bandwidth_limit(self._enet_host,             # <<<<<<<<<<<<<<
 *                 value, self.outgoingBandwidth)
 * 
 */
  enet_host_bandwidth_limit(__pyx_v_self->_enet_host, __pyx_t_1, __pyx_t_3);

  /* "enet.pyx":1000
 *             return self._enet_host.incomingBandwidth
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             enet_host_bandwidth_limit(self._enet_host,
 *                 value, self.outgoingBandwidth)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("bzenet.Host.incomingBandwidth.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1005
 * 
 *     property outgoingBandwidth:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_host.outgoingBandwidth
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_17outgoingBandwidth_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Host_17outgoingBandwidth_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_17outgoingBandwidth___get__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_17outgoingBandwidth___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":1006
 *     property outgoingBandwidth:
 *         def __get__(self):
 *             return self._enet_host.outgoingBandwidth             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_host->outgoingBandwidth); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1006, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "enet.pyx":1005
 * 
 *     property outgoingBandwidth:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_host.outgoingBandwidth
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Host.outgoingBandwidth.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1008
 *             return self._enet_host.outgoingBandwidth
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             enet_host_bandwidth_limit(self._enet_host,
 *                 self.incomingBandwidth, value)
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_4Host_17outgoingBandwidth_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6bzenet_4Host_17outgoingBandwidth_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_17outgoingBandwidth_2__set__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_4Host_17outgoingBandwidth_2__set__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  enet_uint32 __pyx_t_2;
  enet_uint32 __pyx_t_3;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "enet.pyx":1010
 *         def __set__(self, value):
 *             enet_host_bandwidth_limit(self._enet_host,
 *                 self.incomingBandwidth, value)             # <<<<<<<<<<<<<<
 * 
 *     property peers:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_incomingBandwidth); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1010, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_enet_uint32(__pyx_t_1); if (unlikely((__pyx_t_2 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1010, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyInt_As_enet_uint32(__pyx_v_value); if (unlikely((__pyx_t_3 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1010, __pyx_L1_error)

  /* "enet.pyx":1009
 * 
 *         def __set__(self, value):
 *             enet_host_bandwidth_limit(self._enet_host,             # <<<<<<<<<<<<<<
 *                 self.incomingBandwidth, value)
 * 
 */
  enet_host_bandwidth_limit(__pyx_v_self->_enet_host, __pyx_t_2, __pyx_t_3);

  /* "enet.pyx":1008
 *             return self._enet_host.outgoingBandwidth
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             enet_host_bandwidth_limit(self._enet_host,
 *                 self.incomingBandwidth, value)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Host.outgoingBandwidth.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1013
 * 
 *     property peers:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             cdef size_t i
 *             peers = []
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_5peers_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Host_5peers_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_5peers___get__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_5peers___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  size_t __pyx_v_i;
  PyObject *__pyx_v_peers = NULL;
  struct __pyx_obj_6bzenet_Peer *__pyx_v_peer = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  size_t __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":1015
 *         def __get__(self):
 *             cdef size_t i
 *             peers = []             # <<<<<<<<<<<<<<
 *             for i from 0 <= i < self.peerCount:
 *                 peer = Peer()
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1015, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_peers = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "enet.pyx":1016
 *             cdef size_t i
 *             peers = []
 *             for i from 0 <= i < self.peerCount:             # <<<<<<<<<<<<<<
 *                 peer = Peer()
 *                 (<Peer> peer)._enet_peer = &self._enet_host.peers[i]
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_peerCount); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1016, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_2 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1016, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_2; __pyx_v_i++) {

    /* "enet.pyx":1017
 *             peers = []
 *             for i from 0 <= i < self.peerCount:
 *                 peer = Peer()             # <<<<<<<<<<<<<<
 *                 (<Peer> peer)._enet_peer = &self._enet_host.peers[i]
 *                 peers.append(peer)
 */
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_6bzenet_Peer)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1017, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_peer, ((struct __pyx_obj_6bzenet_Peer *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "enet.pyx":1018
 *             for i from 0 <= i < self.peerCount:
 *                 peer = Peer()
 *                 (<Peer> peer)._enet_peer = &self._enet_host.peers[i]             # <<<<<<<<<<<<<<
 *                 peers.append(peer)
 *             return peers
 */
    __pyx_v_peer->_enet_peer = (&(__pyx_v_self->_enet_host->peers[__pyx_v_i]));

    /* "enet.pyx":1019
 *                 peer = Peer()
 *                 (<Peer> peer)._enet_peer = &self._enet_host.peers[i]
 *                 peers.append(peer)             # <<<<<<<<<<<<<<
 *             return peers
 * 
 */
    __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_peers, ((PyObject *)__pyx_v_peer)); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 1019, __pyx_L1_error)
  }

  /* "enet.pyx":1020
 *                 (<Peer> peer)._enet_peer = &self._enet_host.peers[i]
 *                 peers.append(peer)
 *             return peers             # <<<<<<<<<<<<<<
 * 
 *     property peerCount:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_peers);
  __pyx_r = __pyx_v_peers;
  goto __pyx_L0;

  /* "enet.pyx":1013
 * 
 *     property peers:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             cdef size_t i
 *             peers = []
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Host.peers.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_peers);
  __Pyx_XDECREF((PyObject *)__pyx_v_peer);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1023
 * 
 *     property peerCount:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_host.peerCount
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_9peerCount_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Host_9peerCount_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_9peerCount___get__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_9peerCount___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":1024
 *     property peerCount:
 *         def __get__(self):
 *             return self._enet_host.peerCount             # <<<<<<<<<<<<<<
 * 
 *     property channelLimit:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->_enet_host->peerCount); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1024, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "enet.pyx":1023
 * 
 *     property peerCount:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_host.peerCount
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Host.peerCount.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1027
 * 
 *     property channelLimit:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_host.channelLimit
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_12channelLimit_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Host_12channelLimit_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_12channelLimit___get__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_12channelLimit___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":1028
 *     property channelLimit:
 *         def __get__(self):
 *             return self._enet_host.channelLimit             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->_enet_host->channelLimit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1028, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "enet.pyx":1027
 * 
 *     property channelLimit:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_host.channelLimit
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Host.channelLimit.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1030
 *             return self._enet_host.channelLimit
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             enet_host_channel_limit(self._enet_host, value)
 * 
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_4Host_12channelLimit_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6bzenet_4Host_12channelLimit_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_12channelLimit_2__set__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_4Host_12channelLimit_2__set__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "enet.pyx":1031
 * 
 *         def __set__(self, value):
 *             enet_host_channel_limit(self._enet_host, value)             # <<<<<<<<<<<<<<
 * 
 *     property totalSentData:
 */
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1031, __pyx_L1_error)
  enet_host_channel_limit(__pyx_v_self->_enet_host, __pyx_t_1);

  /* "enet.pyx":1030
 *             return self._enet_host.channelLimit
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             enet_host_channel_limit(self._enet_host, value)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("bzenet.Host.channelLimit.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1034
 * 
 *     property totalSentData:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_host.totalSentData
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_13totalSentData_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Host_13totalSentData_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_13totalSentData___get__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_13totalSentData___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":1035
 *     property totalSentData:
 *         def __get__(self):
 *             return self._enet_host.totalSentData             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_host->totalSentData); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1035, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "enet.pyx":1034
 * 
 *     property totalSentData:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_host.totalSentData
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Host.totalSentData.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1037
 *             return self._enet_host.totalSentData
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self._enet_host.totalSentData = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_4Host_13totalSentData_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6bzenet_4Host_13totalSentData_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_13totalSentData_2__set__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_4Host_13totalSentData_2__set__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  enet_uint32 __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "enet.pyx":1038
 * 
 *         def __set__(self, value):
 *             self._enet_host.totalSentData = value             # <<<<<<<<<<<<<<
 * 
 *     property totalSentPackets:
 */
  __pyx_t_1 = __Pyx_PyInt_As_enet_uint32(__pyx_v_value); if (unlikely((__pyx_t_1 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1038, __pyx_L1_error)
  __pyx_v_self->_enet_host->totalSentData = __pyx_t_1;

  /* "enet.pyx":1037
 *             return self._enet_host.totalSentData
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self._enet_host.totalSentData = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("bzenet.Host.totalSentData.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1041
 * 
 *     property totalSentPackets:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_host.totalSentPackets
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_16totalSentPackets_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Host_16totalSentPackets_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_16totalSentPackets___get__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_16totalSentPackets___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":1042
 *     property totalSentPackets:
 *         def __get__(self):
 *             return self._enet_host.totalSentPackets             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_host->totalSentPackets); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1042, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "enet.pyx":1041
 * 
 *     property totalSentPackets:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_host.totalSentPackets
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Host.totalSentPackets.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1044
 *             return self._enet_host.totalSentPackets
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self._enet_host.totalSentPackets = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_4Host_16totalSentPackets_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6bzenet_4Host_16totalSentPackets_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_16totalSentPackets_2__set__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_4Host_16totalSentPackets_2__set__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  enet_uint32 __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "enet.pyx":1045
 * 
 *         def __set__(self, value):
 *             self._enet_host.totalSentPackets = value             # <<<<<<<<<<<<<<
 * 
 *     property totalReceivedData:
 */
  __pyx_t_1 = __Pyx_PyInt_As_enet_uint32(__pyx_v_value); if (unlikely((__pyx_t_1 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1045, __pyx_L1_error)
  __pyx_v_self->_enet_host->totalSentPackets = __pyx_t_1;

  /* "enet.pyx":1044
 *             return self._enet_host.totalSentPackets
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self._enet_host.totalSentPackets = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("bzenet.Host.totalSentPackets.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1048
 * 
 *     property totalReceivedData:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_host.totalReceivedData
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_17totalReceivedData_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Host_17totalReceivedData_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_17totalReceivedData___get__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_17totalReceivedData___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":1049
 *     property totalReceivedData:
 *         def __get__(self):
 *             return self._enet_host.totalReceivedData             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_host->totalReceivedData); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1049, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "enet.pyx":1048
 * 
 *     property totalReceivedData:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_host.totalReceivedData
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Host.totalReceivedData.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1051
 *             return self._enet_host.totalReceivedData
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self._enet_host.totalReceivedData = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_4Host_17totalReceivedData_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6bzenet_4Host_17totalReceivedData_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_17totalReceivedData_2__set__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_4Host_17totalReceivedData_2__set__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  enet_uint32 __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "enet.pyx":1052
 * 
 *         def __set__(self, value):
 *             self._enet_host.totalReceivedData = value             # <<<<<<<<<<<<<<
 * 
 *     property totalReceivedPackets:
 */
  __pyx_t_1 = __Pyx_PyInt_As_enet_uint32(__pyx_v_value); if (unlikely((__pyx_t_1 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1052, __pyx_L1_error)
  __pyx_v_self->_enet_host->totalReceivedData = __pyx_t_1;

  /* "enet.pyx":1051
 *             return self._enet_host.totalReceivedData
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self._enet_host.totalReceivedData = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("bzenet.Host.totalReceivedData.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1055
 * 
 *     property totalReceivedPackets:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_host.totalReceivedPackets
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_20totalReceivedPackets_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Host_20totalReceivedPackets_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_20totalReceivedPackets___get__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_20totalReceivedPackets___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":1056
 *     property totalReceivedPackets:
 *         def __get__(self):
 *             return self._enet_host.totalReceivedPackets             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_enet_uint32(__pyx_v_self->_enet_host->totalReceivedPackets); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1056, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "enet.pyx":1055
 * 
 *     property totalReceivedPackets:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._enet_host.totalReceivedPackets
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Host.totalReceivedPackets.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1058
 *             return self._enet_host.totalReceivedPackets
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self._enet_host.totalReceivedPackets = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_4Host_20totalReceivedPackets_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6bzenet_4Host_20totalReceivedPackets_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_20totalReceivedPackets_2__set__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_4Host_20totalReceivedPackets_2__set__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  enet_uint32 __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "enet.pyx":1059
 * 
 *         def __set__(self, value):
 *             self._enet_host.totalReceivedPackets = value             # <<<<<<<<<<<<<<
 * 
 *     property intercept:
 */
  __pyx_t_1 = __Pyx_PyInt_As_enet_uint32(__pyx_v_value); if (unlikely((__pyx_t_1 == ((enet_uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1059, __pyx_L1_error)
  __pyx_v_self->_enet_host->totalReceivedPackets = __pyx_t_1;

  /* "enet.pyx":1058
 *             return self._enet_host.totalReceivedPackets
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self._enet_host.totalReceivedPackets = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("bzenet.Host.totalReceivedPackets.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1062
 * 
 *     property intercept:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._interceptCallback
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_9intercept_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Host_9intercept_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_9intercept___get__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_9intercept___get__(struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "enet.pyx":1063
 *     property intercept:
 *         def __get__(self):
 *             return self._interceptCallback             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_interceptCallback);
  __pyx_r = __pyx_v_self->_interceptCallback;
  goto __pyx_L0;

  /* "enet.pyx":1062
 * 
 *     property intercept:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._interceptCallback
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1065
 *             return self._interceptCallback
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             if value is None:
 *                 self._enet_host.intercept = NULL
 */

/* Python wrapper */
static int __pyx_pw_6bzenet_4Host_9intercept_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6bzenet_4Host_9intercept_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_9intercept_2__set__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6bzenet_4Host_9intercept_2__set__(struct __pyx_obj_6bzenet_Host *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "enet.pyx":1066
 * 
 *         def __set__(self, value):
 *             if value is None:             # <<<<<<<<<<<<<<
 *                 self._enet_host.intercept = NULL
 *             else:
 */
  __pyx_t_1 = (__pyx_v_value == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "enet.pyx":1067
 *         def __set__(self, value):
 *             if value is None:
 *                 self._enet_host.intercept = NULL             # <<<<<<<<<<<<<<
 *             else:
 *                 self._enet_host.intercept = intercept_callback
 */
    __pyx_v_self->_enet_host->intercept = NULL;

    /* "enet.pyx":1066
 * 
 *         def __set__(self, value):
 *             if value is None:             # <<<<<<<<<<<<<<
 *                 self._enet_host.intercept = NULL
 *             else:
 */
    goto __pyx_L3;
  }

  /* "enet.pyx":1069
 *                 self._enet_host.intercept = NULL
 *             else:
 *                 self._enet_host.intercept = intercept_callback             # <<<<<<<<<<<<<<
 *             self._interceptCallback = value
 * 
 */
  /*else*/ {
    __pyx_v_self->_enet_host->intercept = __pyx_f_6bzenet_intercept_callback;
  }
  __pyx_L3:;

  /* "enet.pyx":1070
 *             else:
 *                 self._enet_host.intercept = intercept_callback
 *             self._interceptCallback = value             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->_interceptCallback);
  __Pyx_DECREF(__pyx_v_self->_interceptCallback);
  __pyx_v_self->_interceptCallback = __pyx_v_value;

  /* "enet.pyx":1065
 *             return self._interceptCallback
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             if value is None:
 *                 self._enet_host.intercept = NULL
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_21__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Host_21__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_20__reduce_cython__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_20__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Host *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Host.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_4Host_23__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6bzenet_4Host_23__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet_4Host_22__setstate_cython__(((struct __pyx_obj_6bzenet_Host *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_4Host_22__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6bzenet_Host *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzenet.Host.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1073
 * 
 * 
 * cdef int __cdecl intercept_callback(ENetHost *host, ENetEvent *event) except -1:             # <<<<<<<<<<<<<<
 *     cdef Address address = Address(None, 0)
 *     address._enet_address = host.receivedAddress
 */

static int __cdecl __pyx_f_6bzenet_intercept_callback(ENetHost *__pyx_v_host, CYTHON_UNUSED ENetEvent *__pyx_v_event) {
  struct __pyx_obj_6bzenet_Address *__pyx_v_address = 0;
  PyObject *__pyx_v_ret = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  ENetAddress __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("intercept_callback", 0);

  /* "enet.pyx":1074
 * 
 * cdef int __cdecl intercept_callback(ENetHost *host, ENetEvent *event) except -1:
 *     cdef Address address = Address(None, 0)             # <<<<<<<<<<<<<<
 *     address._enet_address = host.receivedAddress
 *     cdef object ret = None
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6bzenet_Address), __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1074, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_address = ((struct __pyx_obj_6bzenet_Address *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "enet.pyx":1075
 * cdef int __cdecl intercept_callback(ENetHost *host, ENetEvent *event) except -1:
 *     cdef Address address = Address(None, 0)
 *     address._enet_address = host.receivedAddress             # <<<<<<<<<<<<<<
 *     cdef object ret = None
 * 
 */
  __pyx_t_2 = __pyx_v_host->receivedAddress;
  __pyx_v_address->_enet_address = __pyx_t_2;

  /* "enet.pyx":1076
 *     cdef Address address = Address(None, 0)
 *     address._enet_address = host.receivedAddress
 *     cdef object ret = None             # <<<<<<<<<<<<<<
 * 
 *     if <uintptr_t>host in host_static_instances:
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_ret = Py_None;

  /* "enet.pyx":1078
 *     cdef object ret = None
 * 
 *     if <uintptr_t>host in host_static_instances:             # <<<<<<<<<<<<<<
 *         ret = host_static_instances[<uintptr_t>host].intercept(address, (<char*>host.receivedData)[:host.receivedDataLength])
 *     return int(bool(ret))
 */
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(((uintptr_t)__pyx_v_host)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1078, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_v_6bzenet_host_static_instances, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 1078, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {

    /* "enet.pyx":1079
 * 
 *     if <uintptr_t>host in host_static_instances:
 *         ret = host_static_instances[<uintptr_t>host].intercept(address, (<char*>host.receivedData)[:host.receivedDataLength])             # <<<<<<<<<<<<<<
 *     return int(bool(ret))
 * 
 */
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_v_6bzenet_host_static_instances, ((uintptr_t)__pyx_v_host), uintptr_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1079, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_intercept); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1079, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyBytes_FromStringAndSize(((char *)__pyx_v_host->receivedData) + 0, __pyx_v_host->receivedDataLength - 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1079, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[3] = {__pyx_t_7, ((PyObject *)__pyx_v_address), __pyx_t_5};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1079, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[3] = {__pyx_t_7, ((PyObject *)__pyx_v_address), __pyx_t_5};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1079, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_9 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1079, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (__pyx_t_7) {
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
      }
      __Pyx_INCREF(((PyObject *)__pyx_v_address));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_address));
      PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, ((PyObject *)__pyx_v_address));
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1079, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_ret, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "enet.pyx":1078
 *     cdef object ret = None
 * 
 *     if <uintptr_t>host in host_static_instances:             # <<<<<<<<<<<<<<
 *         ret = host_static_instances[<uintptr_t>host].intercept(address, (<char*>host.receivedData)[:host.receivedDataLength])
 *     return int(bool(ret))
 */
  }

  /* "enet.pyx":1080
 *     if <uintptr_t>host in host_static_instances:
 *         ret = host_static_instances[<uintptr_t>host].intercept(address, (<char*>host.receivedData)[:host.receivedDataLength])
 *     return int(bool(ret))             # <<<<<<<<<<<<<<
 * 
 * def _enet_atexit():
 */
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_7cpython_4bool_bool), __pyx_v_ret); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1080, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyNumber_Int(__pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1080, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_t_6); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1080, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_r = __pyx_t_8;
  goto __pyx_L0;

  /* "enet.pyx":1073
 * 
 * 
 * cdef int __cdecl intercept_callback(ENetHost *host, ENetEvent *event) except -1:             # <<<<<<<<<<<<<<
 *     cdef Address address = Address(None, 0)
 *     address._enet_address = host.receivedAddress
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("bzenet.intercept_callback", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_address);
  __Pyx_XDECREF(__pyx_v_ret);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "enet.pyx":1082
 *     return int(bool(ret))
 * 
 * def _enet_atexit():             # <<<<<<<<<<<<<<
 *     enet_deinitialize()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_1_enet_atexit(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_6bzenet_1_enet_atexit = {"_enet_atexit", (PyCFunction)__pyx_pw_6bzenet_1_enet_atexit, METH_NOARGS, 0};
static PyObject *__pyx_pw_6bzenet_1_enet_atexit(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_enet_atexit (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzenet__enet_atexit(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet__enet_atexit(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_enet_atexit", 0);

  /* "enet.pyx":1083
 * 
 * def _enet_atexit():
 *     enet_deinitialize()             # <<<<<<<<<<<<<<
 * 
 * enet_initialize()
 */
  enet_deinitialize();

  /* "enet.pyx":1082
 *     return int(bool(ret))
 * 
 * def _enet_atexit():             # <<<<<<<<<<<<<<
 *     enet_deinitialize()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_Socket(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzenet_3__pyx_unpickle_Socket(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_6bzenet_3__pyx_unpickle_Socket = {"__pyx_unpickle_Socket", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6bzenet_3__pyx_unpickle_Socket, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6bzenet_3__pyx_unpickle_Socket(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_Socket (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Socket", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Socket", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_Socket") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Socket", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzenet.__pyx_unpickle_Socket", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6bzenet_2__pyx_unpickle_Socket(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzenet_2__pyx_unpickle_Socket(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Socket", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xd719157:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd719157 = (_enet_socket))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xd719157) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xd719157:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd719157 = (_enet_socket))" % __pyx_checksum)
 *     __pyx_result = Socket.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xd719157:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd719157 = (_enet_socket))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = Socket.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xd7, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xd719157:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd719157 = (_enet_socket))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd719157 = (_enet_socket))" % __pyx_checksum)
 *     __pyx_result = Socket.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Socket__set_state(<Socket> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6bzenet_Socket), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd719157 = (_enet_socket))" % __pyx_checksum)
 *     __pyx_result = Socket.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Socket__set_state(<Socket> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = Socket.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Socket__set_state(<Socket> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Socket__set_state(Socket __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6bzenet___pyx_unpickle_Socket__set_state(((struct __pyx_obj_6bzenet_Socket *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd719157 = (_enet_socket))" % __pyx_checksum)
 *     __pyx_result = Socket.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Socket__set_state(<Socket> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Socket__set_state(<Socket> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Socket__set_state(Socket __pyx_result, tuple __pyx_state):
 *     __pyx_result._enet_socket = __pyx_state[0]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Socket(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("bzenet.__pyx_unpickle_Socket", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_Socket__set_state(<Socket> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Socket__set_state(Socket __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._enet_socket = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6bzenet___pyx_unpickle_Socket__set_state(struct __pyx_obj_6bzenet_Socket *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  ENetSocket __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Socket__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_Socket__set_state(Socket __pyx_result, tuple __pyx_state):
 *     __pyx_result._enet_socket = __pyx_state[0]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_ENetSocket(__pyx_t_1); if (unlikely((__pyx_t_2 == ((ENetSocket)-1)) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_enet_socket = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_Socket__set_state(Socket __pyx_result, tuple __pyx_state):
 *     __pyx_result._enet_socket = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 1) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._enet_socket = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[1])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_Socket__set_state(Socket __pyx_result, tuple __pyx_state):
 *     __pyx_result._enet_socket = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_Socket__set_state(<Socket> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Socket__set_state(Socket __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._enet_socket = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("bzenet.__pyx_unpickle_Socket__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_tp_new_6bzenet_Address(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  return o;
}

static void __pyx_tp_dealloc_6bzenet_Address(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_getprop_6bzenet_7Address_host(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_7Address_4host_1__get__(o);
}

static int __pyx_setprop_6bzenet_7Address_host(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6bzenet_7Address_4host_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6bzenet_7Address_hostname(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_7Address_8hostname_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_7Address_port(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_7Address_4port_1__get__(o);
}

static int __pyx_setprop_6bzenet_7Address_port(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6bzenet_7Address_4port_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6bzenet_Address[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6bzenet_7Address_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6bzenet_7Address_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6bzenet_Address[] = {
  {(char *)"host", __pyx_getprop_6bzenet_7Address_host, __pyx_setprop_6bzenet_7Address_host, (char *)0, 0},
  {(char *)"hostname", __pyx_getprop_6bzenet_7Address_hostname, 0, (char *)0, 0},
  {(char *)"port", __pyx_getprop_6bzenet_7Address_port, __pyx_setprop_6bzenet_7Address_port, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6bzenet_Address = {
  PyVarObject_HEAD_INIT(0, 0)
  "bzenet.Address", /*tp_name*/
  sizeof(struct __pyx_obj_6bzenet_Address), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6bzenet_Address, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6bzenet_7Address_3__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "\n    Address (str address, int port)\n\n    ATTRIBUTES\n\n        str host    Hostname referred to by the Address.\n        int port    Port referred to by the Address.\n\n    DESCRIPTION\n\n        An ENet address and port pair.\n\n        When instantiated, performs a resolution upon 'address'. However, if\n        'address' is None, enet.HOST_ANY is assumed.\n    ", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  __pyx_pw_6bzenet_7Address_5__richcmp__, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6bzenet_Address, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6bzenet_Address, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6bzenet_7Address_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6bzenet_Address, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_6bzenet_Socket(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  return o;
}

static void __pyx_tp_dealloc_6bzenet_Socket(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_6bzenet_Socket[] = {
  {"send", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6bzenet_6Socket_1send, METH_VARARGS|METH_KEYWORDS, 0},
  {"fileno", (PyCFunction)__pyx_pw_6bzenet_6Socket_3fileno, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6bzenet_6Socket_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6bzenet_6Socket_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6bzenet_Socket = {
  PyVarObject_HEAD_INIT(0, 0)
  "bzenet.Socket", /*tp_name*/
  sizeof(struct __pyx_obj_6bzenet_Socket), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6bzenet_Socket, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "\n    Socket (int socket)\n\n    DESCRIPTION\n\n        An ENet socket.\n\n        Can be used with select and poll.\n    ", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6bzenet_Socket, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6bzenet_Socket, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_6bzenet_Packet(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6bzenet_Packet *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6bzenet_Packet *)o);
  p->sent = ((PyBoolObject *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6bzenet_Packet(PyObject *o) {
  struct __pyx_obj_6bzenet_Packet *p = (struct __pyx_obj_6bzenet_Packet *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_6bzenet_6Packet_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->sent);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_6bzenet_Packet(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6bzenet_Packet *p = (struct __pyx_obj_6bzenet_Packet *)o;
  if (p->sent) {
    e = (*v)(((PyObject *)p->sent), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6bzenet_Packet(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6bzenet_Packet *p = (struct __pyx_obj_6bzenet_Packet *)o;
  tmp = ((PyObject*)p->sent);
  p->sent = ((PyBoolObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6bzenet_6Packet_data(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_6Packet_4data_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_6Packet_dataLength(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_6Packet_10dataLength_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_6Packet_flags(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_6Packet_5flags_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_6Packet_sent(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_6Packet_4sent_1__get__(o);
}

static int __pyx_setprop_6bzenet_6Packet_sent(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6bzenet_6Packet_4sent_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6bzenet_Packet[] = {
  {"is_valid", (PyCFunction)__pyx_pw_6bzenet_6Packet_5is_valid, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6bzenet_6Packet_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6bzenet_6Packet_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6bzenet_Packet[] = {
  {(char *)"data", __pyx_getprop_6bzenet_6Packet_data, 0, (char *)0, 0},
  {(char *)"dataLength", __pyx_getprop_6bzenet_6Packet_dataLength, 0, (char *)0, 0},
  {(char *)"flags", __pyx_getprop_6bzenet_6Packet_flags, 0, (char *)0, 0},
  {(char *)"sent", __pyx_getprop_6bzenet_6Packet_sent, __pyx_setprop_6bzenet_6Packet_sent, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6bzenet_Packet = {
  PyVarObject_HEAD_INIT(0, 0)
  "bzenet.Packet", /*tp_name*/
  sizeof(struct __pyx_obj_6bzenet_Packet), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6bzenet_Packet, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    Packet (str dataContents, int flags)\n\n    ATTRIBUTES\n\n        str data        Contains the data for the packet.\n        int flags       Flags modifying delivery of the Packet:\n\n            enet.PACKET_FLAG_RELIABLE Packet must be received by the target peer\n                                      and resend attempts should be made until\n                                      the packet is delivered.\n\n            enet.PACKET_FLAG_UNSEQUENCED Packet will not be sequenced with other\n                                         packets not supported for reliable\n                                         packets.\n\n            enet.PACKET_FLAG_NO_ALLOCATE Packet will not allocate data and user\n                                         must supply it instead.\n\n            enet.PACKET_FLAG_UNRELIABLE_FRAGMENT Packet will be fragmented using\n                                                 unreliable (instead of reliable)\n                                                 sends if it exceeds the MTU...\n\n    DESCRIPTION\n\n        An ENet data packet that may be sent to or received from a peer.\n\n    ", /*tp_doc*/
  __pyx_tp_traverse_6bzenet_Packet, /*tp_traverse*/
  __pyx_tp_clear_6bzenet_Packet, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6bzenet_Packet, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6bzenet_Packet, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6bzenet_6Packet_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6bzenet_Packet, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_6bzenet_Peer(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  return o;
}

static void __pyx_tp_dealloc_6bzenet_Peer(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_host(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_4host_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_outgoingPeerID(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_14outgoingPeerID_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_incomingPeerID(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_14incomingPeerID_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_connectID(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_9connectID_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_outgoingSessionID(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_17outgoingSessionID_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_incomingSessionID(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_17incomingSessionID_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_address(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_7address_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_data(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_4data_1__get__(o);
}

static int __pyx_setprop_6bzenet_4Peer_data(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6bzenet_4Peer_4data_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6bzenet_4Peer_state(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_5state_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_channelCount(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_12channelCount_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_incomingBandwidth(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_17incomingBandwidth_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_outgoingBandwidth(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_17outgoingBandwidth_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_incomingBandwidthThrottleEpoch(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_30incomingBandwidthThrottleEpoch_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_outgoingBandwidthThrottleEpoch(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_30outgoingBandwidthThrottleEpoch_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_incomingDataTotal(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_17incomingDataTotal_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_outgoingDataTotal(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_17outgoingDataTotal_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_lastSendTime(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_12lastSendTime_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_lastReceiveTime(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_15lastReceiveTime_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_nextTimeout(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_11nextTimeout_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_earliestTimeout(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_15earliestTimeout_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_packetLossEpoch(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_15packetLossEpoch_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_packetsSent(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_11packetsSent_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_packetsLost(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_11packetsLost_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_packetLoss(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_10packetLoss_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_packetLossVariance(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_18packetLossVariance_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_packetThrottle(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_14packetThrottle_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_packetThrottleLimit(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_19packetThrottleLimit_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_packetThrottleCounter(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_21packetThrottleCounter_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_packetThrottleEpoch(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_19packetThrottleEpoch_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_packetThrottleAcceleration(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_26packetThrottleAcceleration_1__get__(o);
}

static int __pyx_setprop_6bzenet_4Peer_packetThrottleAcceleration(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6bzenet_4Peer_26packetThrottleAcceleration_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6bzenet_4Peer_packetThrottleDeceleration(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_26packetThrottleDeceleration_1__get__(o);
}

static int __pyx_setprop_6bzenet_4Peer_packetThrottleDeceleration(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6bzenet_4Peer_26packetThrottleDeceleration_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6bzenet_4Peer_packetThrottleInterval(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_22packetThrottleInterval_1__get__(o);
}

static int __pyx_setprop_6bzenet_4Peer_packetThrottleInterval(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6bzenet_4Peer_22packetThrottleInterval_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6bzenet_4Peer_lastRoundTripTime(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_17lastRoundTripTime_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_lowestRoundTripTime(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_19lowestRoundTripTime_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_lastRoundTripTimeVariance(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_25lastRoundTripTimeVariance_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_highestRoundTripTimeVariance(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_28highestRoundTripTimeVariance_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_roundTripTime(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_13roundTripTime_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_roundTripTimeVariance(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_21roundTripTimeVariance_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_mtu(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_3mtu_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_windowSize(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_10windowSize_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_reliableDataInTransit(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_21reliableDataInTransit_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_outgoingReliableSequenceNumber(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_30outgoingReliableSequenceNumber_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_needsDispatch(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_13needsDispatch_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_incomingUnsequencedGroup(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_24incomingUnsequencedGroup_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_outgoingUnsequencedGroup(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_24outgoingUnsequencedGroup_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Peer_eventData(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Peer_9eventData_1__get__(o);
}

static PyMethodDef __pyx_methods_6bzenet_Peer[] = {
  {"send", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6bzenet_4Peer_5send, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6bzenet_4Peer_4send},
  {"receive", (PyCFunction)__pyx_pw_6bzenet_4Peer_7receive, METH_O, __pyx_doc_6bzenet_4Peer_6receive},
  {"reset", (PyCFunction)__pyx_pw_6bzenet_4Peer_9reset, METH_NOARGS, __pyx_doc_6bzenet_4Peer_8reset},
  {"timeout", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6bzenet_4Peer_11timeout, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6bzenet_4Peer_10timeout},
  {"ping", (PyCFunction)__pyx_pw_6bzenet_4Peer_13ping, METH_NOARGS, __pyx_doc_6bzenet_4Peer_12ping},
  {"disconnect", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6bzenet_4Peer_15disconnect, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6bzenet_4Peer_14disconnect},
  {"disconnect_later", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6bzenet_4Peer_17disconnect_later, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6bzenet_4Peer_16disconnect_later},
  {"disconnect_now", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6bzenet_4Peer_19disconnect_now, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6bzenet_4Peer_18disconnect_now},
  {"check_valid", (PyCFunction)__pyx_pw_6bzenet_4Peer_21check_valid, METH_NOARGS, __pyx_doc_6bzenet_4Peer_20check_valid},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6bzenet_4Peer_23__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6bzenet_4Peer_25__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6bzenet_Peer[] = {
  {(char *)"host", __pyx_getprop_6bzenet_4Peer_host, 0, (char *)0, 0},
  {(char *)"outgoingPeerID", __pyx_getprop_6bzenet_4Peer_outgoingPeerID, 0, (char *)0, 0},
  {(char *)"incomingPeerID", __pyx_getprop_6bzenet_4Peer_incomingPeerID, 0, (char *)0, 0},
  {(char *)"connectID", __pyx_getprop_6bzenet_4Peer_connectID, 0, (char *)0, 0},
  {(char *)"outgoingSessionID", __pyx_getprop_6bzenet_4Peer_outgoingSessionID, 0, (char *)0, 0},
  {(char *)"incomingSessionID", __pyx_getprop_6bzenet_4Peer_incomingSessionID, 0, (char *)0, 0},
  {(char *)"address", __pyx_getprop_6bzenet_4Peer_address, 0, (char *)0, 0},
  {(char *)"data", __pyx_getprop_6bzenet_4Peer_data, __pyx_setprop_6bzenet_4Peer_data, (char *)0, 0},
  {(char *)"state", __pyx_getprop_6bzenet_4Peer_state, 0, (char *)0, 0},
  {(char *)"channelCount", __pyx_getprop_6bzenet_4Peer_channelCount, 0, (char *)0, 0},
  {(char *)"incomingBandwidth", __pyx_getprop_6bzenet_4Peer_incomingBandwidth, 0, (char *)0, 0},
  {(char *)"outgoingBandwidth", __pyx_getprop_6bzenet_4Peer_outgoingBandwidth, 0, (char *)0, 0},
  {(char *)"incomingBandwidthThrottleEpoch", __pyx_getprop_6bzenet_4Peer_incomingBandwidthThrottleEpoch, 0, (char *)0, 0},
  {(char *)"outgoingBandwidthThrottleEpoch", __pyx_getprop_6bzenet_4Peer_outgoingBandwidthThrottleEpoch, 0, (char *)0, 0},
  {(char *)"incomingDataTotal", __pyx_getprop_6bzenet_4Peer_incomingDataTotal, 0, (char *)0, 0},
  {(char *)"outgoingDataTotal", __pyx_getprop_6bzenet_4Peer_outgoingDataTotal, 0, (char *)0, 0},
  {(char *)"lastSendTime", __pyx_getprop_6bzenet_4Peer_lastSendTime, 0, (char *)0, 0},
  {(char *)"lastReceiveTime", __pyx_getprop_6bzenet_4Peer_lastReceiveTime, 0, (char *)0, 0},
  {(char *)"nextTimeout", __pyx_getprop_6bzenet_4Peer_nextTimeout, 0, (char *)0, 0},
  {(char *)"earliestTimeout", __pyx_getprop_6bzenet_4Peer_earliestTimeout, 0, (char *)0, 0},
  {(char *)"packetLossEpoch", __pyx_getprop_6bzenet_4Peer_packetLossEpoch, 0, (char *)0, 0},
  {(char *)"packetsSent", __pyx_getprop_6bzenet_4Peer_packetsSent, 0, (char *)0, 0},
  {(char *)"packetsLost", __pyx_getprop_6bzenet_4Peer_packetsLost, 0, (char *)0, 0},
  {(char *)"packetLoss", __pyx_getprop_6bzenet_4Peer_packetLoss, 0, (char *)0, 0},
  {(char *)"packetLossVariance", __pyx_getprop_6bzenet_4Peer_packetLossVariance, 0, (char *)0, 0},
  {(char *)"packetThrottle", __pyx_getprop_6bzenet_4Peer_packetThrottle, 0, (char *)0, 0},
  {(char *)"packetThrottleLimit", __pyx_getprop_6bzenet_4Peer_packetThrottleLimit, 0, (char *)0, 0},
  {(char *)"packetThrottleCounter", __pyx_getprop_6bzenet_4Peer_packetThrottleCounter, 0, (char *)0, 0},
  {(char *)"packetThrottleEpoch", __pyx_getprop_6bzenet_4Peer_packetThrottleEpoch, 0, (char *)0, 0},
  {(char *)"packetThrottleAcceleration", __pyx_getprop_6bzenet_4Peer_packetThrottleAcceleration, __pyx_setprop_6bzenet_4Peer_packetThrottleAcceleration, (char *)0, 0},
  {(char *)"packetThrottleDeceleration", __pyx_getprop_6bzenet_4Peer_packetThrottleDeceleration, __pyx_setprop_6bzenet_4Peer_packetThrottleDeceleration, (char *)0, 0},
  {(char *)"packetThrottleInterval", __pyx_getprop_6bzenet_4Peer_packetThrottleInterval, __pyx_setprop_6bzenet_4Peer_packetThrottleInterval, (char *)0, 0},
  {(char *)"lastRoundTripTime", __pyx_getprop_6bzenet_4Peer_lastRoundTripTime, 0, (char *)0, 0},
  {(char *)"lowestRoundTripTime", __pyx_getprop_6bzenet_4Peer_lowestRoundTripTime, 0, (char *)0, 0},
  {(char *)"lastRoundTripTimeVariance", __pyx_getprop_6bzenet_4Peer_lastRoundTripTimeVariance, 0, (char *)0, 0},
  {(char *)"highestRoundTripTimeVariance", __pyx_getprop_6bzenet_4Peer_highestRoundTripTimeVariance, 0, (char *)0, 0},
  {(char *)"roundTripTime", __pyx_getprop_6bzenet_4Peer_roundTripTime, 0, (char *)0, 0},
  {(char *)"roundTripTimeVariance", __pyx_getprop_6bzenet_4Peer_roundTripTimeVariance, 0, (char *)0, 0},
  {(char *)"mtu", __pyx_getprop_6bzenet_4Peer_mtu, 0, (char *)0, 0},
  {(char *)"windowSize", __pyx_getprop_6bzenet_4Peer_windowSize, 0, (char *)0, 0},
  {(char *)"reliableDataInTransit", __pyx_getprop_6bzenet_4Peer_reliableDataInTransit, 0, (char *)0, 0},
  {(char *)"outgoingReliableSequenceNumber", __pyx_getprop_6bzenet_4Peer_outgoingReliableSequenceNumber, 0, (char *)0, 0},
  {(char *)"needsDispatch", __pyx_getprop_6bzenet_4Peer_needsDispatch, 0, (char *)0, 0},
  {(char *)"incomingUnsequencedGroup", __pyx_getprop_6bzenet_4Peer_incomingUnsequencedGroup, 0, (char *)0, 0},
  {(char *)"outgoingUnsequencedGroup", __pyx_getprop_6bzenet_4Peer_outgoingUnsequencedGroup, 0, (char *)0, 0},
  {(char *)"eventData", __pyx_getprop_6bzenet_4Peer_eventData, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6bzenet_Peer = {
  PyVarObject_HEAD_INIT(0, 0)
  "bzenet.Peer", /*tp_name*/
  sizeof(struct __pyx_obj_6bzenet_Peer), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6bzenet_Peer, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  __pyx_pw_6bzenet_4Peer_3__hash__, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "\n    Peer ()\n\n    ATTRIBUTES\n\n        Address address\n        int     state       The peer's current state which is one of\n                            enet.PEER_STATE_*\n        int     packetLoss  Mean packet loss of reliable packets as a ratio with\n                            respect to the constant enet.PEER_PACKET_LOSS_SCALE.\n        int     packetThrottleAcceleration\n        int     packetThrottleDeceleration\n        int     packetThrottleInterval\n        int     roundTripTime Mean round trip time (RTT), in milliseconds,\n                              between sending a reliable packet and receiving\n                              its acknowledgement.\n        int     incomingPeerID\n\n    DESCRIPTION\n\n        An ENet peer which data packets may be sent or received from.\n\n        This class should never be instantiated directly, but rather via\n        enet.Host.connect or enet.Event.Peer.  If you try to access any members\n        of a Peer without being properly instantiated from a Host or Event\n        object then a MemoryError will be raised.\n\n    ", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  __pyx_pw_6bzenet_4Peer_1__richcmp__, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6bzenet_Peer, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6bzenet_Peer, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6bzenet_Peer, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_6bzenet_Event(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6bzenet_Event *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6bzenet_Event *)o);
  p->_packet = ((struct __pyx_obj_6bzenet_Packet *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6bzenet_Event(PyObject *o) {
  struct __pyx_obj_6bzenet_Event *p = (struct __pyx_obj_6bzenet_Event *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_packet);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_6bzenet_Event(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6bzenet_Event *p = (struct __pyx_obj_6bzenet_Event *)o;
  if (p->_packet) {
    e = (*v)(((PyObject *)p->_packet), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6bzenet_Event(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6bzenet_Event *p = (struct __pyx_obj_6bzenet_Event *)o;
  tmp = ((PyObject*)p->_packet);
  p->_packet = ((struct __pyx_obj_6bzenet_Packet *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6bzenet_5Event_type(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_5Event_4type_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_5Event_peer(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_5Event_4peer_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_5Event_channelID(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_5Event_9channelID_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_5Event_data(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_5Event_4data_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_5Event_packet(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_5Event_6packet_1__get__(o);
}

static PyMethodDef __pyx_methods_6bzenet_Event[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6bzenet_5Event_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6bzenet_5Event_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6bzenet_Event[] = {
  {(char *)"type", __pyx_getprop_6bzenet_5Event_type, 0, (char *)0, 0},
  {(char *)"peer", __pyx_getprop_6bzenet_5Event_peer, 0, (char *)0, 0},
  {(char *)"channelID", __pyx_getprop_6bzenet_5Event_channelID, 0, (char *)0, 0},
  {(char *)"data", __pyx_getprop_6bzenet_5Event_data, 0, (char *)0, 0},
  {(char *)"packet", __pyx_getprop_6bzenet_5Event_packet, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6bzenet_Event = {
  PyVarObject_HEAD_INIT(0, 0)
  "bzenet.Event", /*tp_name*/
  sizeof(struct __pyx_obj_6bzenet_Event), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6bzenet_Event, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    Event ()\n\n    ATTRIBUTES\n\n        int     type        Type of the event.  Will be enet.EVENT_TYPE_*.\n        Peer    peer        Peer that generated the event.\n        int     channelID\n        Packet  packet\n\n    DESCRIPTION\n\n        An ENet event as returned by enet.Host.service.\n\n        This class should never be instantiated directly.\n    ", /*tp_doc*/
  __pyx_tp_traverse_6bzenet_Event, /*tp_traverse*/
  __pyx_tp_clear_6bzenet_Event, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6bzenet_Event, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6bzenet_Event, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6bzenet_5Event_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6bzenet_Event, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_6bzenet_Host(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6bzenet_Host *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6bzenet_Host *)o);
  p->dealloc = ((PyBoolObject *)Py_None); Py_INCREF(Py_None);
  p->_interceptCallback = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_6bzenet_4Host_5__cinit__(o, __pyx_empty_tuple, NULL) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_6bzenet_Host(PyObject *o) {
  struct __pyx_obj_6bzenet_Host *p = (struct __pyx_obj_6bzenet_Host *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_6bzenet_4Host_7__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  if (p->__weakref__) PyObject_ClearWeakRefs(o);
  Py_CLEAR(p->dealloc);
  Py_CLEAR(p->_interceptCallback);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_6bzenet_Host(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6bzenet_Host *p = (struct __pyx_obj_6bzenet_Host *)o;
  if (p->dealloc) {
    e = (*v)(((PyObject *)p->dealloc), a); if (e) return e;
  }
  if (p->_interceptCallback) {
    e = (*v)(p->_interceptCallback, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6bzenet_Host(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6bzenet_Host *p = (struct __pyx_obj_6bzenet_Host *)o;
  tmp = ((PyObject*)p->dealloc);
  p->dealloc = ((PyBoolObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_interceptCallback);
  p->_interceptCallback = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6bzenet_4Host_socket(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Host_6socket_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Host_address(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Host_7address_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Host_incomingBandwidth(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Host_17incomingBandwidth_1__get__(o);
}

static int __pyx_setprop_6bzenet_4Host_incomingBandwidth(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6bzenet_4Host_17incomingBandwidth_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6bzenet_4Host_outgoingBandwidth(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Host_17outgoingBandwidth_1__get__(o);
}

static int __pyx_setprop_6bzenet_4Host_outgoingBandwidth(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6bzenet_4Host_17outgoingBandwidth_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6bzenet_4Host_peers(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Host_5peers_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Host_peerCount(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Host_9peerCount_1__get__(o);
}

static PyObject *__pyx_getprop_6bzenet_4Host_channelLimit(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Host_12channelLimit_1__get__(o);
}

static int __pyx_setprop_6bzenet_4Host_channelLimit(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6bzenet_4Host_12channelLimit_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6bzenet_4Host_totalSentData(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Host_13totalSentData_1__get__(o);
}

static int __pyx_setprop_6bzenet_4Host_totalSentData(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6bzenet_4Host_13totalSentData_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6bzenet_4Host_totalSentPackets(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Host_16totalSentPackets_1__get__(o);
}

static int __pyx_setprop_6bzenet_4Host_totalSentPackets(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6bzenet_4Host_16totalSentPackets_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6bzenet_4Host_totalReceivedData(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Host_17totalReceivedData_1__get__(o);
}

static int __pyx_setprop_6bzenet_4Host_totalReceivedData(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6bzenet_4Host_17totalReceivedData_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6bzenet_4Host_totalReceivedPackets(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Host_20totalReceivedPackets_1__get__(o);
}

static int __pyx_setprop_6bzenet_4Host_totalReceivedPackets(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6bzenet_4Host_20totalReceivedPackets_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6bzenet_4Host_intercept(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzenet_4Host_9intercept_1__get__(o);
}

static int __pyx_setprop_6bzenet_4Host_intercept(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6bzenet_4Host_9intercept_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6bzenet_Host[] = {
  {"connect", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6bzenet_4Host_9connect, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6bzenet_4Host_8connect},
  {"check_events", (PyCFunction)__pyx_pw_6bzenet_4Host_11check_events, METH_NOARGS, __pyx_doc_6bzenet_4Host_10check_events},
  {"service", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6bzenet_4Host_13service, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6bzenet_4Host_12service},
  {"flush", (PyCFunction)__pyx_pw_6bzenet_4Host_15flush, METH_NOARGS, __pyx_doc_6bzenet_4Host_14flush},
  {"broadcast", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6bzenet_4Host_17broadcast, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6bzenet_4Host_16broadcast},
  {"compress_with_range_coder", (PyCFunction)__pyx_pw_6bzenet_4Host_19compress_with_range_coder, METH_NOARGS, __pyx_doc_6bzenet_4Host_18compress_with_range_coder},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6bzenet_4Host_21__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6bzenet_4Host_23__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6bzenet_Host[] = {
  {(char *)"socket", __pyx_getprop_6bzenet_4Host_socket, 0, (char *)0, 0},
  {(char *)"address", __pyx_getprop_6bzenet_4Host_address, 0, (char *)0, 0},
  {(char *)"incomingBandwidth", __pyx_getprop_6bzenet_4Host_incomingBandwidth, __pyx_setprop_6bzenet_4Host_incomingBandwidth, (char *)0, 0},
  {(char *)"outgoingBandwidth", __pyx_getprop_6bzenet_4Host_outgoingBandwidth, __pyx_setprop_6bzenet_4Host_outgoingBandwidth, (char *)0, 0},
  {(char *)"peers", __pyx_getprop_6bzenet_4Host_peers, 0, (char *)0, 0},
  {(char *)"peerCount", __pyx_getprop_6bzenet_4Host_peerCount, 0, (char *)0, 0},
  {(char *)"channelLimit", __pyx_getprop_6bzenet_4Host_channelLimit, __pyx_setprop_6bzenet_4Host_channelLimit, (char *)0, 0},
  {(char *)"totalSentData", __pyx_getprop_6bzenet_4Host_totalSentData, __pyx_setprop_6bzenet_4Host_totalSentData, (char *)0, 0},
  {(char *)"totalSentPackets", __pyx_getprop_6bzenet_4Host_totalSentPackets, __pyx_setprop_6bzenet_4Host_totalSentPackets, (char *)0, 0},
  {(char *)"totalReceivedData", __pyx_getprop_6bzenet_4Host_totalReceivedData, __pyx_setprop_6bzenet_4Host_totalReceivedData, (char *)0, 0},
  {(char *)"totalReceivedPackets", __pyx_getprop_6bzenet_4Host_totalReceivedPackets, __pyx_setprop_6bzenet_4Host_totalReceivedPackets, (char *)0, 0},
  {(char *)"intercept", __pyx_getprop_6bzenet_4Host_intercept, __pyx_setprop_6bzenet_4Host_intercept, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6bzenet_Host = {
  PyVarObject_HEAD_INIT(0, 0)
  "bzenet.Host", /*tp_name*/
  sizeof(struct __pyx_obj_6bzenet_Host), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6bzenet_Host, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  __pyx_pw_6bzenet_4Host_3__hash__, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    Host (Address address, int peerCount, int channelLimit,\n        int incomingBandwidth, int outgoingBandwidth)\n\n    ATTRIBUTES\n\n        Address address             Internet address of the host.\n        Socket  socket              The socket the host services.\n        int     incomingBandwidth   Downstream bandwidth of the host.\n        int     outgoingBandwidth   Upstream bandwidth of the host.\n\n    DESCRIPTION\n\n        An ENet host for communicating with peers.\n\n        If 'address' is None, then the Host will be client only.\n    ", /*tp_doc*/
  __pyx_tp_traverse_6bzenet_Host, /*tp_traverse*/
  __pyx_tp_clear_6bzenet_Host, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6bzenet_Host, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6bzenet_Host, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6bzenet_4Host_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6bzenet_Host, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_bzenet(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_bzenet},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "bzenet",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_, __pyx_k_, sizeof(__pyx_k_), 0, 0, 1, 0},
  {&__pyx_kp_s_0_1, __pyx_k_0_1, sizeof(__pyx_k_0_1), 0, 0, 1, 0},
  {&__pyx_n_s_Address, __pyx_k_Address, sizeof(__pyx_k_Address), 0, 0, 1, 1},
  {&__pyx_kp_s_Connection_failure, __pyx_k_Connection_failure, sizeof(__pyx_k_Connection_failure), 0, 0, 1, 0},
  {&__pyx_n_s_EVENT_TYPE_CONNECT, __pyx_k_EVENT_TYPE_CONNECT, sizeof(__pyx_k_EVENT_TYPE_CONNECT), 0, 0, 1, 1},
  {&__pyx_n_s_EVENT_TYPE_DISCONNECT, __pyx_k_EVENT_TYPE_DISCONNECT, sizeof(__pyx_k_EVENT_TYPE_DISCONNECT), 0, 0, 1, 1},
  {&__pyx_n_s_EVENT_TYPE_NONE, __pyx_k_EVENT_TYPE_NONE, sizeof(__pyx_k_EVENT_TYPE_NONE), 0, 0, 1, 1},
  {&__pyx_n_s_EVENT_TYPE_RECEIVE, __pyx_k_EVENT_TYPE_RECEIVE, sizeof(__pyx_k_EVENT_TYPE_RECEIVE), 0, 0, 1, 1},
  {&__pyx_kp_s_Empty_Peer_object_accessed, __pyx_k_Empty_Peer_object_accessed, sizeof(__pyx_k_Empty_Peer_object_accessed), 0, 0, 1, 0},
  {&__pyx_n_s_Event, __pyx_k_Event, sizeof(__pyx_k_Event), 0, 0, 1, 1},
  {&__pyx_n_s_Host, __pyx_k_Host, sizeof(__pyx_k_Host), 0, 0, 1, 1},
  {&__pyx_n_s_IOError, __pyx_k_IOError, sizeof(__pyx_k_IOError), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xd7, __pyx_k_Incompatible_checksums_s_vs_0xd7, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xd7), 0, 0, 1, 0},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_n_s_NotImplementedError, __pyx_k_NotImplementedError, sizeof(__pyx_k_NotImplementedError), 0, 0, 1, 1},
  {&__pyx_n_s_PACKET_FLAG_NO_ALLOCATE, __pyx_k_PACKET_FLAG_NO_ALLOCATE, sizeof(__pyx_k_PACKET_FLAG_NO_ALLOCATE), 0, 0, 1, 1},
  {&__pyx_n_s_PACKET_FLAG_RELIABLE, __pyx_k_PACKET_FLAG_RELIABLE, sizeof(__pyx_k_PACKET_FLAG_RELIABLE), 0, 0, 1, 1},
  {&__pyx_n_s_PACKET_FLAG_UNRELIABLE_FRAGMENT, __pyx_k_PACKET_FLAG_UNRELIABLE_FRAGMENT, sizeof(__pyx_k_PACKET_FLAG_UNRELIABLE_FRAGMENT), 0, 0, 1, 1},
  {&__pyx_n_s_PACKET_FLAG_UNSEQUENCED, __pyx_k_PACKET_FLAG_UNSEQUENCED, sizeof(__pyx_k_PACKET_FLAG_UNSEQUENCED), 0, 0, 1, 1},
  {&__pyx_n_s_PEER_STATE_ACKNOWLEDGING_CONNECT, __pyx_k_PEER_STATE_ACKNOWLEDGING_CONNECT, sizeof(__pyx_k_PEER_STATE_ACKNOWLEDGING_CONNECT), 0, 0, 1, 1},
  {&__pyx_n_s_PEER_STATE_ACKNOWLEDGING_DISCONN, __pyx_k_PEER_STATE_ACKNOWLEDGING_DISCONN, sizeof(__pyx_k_PEER_STATE_ACKNOWLEDGING_DISCONN), 0, 0, 1, 1},
  {&__pyx_n_s_PEER_STATE_CONNECTED, __pyx_k_PEER_STATE_CONNECTED, sizeof(__pyx_k_PEER_STATE_CONNECTED), 0, 0, 1, 1},
  {&__pyx_n_s_PEER_STATE_CONNECTING, __pyx_k_PEER_STATE_CONNECTING, sizeof(__pyx_k_PEER_STATE_CONNECTING), 0, 0, 1, 1},
  {&__pyx_n_s_PEER_STATE_CONNECTION_PENDING, __pyx_k_PEER_STATE_CONNECTION_PENDING, sizeof(__pyx_k_PEER_STATE_CONNECTION_PENDING), 0, 0, 1, 1},
  {&__pyx_n_s_PEER_STATE_CONNECTION_SUCCEEDED, __pyx_k_PEER_STATE_CONNECTION_SUCCEEDED, sizeof(__pyx_k_PEER_STATE_CONNECTION_SUCCEEDED), 0, 0, 1, 1},
  {&__pyx_n_s_PEER_STATE_DISCONNECTED, __pyx_k_PEER_STATE_DISCONNECTED, sizeof(__pyx_k_PEER_STATE_DISCONNECTED), 0, 0, 1, 1},
  {&__pyx_n_s_PEER_STATE_DISCONNECTING, __pyx_k_PEER_STATE_DISCONNECTING, sizeof(__pyx_k_PEER_STATE_DISCONNECTING), 0, 0, 1, 1},
  {&__pyx_n_s_PEER_STATE_DISCONNECT_LATER, __pyx_k_PEER_STATE_DISCONNECT_LATER, sizeof(__pyx_k_PEER_STATE_DISCONNECT_LATER), 0, 0, 1, 1},
  {&__pyx_n_s_PEER_STATE_ZOMBIE, __pyx_k_PEER_STATE_ZOMBIE, sizeof(__pyx_k_PEER_STATE_ZOMBIE), 0, 0, 1, 1},
  {&__pyx_n_s_Packet, __pyx_k_Packet, sizeof(__pyx_k_Packet), 0, 0, 1, 1},
  {&__pyx_kp_s_Packet_has_not_been_initiliazed, __pyx_k_Packet_has_not_been_initiliazed, sizeof(__pyx_k_Packet_has_not_been_initiliazed), 0, 0, 1, 0},
  {&__pyx_n_s_Peer, __pyx_k_Peer, sizeof(__pyx_k_Peer), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_kp_s_Pickling_of_struct_members_such, __pyx_k_Pickling_of_struct_members_such, sizeof(__pyx_k_Pickling_of_struct_members_such), 0, 0, 1, 0},
  {&__pyx_kp_s_Resolution_failure, __pyx_k_Resolution_failure, sizeof(__pyx_k_Resolution_failure), 0, 0, 1, 0},
  {&__pyx_kp_s_Servicing_error_probably_disconn, __pyx_k_Servicing_error_probably_disconn, sizeof(__pyx_k_Servicing_error_probably_disconn), 0, 0, 1, 0},
  {&__pyx_n_s_Socket, __pyx_k_Socket, sizeof(__pyx_k_Socket), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Unable_to_create_host_structure, __pyx_k_Unable_to_create_host_structure, sizeof(__pyx_k_Unable_to_create_host_structure), 0, 0, 1, 0},
  {&__pyx_n_s_WeakValueDictionary, __pyx_k_WeakValueDictionary, sizeof(__pyx_k_WeakValueDictionary), 0, 0, 1, 1},
  {&__pyx_n_s_address, __pyx_k_address, sizeof(__pyx_k_address), 0, 0, 1, 1},
  {&__pyx_n_s_ascii, __pyx_k_ascii, sizeof(__pyx_k_ascii), 0, 0, 1, 1},
  {&__pyx_n_s_atexit, __pyx_k_atexit, sizeof(__pyx_k_atexit), 0, 0, 1, 1},
  {&__pyx_n_s_bzenet, __pyx_k_bzenet, sizeof(__pyx_k_bzenet), 0, 0, 1, 1},
  {&__pyx_n_s_channelCount, __pyx_k_channelCount, sizeof(__pyx_k_channelCount), 0, 0, 1, 1},
  {&__pyx_n_s_channelID, __pyx_k_channelID, sizeof(__pyx_k_channelID), 0, 0, 1, 1},
  {&__pyx_n_s_channelLimit, __pyx_k_channelLimit, sizeof(__pyx_k_channelLimit), 0, 0, 1, 1},
  {&__pyx_n_s_check_valid, __pyx_k_check_valid, sizeof(__pyx_k_check_valid), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_enet_atexit, __pyx_k_enet_atexit, sizeof(__pyx_k_enet_atexit), 0, 0, 1, 1},
  {&__pyx_kp_s_enet_pyx, __pyx_k_enet_pyx, sizeof(__pyx_k_enet_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_fast_drop, __pyx_k_fast_drop, sizeof(__pyx_k_fast_drop), 0, 0, 1, 1},
  {&__pyx_n_s_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_host, __pyx_k_host, sizeof(__pyx_k_host), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_incomingBandwidth, __pyx_k_incomingBandwidth, sizeof(__pyx_k_incomingBandwidth), 0, 0, 1, 1},
  {&__pyx_n_s_intercept, __pyx_k_intercept, sizeof(__pyx_k_intercept), 0, 0, 1, 1},
  {&__pyx_n_s_is_valid, __pyx_k_is_valid, sizeof(__pyx_k_is_valid), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
  {&__pyx_n_s_outgoingBandwidth, __pyx_k_outgoingBandwidth, sizeof(__pyx_k_outgoingBandwidth), 0, 0, 1, 1},
  {&__pyx_n_s_packet, __pyx_k_packet, sizeof(__pyx_k_packet), 0, 0, 1, 1},
  {&__pyx_n_s_packetThrottleAcceleration, __pyx_k_packetThrottleAcceleration, sizeof(__pyx_k_packetThrottleAcceleration), 0, 0, 1, 1},
  {&__pyx_n_s_packetThrottleDeceleration, __pyx_k_packetThrottleDeceleration, sizeof(__pyx_k_packetThrottleDeceleration), 0, 0, 1, 1},
  {&__pyx_n_s_packetThrottleInterval, __pyx_k_packetThrottleInterval, sizeof(__pyx_k_packetThrottleInterval), 0, 0, 1, 1},
  {&__pyx_n_s_peerCount, __pyx_k_peerCount, sizeof(__pyx_k_peerCount), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_port, __pyx_k_port, sizeof(__pyx_k_port), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_Socket, __pyx_k_pyx_unpickle_Socket, sizeof(__pyx_k_pyx_unpickle_Socket), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_register, __pyx_k_register, sizeof(__pyx_k_register), 0, 0, 1, 1},
  {&__pyx_kp_s_self__enet_event_cannot_be_conve, __pyx_k_self__enet_event_cannot_be_conve, sizeof(__pyx_k_self__enet_event_cannot_be_conve), 0, 0, 1, 0},
  {&__pyx_kp_s_self__enet_packet_cannot_be_conv, __pyx_k_self__enet_packet_cannot_be_conv, sizeof(__pyx_k_self__enet_packet_cannot_be_conv), 0, 0, 1, 0},
  {&__pyx_kp_s_self__enet_peer_cannot_be_conver, __pyx_k_self__enet_peer_cannot_be_conver, sizeof(__pyx_k_self__enet_peer_cannot_be_conver), 0, 0, 1, 0},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_time, __pyx_k_time, sizeof(__pyx_k_time), 0, 0, 1, 1},
  {&__pyx_n_s_time_max, __pyx_k_time_max, sizeof(__pyx_k_time_max), 0, 0, 1, 1},
  {&__pyx_n_s_time_min, __pyx_k_time_min, sizeof(__pyx_k_time_min), 0, 0, 1, 1},
  {&__pyx_n_s_timeout, __pyx_k_timeout, sizeof(__pyx_k_timeout), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_weakref, __pyx_k_weakref, sizeof(__pyx_k_weakref), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_NotImplementedError = __Pyx_GetBuiltinName(__pyx_n_s_NotImplementedError); if (!__pyx_builtin_NotImplementedError) __PYX_ERR(0, 273, __pyx_L1_error)
  __pyx_builtin_IOError = __Pyx_GetBuiltinName(__pyx_n_s_IOError); if (!__pyx_builtin_IOError) __PYX_ERR(0, 283, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(0, 368, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "enet.pyx":283
 *             elif self._enet_address.host:
 *                 if enet_address_get_host_ip(&self._enet_address, host, MAXHOSTNAME):
 *                     raise IOError("Resolution failure!")             # <<<<<<<<<<<<<<
 *                 return unicode(host, "ascii")
 * 
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_Resolution_failure); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self._enet_address must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self._enet_address must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self._enet_address must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self._enet_address must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "enet.pyx":368
 *                 return (<char *>self._enet_packet.data)[:self._enet_packet.dataLength]
 *             else:
 *                 raise MemoryError("Packet has not been initiliazed properly!")             # <<<<<<<<<<<<<<
 * 
 *     property dataLength:
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_Packet_has_not_been_initiliazed); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 368, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._enet_packet cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._enet_packet cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_self__enet_packet_cannot_be_conv); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "(tree fragment)":4
 *     raise TypeError("self._enet_packet cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._enet_packet cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_s_self__enet_packet_cannot_be_conv); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "enet.pyx":535
 *             return True
 *         else:
 *             raise MemoryError("Empty Peer object accessed!")             # <<<<<<<<<<<<<<
 * 
 *     property host:
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_Empty_Peer_object_accessed); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "enet.pyx":574
 *         def __get__(self):
 *             if self.check_valid():
 *                 a = Address(None, 0)             # <<<<<<<<<<<<<<
 *                 (<Address> a)._enet_address = self._enet_peer.address
 *                 return a
 */
  __pyx_tuple__9 = PyTuple_Pack(2, Py_None, __pyx_int_0); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._enet_peer cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._enet_peer cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_self__enet_peer_cannot_be_conver); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "(tree fragment)":4
 *     raise TypeError("self._enet_peer cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._enet_peer cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_self__enet_peer_cannot_be_conver); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._enet_event cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._enet_event cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_s_self__enet_event_cannot_be_conve); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "(tree fragment)":4
 *     raise TypeError("self._enet_event cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._enet_event cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s_self__enet_event_cannot_be_conve); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "enet.pyx":880
 * 
 *         if not self._enet_host:
 *             raise MemoryError("Unable to create host structure!")             # <<<<<<<<<<<<<<
 *         self.dealloc = True
 * 
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_kp_s_Unable_to_create_host_structure); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(0, 880, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "enet.pyx":910
 * 
 *             if not (<Peer> peer)._enet_peer:
 *                 raise IOError("Connection failure!")             # <<<<<<<<<<<<<<
 * 
 *             return peer
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_s_Connection_failure); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 910, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "enet.pyx":925
 * 
 *             if result < 0:
 *                 raise IOError("Servicing error - probably disconnected.")             # <<<<<<<<<<<<<<
 *             elif result == 0:
 *                 return None
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_s_Servicing_error_probably_disconn); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 925, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "enet.pyx":1082
 *     return int(bool(ret))
 * 
 * def _enet_atexit():             # <<<<<<<<<<<<<<
 *     enet_deinitialize()
 * 
 */
  __pyx_codeobj__19 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_enet_pyx, __pyx_n_s_enet_atexit, 1082, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__19)) __PYX_ERR(0, 1082, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_Socket(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__20 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);
  __pyx_codeobj__21 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__20, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_Socket, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__21)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_225546583 = PyInt_FromLong(225546583L); if (unlikely(!__pyx_int_225546583)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __pyx_v_6bzenet_host_static_instances = Py_None; Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  if (PyType_Ready(&__pyx_type_6bzenet_Address) < 0) __PYX_ERR(0, 234, __pyx_L1_error)
  __pyx_type_6bzenet_Address.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6bzenet_Address.tp_dictoffset && __pyx_type_6bzenet_Address.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6bzenet_Address.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Address, (PyObject *)&__pyx_type_6bzenet_Address) < 0) __PYX_ERR(0, 234, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6bzenet_Address) < 0) __PYX_ERR(0, 234, __pyx_L1_error)
  __pyx_ptype_6bzenet_Address = &__pyx_type_6bzenet_Address;
  if (PyType_Ready(&__pyx_type_6bzenet_Socket) < 0) __PYX_ERR(0, 207, __pyx_L1_error)
  __pyx_type_6bzenet_Socket.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6bzenet_Socket.tp_dictoffset && __pyx_type_6bzenet_Socket.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6bzenet_Socket.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Socket, (PyObject *)&__pyx_type_6bzenet_Socket) < 0) __PYX_ERR(0, 207, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6bzenet_Socket) < 0) __PYX_ERR(0, 207, __pyx_L1_error)
  __pyx_ptype_6bzenet_Socket = &__pyx_type_6bzenet_Socket;
  if (PyType_Ready(&__pyx_type_6bzenet_Packet) < 0) __PYX_ERR(0, 311, __pyx_L1_error)
  __pyx_type_6bzenet_Packet.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6bzenet_Packet.tp_dictoffset && __pyx_type_6bzenet_Packet.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6bzenet_Packet.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Packet, (PyObject *)&__pyx_type_6bzenet_Packet) < 0) __PYX_ERR(0, 311, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6bzenet_Packet) < 0) __PYX_ERR(0, 311, __pyx_L1_error)
  __pyx_ptype_6bzenet_Packet = &__pyx_type_6bzenet_Packet;
  if (PyType_Ready(&__pyx_type_6bzenet_Peer) < 0) __PYX_ERR(0, 390, __pyx_L1_error)
  __pyx_type_6bzenet_Peer.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6bzenet_Peer.tp_dictoffset && __pyx_type_6bzenet_Peer.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6bzenet_Peer.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Peer, (PyObject *)&__pyx_type_6bzenet_Peer) < 0) __PYX_ERR(0, 390, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6bzenet_Peer) < 0) __PYX_ERR(0, 390, __pyx_L1_error)
  __pyx_ptype_6bzenet_Peer = &__pyx_type_6bzenet_Peer;
  if (PyType_Ready(&__pyx_type_6bzenet_Event) < 0) __PYX_ERR(0, 795, __pyx_L1_error)
  __pyx_type_6bzenet_Event.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6bzenet_Event.tp_dictoffset && __pyx_type_6bzenet_Event.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6bzenet_Event.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Event, (PyObject *)&__pyx_type_6bzenet_Event) < 0) __PYX_ERR(0, 795, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6bzenet_Event) < 0) __PYX_ERR(0, 795, __pyx_L1_error)
  __pyx_ptype_6bzenet_Event = &__pyx_type_6bzenet_Event;
  if (PyType_Ready(&__pyx_type_6bzenet_Host) < 0) __PYX_ERR(0, 847, __pyx_L1_error)
  __pyx_type_6bzenet_Host.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6bzenet_Host.tp_dictoffset && __pyx_type_6bzenet_Host.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6bzenet_Host.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Host, (PyObject *)&__pyx_type_6bzenet_Host) < 0) __PYX_ERR(0, 847, __pyx_L1_error)
  if (__pyx_type_6bzenet_Host.tp_weaklistoffset == 0) __pyx_type_6bzenet_Host.tp_weaklistoffset = offsetof(struct __pyx_obj_6bzenet_Host, __weakref__);
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6bzenet_Host) < 0) __PYX_ERR(0, 847, __pyx_L1_error)
  __pyx_ptype_6bzenet_Host = &__pyx_type_6bzenet_Host;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "bool", sizeof(PyBoolObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4bool_bool) __PYX_ERR(3, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_7complex_complex = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "complex", sizeof(PyComplexObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_7complex_complex) __PYX_ERR(4, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#if PY_MAJOR_VERSION < 3
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC void
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#else
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initbzenet(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initbzenet(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_bzenet(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_bzenet(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_bzenet(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'bzenet' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_bzenet(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("bzenet", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_bzenet) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "bzenet")) {
      if (unlikely(PyDict_SetItemString(modules, "bzenet", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() != 0)) goto __pyx_L1_error;
  if (unlikely(__Pyx_modinit_type_import_code() != 0)) goto __pyx_L1_error;
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "enet.pyx":1
 * import atexit             # <<<<<<<<<<<<<<
 * 
 * from cpython cimport bool
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_atexit, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_atexit, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":184
 *     MAXHOSTNAME = 257
 * 
 * PACKET_FLAG_RELIABLE = ENET_PACKET_FLAG_RELIABLE             # <<<<<<<<<<<<<<
 * PACKET_FLAG_UNSEQUENCED = ENET_PACKET_FLAG_UNSEQUENCED
 * PACKET_FLAG_NO_ALLOCATE = ENET_PACKET_FLAG_NO_ALLOCATE
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetPacketFlag(ENET_PACKET_FLAG_RELIABLE); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PACKET_FLAG_RELIABLE, __pyx_t_1) < 0) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":185
 * 
 * PACKET_FLAG_RELIABLE = ENET_PACKET_FLAG_RELIABLE
 * PACKET_FLAG_UNSEQUENCED = ENET_PACKET_FLAG_UNSEQUENCED             # <<<<<<<<<<<<<<
 * PACKET_FLAG_NO_ALLOCATE = ENET_PACKET_FLAG_NO_ALLOCATE
 * PACKET_FLAG_UNRELIABLE_FRAGMENT = ENET_PACKET_FLAG_UNRELIABLE_FRAGMENT
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetPacketFlag(ENET_PACKET_FLAG_UNSEQUENCED); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PACKET_FLAG_UNSEQUENCED, __pyx_t_1) < 0) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":186
 * PACKET_FLAG_RELIABLE = ENET_PACKET_FLAG_RELIABLE
 * PACKET_FLAG_UNSEQUENCED = ENET_PACKET_FLAG_UNSEQUENCED
 * PACKET_FLAG_NO_ALLOCATE = ENET_PACKET_FLAG_NO_ALLOCATE             # <<<<<<<<<<<<<<
 * PACKET_FLAG_UNRELIABLE_FRAGMENT = ENET_PACKET_FLAG_UNRELIABLE_FRAGMENT
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetPacketFlag(ENET_PACKET_FLAG_NO_ALLOCATE); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PACKET_FLAG_NO_ALLOCATE, __pyx_t_1) < 0) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":187
 * PACKET_FLAG_UNSEQUENCED = ENET_PACKET_FLAG_UNSEQUENCED
 * PACKET_FLAG_NO_ALLOCATE = ENET_PACKET_FLAG_NO_ALLOCATE
 * PACKET_FLAG_UNRELIABLE_FRAGMENT = ENET_PACKET_FLAG_UNRELIABLE_FRAGMENT             # <<<<<<<<<<<<<<
 * 
 * EVENT_TYPE_NONE = ENET_EVENT_TYPE_NONE
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetPacketFlag(ENET_PACKET_FLAG_UNRELIABLE_FRAGMENT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PACKET_FLAG_UNRELIABLE_FRAGMENT, __pyx_t_1) < 0) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":189
 * PACKET_FLAG_UNRELIABLE_FRAGMENT = ENET_PACKET_FLAG_UNRELIABLE_FRAGMENT
 * 
 * EVENT_TYPE_NONE = ENET_EVENT_TYPE_NONE             # <<<<<<<<<<<<<<
 * EVENT_TYPE_CONNECT = ENET_EVENT_TYPE_CONNECT
 * EVENT_TYPE_DISCONNECT = ENET_EVENT_TYPE_DISCONNECT
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetEventType(ENET_EVENT_TYPE_NONE); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_EVENT_TYPE_NONE, __pyx_t_1) < 0) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":190
 * 
 * EVENT_TYPE_NONE = ENET_EVENT_TYPE_NONE
 * EVENT_TYPE_CONNECT = ENET_EVENT_TYPE_CONNECT             # <<<<<<<<<<<<<<
 * EVENT_TYPE_DISCONNECT = ENET_EVENT_TYPE_DISCONNECT
 * EVENT_TYPE_RECEIVE = ENET_EVENT_TYPE_RECEIVE
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetEventType(ENET_EVENT_TYPE_CONNECT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_EVENT_TYPE_CONNECT, __pyx_t_1) < 0) __PYX_ERR(0, 190, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":191
 * EVENT_TYPE_NONE = ENET_EVENT_TYPE_NONE
 * EVENT_TYPE_CONNECT = ENET_EVENT_TYPE_CONNECT
 * EVENT_TYPE_DISCONNECT = ENET_EVENT_TYPE_DISCONNECT             # <<<<<<<<<<<<<<
 * EVENT_TYPE_RECEIVE = ENET_EVENT_TYPE_RECEIVE
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetEventType(ENET_EVENT_TYPE_DISCONNECT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_EVENT_TYPE_DISCONNECT, __pyx_t_1) < 0) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":192
 * EVENT_TYPE_CONNECT = ENET_EVENT_TYPE_CONNECT
 * EVENT_TYPE_DISCONNECT = ENET_EVENT_TYPE_DISCONNECT
 * EVENT_TYPE_RECEIVE = ENET_EVENT_TYPE_RECEIVE             # <<<<<<<<<<<<<<
 * 
 * PEER_STATE_DISCONNECTED = ENET_PEER_STATE_DISCONNECTED
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetEventType(ENET_EVENT_TYPE_RECEIVE); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_EVENT_TYPE_RECEIVE, __pyx_t_1) < 0) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":194
 * EVENT_TYPE_RECEIVE = ENET_EVENT_TYPE_RECEIVE
 * 
 * PEER_STATE_DISCONNECTED = ENET_PEER_STATE_DISCONNECTED             # <<<<<<<<<<<<<<
 * PEER_STATE_CONNECTING = ENET_PEER_STATE_CONNECTING
 * PEER_STATE_ACKNOWLEDGING_CONNECT = ENET_PEER_STATE_ACKNOWLEDGING_CONNECT
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetPeerState(ENET_PEER_STATE_DISCONNECTED); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PEER_STATE_DISCONNECTED, __pyx_t_1) < 0) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":195
 * 
 * PEER_STATE_DISCONNECTED = ENET_PEER_STATE_DISCONNECTED
 * PEER_STATE_CONNECTING = ENET_PEER_STATE_CONNECTING             # <<<<<<<<<<<<<<
 * PEER_STATE_ACKNOWLEDGING_CONNECT = ENET_PEER_STATE_ACKNOWLEDGING_CONNECT
 * PEER_STATE_CONNECTION_PENDING = ENET_PEER_STATE_CONNECTION_PENDING
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetPeerState(ENET_PEER_STATE_CONNECTING); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PEER_STATE_CONNECTING, __pyx_t_1) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":196
 * PEER_STATE_DISCONNECTED = ENET_PEER_STATE_DISCONNECTED
 * PEER_STATE_CONNECTING = ENET_PEER_STATE_CONNECTING
 * PEER_STATE_ACKNOWLEDGING_CONNECT = ENET_PEER_STATE_ACKNOWLEDGING_CONNECT             # <<<<<<<<<<<<<<
 * PEER_STATE_CONNECTION_PENDING = ENET_PEER_STATE_CONNECTION_PENDING
 * PEER_STATE_CONNECTION_SUCCEEDED = ENET_PEER_STATE_CONNECTION_SUCCEEDED
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetPeerState(ENET_PEER_STATE_ACKNOWLEDGING_CONNECT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PEER_STATE_ACKNOWLEDGING_CONNECT, __pyx_t_1) < 0) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":197
 * PEER_STATE_CONNECTING = ENET_PEER_STATE_CONNECTING
 * PEER_STATE_ACKNOWLEDGING_CONNECT = ENET_PEER_STATE_ACKNOWLEDGING_CONNECT
 * PEER_STATE_CONNECTION_PENDING = ENET_PEER_STATE_CONNECTION_PENDING             # <<<<<<<<<<<<<<
 * PEER_STATE_CONNECTION_SUCCEEDED = ENET_PEER_STATE_CONNECTION_SUCCEEDED
 * PEER_STATE_CONNECTED = ENET_PEER_STATE_CONNECTED
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetPeerState(ENET_PEER_STATE_CONNECTION_PENDING); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PEER_STATE_CONNECTION_PENDING, __pyx_t_1) < 0) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":198
 * PEER_STATE_ACKNOWLEDGING_CONNECT = ENET_PEER_STATE_ACKNOWLEDGING_CONNECT
 * PEER_STATE_CONNECTION_PENDING = ENET_PEER_STATE_CONNECTION_PENDING
 * PEER_STATE_CONNECTION_SUCCEEDED = ENET_PEER_STATE_CONNECTION_SUCCEEDED             # <<<<<<<<<<<<<<
 * PEER_STATE_CONNECTED = ENET_PEER_STATE_CONNECTED
 * PEER_STATE_DISCONNECT_LATER = ENET_PEER_STATE_DISCONNECT_LATER
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetPeerState(ENET_PEER_STATE_CONNECTION_SUCCEEDED); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PEER_STATE_CONNECTION_SUCCEEDED, __pyx_t_1) < 0) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":199
 * PEER_STATE_CONNECTION_PENDING = ENET_PEER_STATE_CONNECTION_PENDING
 * PEER_STATE_CONNECTION_SUCCEEDED = ENET_PEER_STATE_CONNECTION_SUCCEEDED
 * PEER_STATE_CONNECTED = ENET_PEER_STATE_CONNECTED             # <<<<<<<<<<<<<<
 * PEER_STATE_DISCONNECT_LATER = ENET_PEER_STATE_DISCONNECT_LATER
 * PEER_STATE_DISCONNECTING = ENET_PEER_STATE_DISCONNECTING
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetPeerState(ENET_PEER_STATE_CONNECTED); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PEER_STATE_CONNECTED, __pyx_t_1) < 0) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":200
 * PEER_STATE_CONNECTION_SUCCEEDED = ENET_PEER_STATE_CONNECTION_SUCCEEDED
 * PEER_STATE_CONNECTED = ENET_PEER_STATE_CONNECTED
 * PEER_STATE_DISCONNECT_LATER = ENET_PEER_STATE_DISCONNECT_LATER             # <<<<<<<<<<<<<<
 * PEER_STATE_DISCONNECTING = ENET_PEER_STATE_DISCONNECTING
 * PEER_STATE_ACKNOWLEDGING_DISCONNECT = ENET_PEER_STATE_ACKNOWLEDGING_DISCONNECT
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetPeerState(ENET_PEER_STATE_DISCONNECT_LATER); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PEER_STATE_DISCONNECT_LATER, __pyx_t_1) < 0) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":201
 * PEER_STATE_CONNECTED = ENET_PEER_STATE_CONNECTED
 * PEER_STATE_DISCONNECT_LATER = ENET_PEER_STATE_DISCONNECT_LATER
 * PEER_STATE_DISCONNECTING = ENET_PEER_STATE_DISCONNECTING             # <<<<<<<<<<<<<<
 * PEER_STATE_ACKNOWLEDGING_DISCONNECT = ENET_PEER_STATE_ACKNOWLEDGING_DISCONNECT
 * PEER_STATE_ZOMBIE = ENET_PEER_STATE_ZOMBIE
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetPeerState(ENET_PEER_STATE_DISCONNECTING); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PEER_STATE_DISCONNECTING, __pyx_t_1) < 0) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":202
 * PEER_STATE_DISCONNECT_LATER = ENET_PEER_STATE_DISCONNECT_LATER
 * PEER_STATE_DISCONNECTING = ENET_PEER_STATE_DISCONNECTING
 * PEER_STATE_ACKNOWLEDGING_DISCONNECT = ENET_PEER_STATE_ACKNOWLEDGING_DISCONNECT             # <<<<<<<<<<<<<<
 * PEER_STATE_ZOMBIE = ENET_PEER_STATE_ZOMBIE
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetPeerState(ENET_PEER_STATE_ACKNOWLEDGING_DISCONNECT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PEER_STATE_ACKNOWLEDGING_DISCONN, __pyx_t_1) < 0) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":203
 * PEER_STATE_DISCONNECTING = ENET_PEER_STATE_DISCONNECTING
 * PEER_STATE_ACKNOWLEDGING_DISCONNECT = ENET_PEER_STATE_ACKNOWLEDGING_DISCONNECT
 * PEER_STATE_ZOMBIE = ENET_PEER_STATE_ZOMBIE             # <<<<<<<<<<<<<<
 * 
 * cdef class Address
 */
  __pyx_t_1 = __Pyx_PyInt_From_ENetPeerState(ENET_PEER_STATE_ZOMBIE); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PEER_STATE_ZOMBIE, __pyx_t_1) < 0) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":844
 *             return self._packet
 * 
 * from weakref import WeakValueDictionary             # <<<<<<<<<<<<<<
 * cdef host_static_instances = WeakValueDictionary()
 * 
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 844, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_WeakValueDictionary);
  __Pyx_GIVEREF(__pyx_n_s_WeakValueDictionary);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_WeakValueDictionary);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_weakref, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 844, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_WeakValueDictionary); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 844, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WeakValueDictionary, __pyx_t_1) < 0) __PYX_ERR(0, 844, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "enet.pyx":845
 * 
 * from weakref import WeakValueDictionary
 * cdef host_static_instances = WeakValueDictionary()             # <<<<<<<<<<<<<<
 * 
 * cdef class Host:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_WeakValueDictionary); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 845, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 845, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_6bzenet_host_static_instances);
  __Pyx_DECREF_SET(__pyx_v_6bzenet_host_static_instances, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "enet.pyx":1082
 *     return int(bool(ret))
 * 
 * def _enet_atexit():             # <<<<<<<<<<<<<<
 *     enet_deinitialize()
 * 
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_6bzenet_1_enet_atexit, NULL, __pyx_n_s_bzenet); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1082, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_enet_atexit, __pyx_t_1) < 0) __PYX_ERR(0, 1082, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "enet.pyx":1085
 *     enet_deinitialize()
 * 
 * enet_initialize()             # <<<<<<<<<<<<<<
 * atexit.register(_enet_atexit)
 */
  (void)(enet_initialize());

  /* "enet.pyx":1086
 * 
 * enet_initialize()
 * atexit.register(_enet_atexit)             # <<<<<<<<<<<<<<
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_atexit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1086, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_register); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1086, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enet_atexit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1086, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1086, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Socket(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_6bzenet_3__pyx_unpickle_Socket, NULL, __pyx_n_s_bzenet); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_Socket, __pyx_t_3) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "enet.pyx":1
 * import atexit             # <<<<<<<<<<<<<<
 * 
 * from cpython cimport bool
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_3) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init bzenet", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init bzenet");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* PyIntCompare */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* KeywordStringCheck */
static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* SetItemInt */
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o)))
#else
    if (is_list || PySequence_Check(o))
#endif
    {
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_ENetSocket(ENetSocket value) {
    const ENetSocket neg_one = (ENetSocket) ((ENetSocket) 0 - (ENetSocket) 1), const_zero = (ENetSocket) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(ENetSocket) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(ENetSocket) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(ENetSocket) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(ENetSocket) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(ENetSocket) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(ENetSocket),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_ENetPacketFlag(ENetPacketFlag value) {
    const ENetPacketFlag neg_one = (ENetPacketFlag) ((ENetPacketFlag) 0 - (ENetPacketFlag) 1), const_zero = (ENetPacketFlag) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(ENetPacketFlag) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(ENetPacketFlag) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(ENetPacketFlag) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(ENetPacketFlag) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(ENetPacketFlag) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(ENetPacketFlag),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_ENetEventType(ENetEventType value) {
    const ENetEventType neg_one = (ENetEventType) ((ENetEventType) 0 - (ENetEventType) 1), const_zero = (ENetEventType) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(ENetEventType) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(ENetEventType) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(ENetEventType) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(ENetEventType) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(ENetEventType) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(ENetEventType),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_ENetPeerState(ENetPeerState value) {
    const ENetPeerState neg_one = (ENetPeerState) ((ENetPeerState) 0 - (ENetPeerState) 1), const_zero = (ENetPeerState) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(ENetPeerState) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(ENetPeerState) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(ENetPeerState) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(ENetPeerState) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(ENetPeerState) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(ENetPeerState),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enet_uint16(enet_uint16 value) {
    const enet_uint16 neg_one = (enet_uint16) ((enet_uint16) 0 - (enet_uint16) 1), const_zero = (enet_uint16) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enet_uint16) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enet_uint16) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enet_uint16) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enet_uint16) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enet_uint16) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enet_uint16),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enet_uint32(enet_uint32 value) {
    const enet_uint32 neg_one = (enet_uint32) ((enet_uint32) 0 - (enet_uint32) 1), const_zero = (enet_uint32) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enet_uint32) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enet_uint32) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enet_uint32) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enet_uint32) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enet_uint32) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enet_uint32),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enet_uint8(enet_uint8 value) {
    const enet_uint8 neg_one = (enet_uint8) ((enet_uint8) 0 - (enet_uint8) 1), const_zero = (enet_uint8) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enet_uint8) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enet_uint8) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enet_uint8) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enet_uint8) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enet_uint8) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enet_uint8),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE ENetSocket __Pyx_PyInt_As_ENetSocket(PyObject *x) {
    const ENetSocket neg_one = (ENetSocket) ((ENetSocket) 0 - (ENetSocket) 1), const_zero = (ENetSocket) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(ENetSocket) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(ENetSocket, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (ENetSocket) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (ENetSocket) 0;
                case  1: __PYX_VERIFY_RETURN_INT(ENetSocket, digit, digits[0])
                case 2:
                    if (8 * sizeof(ENetSocket) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ENetSocket, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ENetSocket) >= 2 * PyLong_SHIFT) {
                            return (ENetSocket) (((((ENetSocket)digits[1]) << PyLong_SHIFT) | (ENetSocket)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(ENetSocket) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ENetSocket, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ENetSocket) >= 3 * PyLong_SHIFT) {
                            return (ENetSocket) (((((((ENetSocket)digits[2]) << PyLong_SHIFT) | (ENetSocket)digits[1]) << PyLong_SHIFT) | (ENetSocket)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(ENetSocket) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ENetSocket, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ENetSocket) >= 4 * PyLong_SHIFT) {
                            return (ENetSocket) (((((((((ENetSocket)digits[3]) << PyLong_SHIFT) | (ENetSocket)digits[2]) << PyLong_SHIFT) | (ENetSocket)digits[1]) << PyLong_SHIFT) | (ENetSocket)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (ENetSocket) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(ENetSocket) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(ENetSocket, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(ENetSocket) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(ENetSocket, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (ENetSocket) 0;
                case -1: __PYX_VERIFY_RETURN_INT(ENetSocket, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(ENetSocket,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(ENetSocket) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ENetSocket, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ENetSocket) - 1 > 2 * PyLong_SHIFT) {
                            return (ENetSocket) (((ENetSocket)-1)*(((((ENetSocket)digits[1]) << PyLong_SHIFT) | (ENetSocket)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(ENetSocket) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ENetSocket, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ENetSocket) - 1 > 2 * PyLong_SHIFT) {
                            return (ENetSocket) ((((((ENetSocket)digits[1]) << PyLong_SHIFT) | (ENetSocket)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(ENetSocket) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ENetSocket, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ENetSocket) - 1 > 3 * PyLong_SHIFT) {
                            return (ENetSocket) (((ENetSocket)-1)*(((((((ENetSocket)digits[2]) << PyLong_SHIFT) | (ENetSocket)digits[1]) << PyLong_SHIFT) | (ENetSocket)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(ENetSocket) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ENetSocket, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ENetSocket) - 1 > 3 * PyLong_SHIFT) {
                            return (ENetSocket) ((((((((ENetSocket)digits[2]) << PyLong_SHIFT) | (ENetSocket)digits[1]) << PyLong_SHIFT) | (ENetSocket)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(ENetSocket) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ENetSocket, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ENetSocket) - 1 > 4 * PyLong_SHIFT) {
                            return (ENetSocket) (((ENetSocket)-1)*(((((((((ENetSocket)digits[3]) << PyLong_SHIFT) | (ENetSocket)digits[2]) << PyLong_SHIFT) | (ENetSocket)digits[1]) << PyLong_SHIFT) | (ENetSocket)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(ENetSocket) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ENetSocket, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ENetSocket) - 1 > 4 * PyLong_SHIFT) {
                            return (ENetSocket) ((((((((((ENetSocket)digits[3]) << PyLong_SHIFT) | (ENetSocket)digits[2]) << PyLong_SHIFT) | (ENetSocket)digits[1]) << PyLong_SHIFT) | (ENetSocket)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(ENetSocket) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(ENetSocket, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(ENetSocket) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(ENetSocket, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            ENetSocket val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (ENetSocket) -1;
        }
    } else {
        ENetSocket val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (ENetSocket) -1;
        val = __Pyx_PyInt_As_ENetSocket(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to ENetSocket");
    return (ENetSocket) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to ENetSocket");
    return (ENetSocket) -1;
}

/* CIntFromPy */
static CYTHON_INLINE unsigned char __Pyx_PyInt_As_unsigned_char(PyObject *x) {
    const unsigned char neg_one = (unsigned char) ((unsigned char) 0 - (unsigned char) 1), const_zero = (unsigned char) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned char) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned char, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned char) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned char) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned char, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) >= 2 * PyLong_SHIFT) {
                            return (unsigned char) (((((unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) >= 3 * PyLong_SHIFT) {
                            return (unsigned char) (((((((unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) >= 4 * PyLong_SHIFT) {
                            return (unsigned char) (((((((((unsigned char)digits[3]) << PyLong_SHIFT) | (unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned char) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned char) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned char, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned char) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned char, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned char) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned char, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned char,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned char) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned char) (((unsigned char)-1)*(((((unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned char) ((((((unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned char) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned char) (((unsigned char)-1)*(((((((unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned char) ((((((((unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned char) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned char) (((unsigned char)-1)*(((((((((unsigned char)digits[3]) << PyLong_SHIFT) | (unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned char) ((((((((((unsigned char)digits[3]) << PyLong_SHIFT) | (unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned char) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned char, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned char) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned char, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned char val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned char) -1;
        }
    } else {
        unsigned char val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned char) -1;
        val = __Pyx_PyInt_As_unsigned_char(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned char");
    return (unsigned char) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned char");
    return (unsigned char) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE enet_uint16 __Pyx_PyInt_As_enet_uint16(PyObject *x) {
    const enet_uint16 neg_one = (enet_uint16) ((enet_uint16) 0 - (enet_uint16) 1), const_zero = (enet_uint16) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(enet_uint16) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(enet_uint16, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (enet_uint16) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (enet_uint16) 0;
                case  1: __PYX_VERIFY_RETURN_INT(enet_uint16, digit, digits[0])
                case 2:
                    if (8 * sizeof(enet_uint16) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint16, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint16) >= 2 * PyLong_SHIFT) {
                            return (enet_uint16) (((((enet_uint16)digits[1]) << PyLong_SHIFT) | (enet_uint16)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(enet_uint16) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint16, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint16) >= 3 * PyLong_SHIFT) {
                            return (enet_uint16) (((((((enet_uint16)digits[2]) << PyLong_SHIFT) | (enet_uint16)digits[1]) << PyLong_SHIFT) | (enet_uint16)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(enet_uint16) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint16, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint16) >= 4 * PyLong_SHIFT) {
                            return (enet_uint16) (((((((((enet_uint16)digits[3]) << PyLong_SHIFT) | (enet_uint16)digits[2]) << PyLong_SHIFT) | (enet_uint16)digits[1]) << PyLong_SHIFT) | (enet_uint16)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (enet_uint16) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(enet_uint16) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(enet_uint16, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(enet_uint16) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(enet_uint16, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (enet_uint16) 0;
                case -1: __PYX_VERIFY_RETURN_INT(enet_uint16, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(enet_uint16,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(enet_uint16) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint16, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint16) - 1 > 2 * PyLong_SHIFT) {
                            return (enet_uint16) (((enet_uint16)-1)*(((((enet_uint16)digits[1]) << PyLong_SHIFT) | (enet_uint16)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(enet_uint16) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint16, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint16) - 1 > 2 * PyLong_SHIFT) {
                            return (enet_uint16) ((((((enet_uint16)digits[1]) << PyLong_SHIFT) | (enet_uint16)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(enet_uint16) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint16, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint16) - 1 > 3 * PyLong_SHIFT) {
                            return (enet_uint16) (((enet_uint16)-1)*(((((((enet_uint16)digits[2]) << PyLong_SHIFT) | (enet_uint16)digits[1]) << PyLong_SHIFT) | (enet_uint16)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(enet_uint16) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint16, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint16) - 1 > 3 * PyLong_SHIFT) {
                            return (enet_uint16) ((((((((enet_uint16)digits[2]) << PyLong_SHIFT) | (enet_uint16)digits[1]) << PyLong_SHIFT) | (enet_uint16)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(enet_uint16) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint16, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint16) - 1 > 4 * PyLong_SHIFT) {
                            return (enet_uint16) (((enet_uint16)-1)*(((((((((enet_uint16)digits[3]) << PyLong_SHIFT) | (enet_uint16)digits[2]) << PyLong_SHIFT) | (enet_uint16)digits[1]) << PyLong_SHIFT) | (enet_uint16)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(enet_uint16) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint16, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint16) - 1 > 4 * PyLong_SHIFT) {
                            return (enet_uint16) ((((((((((enet_uint16)digits[3]) << PyLong_SHIFT) | (enet_uint16)digits[2]) << PyLong_SHIFT) | (enet_uint16)digits[1]) << PyLong_SHIFT) | (enet_uint16)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(enet_uint16) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(enet_uint16, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(enet_uint16) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(enet_uint16, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            enet_uint16 val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (enet_uint16) -1;
        }
    } else {
        enet_uint16 val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (enet_uint16) -1;
        val = __Pyx_PyInt_As_enet_uint16(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to enet_uint16");
    return (enet_uint16) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to enet_uint16");
    return (enet_uint16) -1;
}

/* CIntFromPy */
static CYTHON_INLINE enet_uint32 __Pyx_PyInt_As_enet_uint32(PyObject *x) {
    const enet_uint32 neg_one = (enet_uint32) ((enet_uint32) 0 - (enet_uint32) 1), const_zero = (enet_uint32) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(enet_uint32) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(enet_uint32, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (enet_uint32) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (enet_uint32) 0;
                case  1: __PYX_VERIFY_RETURN_INT(enet_uint32, digit, digits[0])
                case 2:
                    if (8 * sizeof(enet_uint32) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint32, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint32) >= 2 * PyLong_SHIFT) {
                            return (enet_uint32) (((((enet_uint32)digits[1]) << PyLong_SHIFT) | (enet_uint32)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(enet_uint32) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint32, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint32) >= 3 * PyLong_SHIFT) {
                            return (enet_uint32) (((((((enet_uint32)digits[2]) << PyLong_SHIFT) | (enet_uint32)digits[1]) << PyLong_SHIFT) | (enet_uint32)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(enet_uint32) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint32, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint32) >= 4 * PyLong_SHIFT) {
                            return (enet_uint32) (((((((((enet_uint32)digits[3]) << PyLong_SHIFT) | (enet_uint32)digits[2]) << PyLong_SHIFT) | (enet_uint32)digits[1]) << PyLong_SHIFT) | (enet_uint32)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (enet_uint32) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(enet_uint32) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(enet_uint32, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(enet_uint32) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(enet_uint32, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (enet_uint32) 0;
                case -1: __PYX_VERIFY_RETURN_INT(enet_uint32, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(enet_uint32,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(enet_uint32) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint32, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint32) - 1 > 2 * PyLong_SHIFT) {
                            return (enet_uint32) (((enet_uint32)-1)*(((((enet_uint32)digits[1]) << PyLong_SHIFT) | (enet_uint32)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(enet_uint32) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint32, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint32) - 1 > 2 * PyLong_SHIFT) {
                            return (enet_uint32) ((((((enet_uint32)digits[1]) << PyLong_SHIFT) | (enet_uint32)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(enet_uint32) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint32, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint32) - 1 > 3 * PyLong_SHIFT) {
                            return (enet_uint32) (((enet_uint32)-1)*(((((((enet_uint32)digits[2]) << PyLong_SHIFT) | (enet_uint32)digits[1]) << PyLong_SHIFT) | (enet_uint32)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(enet_uint32) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint32, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint32) - 1 > 3 * PyLong_SHIFT) {
                            return (enet_uint32) ((((((((enet_uint32)digits[2]) << PyLong_SHIFT) | (enet_uint32)digits[1]) << PyLong_SHIFT) | (enet_uint32)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(enet_uint32) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint32, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint32) - 1 > 4 * PyLong_SHIFT) {
                            return (enet_uint32) (((enet_uint32)-1)*(((((((((enet_uint32)digits[3]) << PyLong_SHIFT) | (enet_uint32)digits[2]) << PyLong_SHIFT) | (enet_uint32)digits[1]) << PyLong_SHIFT) | (enet_uint32)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(enet_uint32) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint32, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint32) - 1 > 4 * PyLong_SHIFT) {
                            return (enet_uint32) ((((((((((enet_uint32)digits[3]) << PyLong_SHIFT) | (enet_uint32)digits[2]) << PyLong_SHIFT) | (enet_uint32)digits[1]) << PyLong_SHIFT) | (enet_uint32)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(enet_uint32) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(enet_uint32, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(enet_uint32) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(enet_uint32, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            enet_uint32 val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (enet_uint32) -1;
        }
    } else {
        enet_uint32 val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (enet_uint32) -1;
        val = __Pyx_PyInt_As_enet_uint32(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to enet_uint32");
    return (enet_uint32) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to enet_uint32");
    return (enet_uint32) -1;
}

/* CIntFromPy */
static CYTHON_INLINE enet_uint8 __Pyx_PyInt_As_enet_uint8(PyObject *x) {
    const enet_uint8 neg_one = (enet_uint8) ((enet_uint8) 0 - (enet_uint8) 1), const_zero = (enet_uint8) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(enet_uint8) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(enet_uint8, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (enet_uint8) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (enet_uint8) 0;
                case  1: __PYX_VERIFY_RETURN_INT(enet_uint8, digit, digits[0])
                case 2:
                    if (8 * sizeof(enet_uint8) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint8, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint8) >= 2 * PyLong_SHIFT) {
                            return (enet_uint8) (((((enet_uint8)digits[1]) << PyLong_SHIFT) | (enet_uint8)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(enet_uint8) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint8, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint8) >= 3 * PyLong_SHIFT) {
                            return (enet_uint8) (((((((enet_uint8)digits[2]) << PyLong_SHIFT) | (enet_uint8)digits[1]) << PyLong_SHIFT) | (enet_uint8)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(enet_uint8) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint8, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint8) >= 4 * PyLong_SHIFT) {
                            return (enet_uint8) (((((((((enet_uint8)digits[3]) << PyLong_SHIFT) | (enet_uint8)digits[2]) << PyLong_SHIFT) | (enet_uint8)digits[1]) << PyLong_SHIFT) | (enet_uint8)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (enet_uint8) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(enet_uint8) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(enet_uint8, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(enet_uint8) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(enet_uint8, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (enet_uint8) 0;
                case -1: __PYX_VERIFY_RETURN_INT(enet_uint8, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(enet_uint8,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(enet_uint8) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint8, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint8) - 1 > 2 * PyLong_SHIFT) {
                            return (enet_uint8) (((enet_uint8)-1)*(((((enet_uint8)digits[1]) << PyLong_SHIFT) | (enet_uint8)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(enet_uint8) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint8, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint8) - 1 > 2 * PyLong_SHIFT) {
                            return (enet_uint8) ((((((enet_uint8)digits[1]) << PyLong_SHIFT) | (enet_uint8)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(enet_uint8) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint8, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint8) - 1 > 3 * PyLong_SHIFT) {
                            return (enet_uint8) (((enet_uint8)-1)*(((((((enet_uint8)digits[2]) << PyLong_SHIFT) | (enet_uint8)digits[1]) << PyLong_SHIFT) | (enet_uint8)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(enet_uint8) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint8, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint8) - 1 > 3 * PyLong_SHIFT) {
                            return (enet_uint8) ((((((((enet_uint8)digits[2]) << PyLong_SHIFT) | (enet_uint8)digits[1]) << PyLong_SHIFT) | (enet_uint8)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(enet_uint8) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint8, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint8) - 1 > 4 * PyLong_SHIFT) {
                            return (enet_uint8) (((enet_uint8)-1)*(((((((((enet_uint8)digits[3]) << PyLong_SHIFT) | (enet_uint8)digits[2]) << PyLong_SHIFT) | (enet_uint8)digits[1]) << PyLong_SHIFT) | (enet_uint8)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(enet_uint8) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enet_uint8, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enet_uint8) - 1 > 4 * PyLong_SHIFT) {
                            return (enet_uint8) ((((((((((enet_uint8)digits[3]) << PyLong_SHIFT) | (enet_uint8)digits[2]) << PyLong_SHIFT) | (enet_uint8)digits[1]) << PyLong_SHIFT) | (enet_uint8)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(enet_uint8) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(enet_uint8, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(enet_uint8) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(enet_uint8, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            enet_uint8 val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (enet_uint8) -1;
        }
    } else {
        enet_uint8 val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (enet_uint8) -1;
        val = __Pyx_PyInt_As_enet_uint8(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to enet_uint8");
    return (enet_uint8) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to enet_uint8");
    return (enet_uint8) -1;
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
    const size_t neg_one = (size_t) ((size_t) 0 - (size_t) 1), const_zero = (size_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
